@@ -75,10 +75,6 @@ tar -xvf $XORGMACROS_VERSION.tar.bz2 && rm $XORGMACROS_VERSION.tar.bz2
 cd $XORGMACROS_VERSION; ./configure; make install; cd ..
 rm -rf $XORGMACROS_VERSION
 
-. .gitlab-ci/container/build-llvm-spirv.sh
-
-. .gitlab-ci/container/build-libclc.sh
-
 . .gitlab-ci/container/build-wayland.sh
 
 . .gitlab-ci/container/build-shader-db.sh
@@ -89,7 +85,14 @@ python3 -m pip install --break-system-packages -r .gitlab-ci/lava/requirements.t
 
 # install bindgen
 RUSTFLAGS='-L native=/usr/local/lib' cargo install \
-  bindgen-cli --version 0.62.0 \
+  bindgen-cli --version 0.65.1 \
+  --locked \
+  -j ${FDO_CI_CONCURRENT:-4} \
+  --root /usr/local
+
+# install cbindgen
+RUSTFLAGS='-L native=/usr/local/lib' cargo install \
+  cbindgen --version 0.26.0 \
   --locked \
   -j ${FDO_CI_CONCURRENT:-4} \
   --root /usr/local
