@@ -26,7 +26,7 @@
 #include "nir_builder.h"
 
 static bool
-assert_ssa_def_is_not_int(nir_ssa_def *def, void *arg)
+assert_ssa_def_is_not_int(nir_def *def, void *arg)
 {
    ASSERTED BITSET_WORD *int_types = arg;
    assert(!BITSET_TEST(int_types, def->index));
@@ -39,7 +39,7 @@ instr_has_only_trivial_swizzles(nir_alu_instr *alu)
    const nir_op_info *info = &nir_op_infos[alu->op];
 
    for (unsigned i = 0; i < info->num_inputs; i++) {
-      for (unsigned chan = 0; chan < alu->dest.dest.ssa.num_components; chan++) {
+      for (unsigned chan = 0; chan < alu->def.num_components; chan++) {
          if (alu->src[i].swizzle[chan] != chan)
             return false;
       }
@@ -83,7 +83,7 @@ lower_alu_instr(nir_builder *b, nir_alu_instr *alu)
 {
    const nir_op_info *info = &nir_op_infos[alu->op];
 
-   bool is_bool_only = alu->dest.dest.ssa.bit_size == 1;
+   bool is_bool_only = alu->def.bit_size == 1;
    for (unsigned i = 0; i < info->num_inputs; i++) {
       if (alu->src[i].src.ssa->bit_size != 1)
          is_bool_only = false;
@@ -97,7 +97,7 @@ lower_alu_instr(nir_builder *b, nir_alu_instr *alu)
    b->cursor = nir_before_instr(&alu->instr);
 
    /* Replacement SSA value */
-   nir_ssa_def *rep = NULL;
+   nir_def *rep = NULL;
    switch (alu->op) {
    case nir_op_mov:
    case nir_op_vec2:
@@ -107,9 +107,15 @@ lower_alu_instr(nir_builder *b, nir_alu_instr *alu)
       /* These we expect to have integers but the opcode doesn't change */
       break;
 
-   case nir_op_b2i32: alu->op = nir_op_b2f32; break;
-   case nir_op_i2f32: alu->op = nir_op_mov; break;
-   case nir_op_u2f32: alu->op = nir_op_mov; break;
+   case nir_op_b2i32:
+      alu->op = nir_op_b2f32;
+      break;
+   case nir_op_i2f32:
+      alu->op = nir_op_mov;
+      break;
+   case nir_op_u2f32:
+      alu->op = nir_op_mov;
+      break;
 
    case nir_op_f2i32: {
       alu->op = nir_op_ftrunc;
@@ -138,22 +144,42 @@ lower_alu_instr(nir_builder *b, nir_alu_instr *alu)
       break;
    }
 
-   case nir_op_f2u32: alu->op = nir_op_ffloor; break;
+   case nir_op_f2u32:
+      alu->op = nir_op_ffloor;
+      break;
 
-   case nir_op_ilt: alu->op = nir_op_flt; break;
-   case nir_op_ige: alu->op = nir_op_fge; break;
-   case nir_op_ieq: alu->op = nir_op_feq; break;
-   case nir_op_ine: alu->op = nir_op_fneu; break;
-   case nir_op_ult: alu->op = nir_op_flt; break;
-   case nir_op_uge: alu->op = nir_op_fge; break;
+   case nir_op_ilt:
+      alu->op = nir_op_flt;
+      break;
+   case nir_op_ige:
+      alu->op = nir_op_fge;
+      break;
+   case nir_op_ieq:
+      alu->op = nir_op_feq;
+      break;
+   case nir_op_ine:
+      alu->op = nir_op_fneu;
+      break;
+   case nir_op_ult:
+      alu->op = nir_op_flt;
+      break;
+   case nir_op_uge:
+      alu->op = nir_op_fge;
+      break;
 
-   case nir_op_iadd: alu->op = nir_op_fadd; break;
-   case nir_op_isub: alu->op = nir_op_fsub; break;
-   case nir_op_imul: alu->op = nir_op_fmul; break;
+   case nir_op_iadd:
+      alu->op = nir_op_fadd;
+      break;
+   case nir_op_isub:
+      alu->op = nir_op_fsub;
+      break;
+   case nir_op_imul:
+      alu->op = nir_op_fmul;
+      break;
 
    case nir_op_idiv: {
-      nir_ssa_def *x = nir_ssa_for_alu_src(b, alu, 0);
-      nir_ssa_def *y = nir_ssa_for_alu_src(b, alu, 1);
+      nir_def *x = nir_ssa_for_alu_src(b, alu, 0);
+      nir_def *y = nir_ssa_for_alu_src(b, alu, 1);
 
       /* Hand-lower fdiv, since lower_int_to_float is after nir_opt_algebraic. */
       if (b->shader->options->lower_fdiv) {
@@ -164,22 +190,50 @@ lower_alu_instr(nir_builder *b, nir_alu_instr *alu)
       break;
    }
 
-   case nir_op_iabs: alu->op = nir_op_fabs; break;
-   case nir_op_ineg: alu->op = nir_op_fneg; break;
-   case nir_op_imax: alu->op = nir_op_fmax; break;
-   case nir_op_imin: alu->op = nir_op_fmin; break;
-   case nir_op_umax: alu->op = nir_op_fmax; break;
-   case nir_op_umin: alu->op = nir_op_fmin; break;
+   case nir_op_iabs:
+      alu->op = nir_op_fabs;
+      break;
+   case nir_op_ineg:
+      alu->op = nir_op_fneg;
+      break;
+   case nir_op_imax:
+      alu->op = nir_op_fmax;
+      break;
+   case nir_op_imin:
+      alu->op = nir_op_fmin;
+      break;
+   case nir_op_umax:
+      alu->op = nir_op_fmax;
+      break;
+   case nir_op_umin:
+      alu->op = nir_op_fmin;
+      break;
 
-   case nir_op_ball_iequal2:  alu->op = nir_op_ball_fequal2; break;
-   case nir_op_ball_iequal3:  alu->op = nir_op_ball_fequal3; break;
-   case nir_op_ball_iequal4:  alu->op = nir_op_ball_fequal4; break;
-   case nir_op_bany_inequal2: alu->op = nir_op_bany_fnequal2; break;
-   case nir_op_bany_inequal3: alu->op = nir_op_bany_fnequal3; break;
-   case nir_op_bany_inequal4: alu->op = nir_op_bany_fnequal4; break;
+   case nir_op_ball_iequal2:
+      alu->op = nir_op_ball_fequal2;
+      break;
+   case nir_op_ball_iequal3:
+      alu->op = nir_op_ball_fequal3;
+      break;
+   case nir_op_ball_iequal4:
+      alu->op = nir_op_ball_fequal4;
+      break;
+   case nir_op_bany_inequal2:
+      alu->op = nir_op_bany_fnequal2;
+      break;
+   case nir_op_bany_inequal3:
+      alu->op = nir_op_bany_fnequal3;
+      break;
+   case nir_op_bany_inequal4:
+      alu->op = nir_op_bany_fnequal4;
+      break;
 
-   case nir_op_i32csel_gt: alu->op = nir_op_fcsel_gt; break;
-   case nir_op_i32csel_ge: alu->op = nir_op_fcsel_ge; break;
+   case nir_op_i32csel_gt:
+      alu->op = nir_op_fcsel_gt;
+      break;
+   case nir_op_i32csel_ge:
+      alu->op = nir_op_fcsel_ge;
+      break;
 
    default:
       assert(nir_alu_type_get_base_type(info->output_type) != nir_type_int &&
@@ -193,7 +247,7 @@ lower_alu_instr(nir_builder *b, nir_alu_instr *alu)
 
    if (rep) {
       /* We've emitted a replacement instruction */
-      nir_ssa_def_rewrite_uses(&alu->dest.dest.ssa, rep);
+      nir_def_rewrite_uses(&alu->def, rep);
       nir_instr_remove(&alu->instr);
    }
 
@@ -213,7 +267,7 @@ nir_lower_int_to_float_impl(nir_function_impl *impl)
                         sizeof(BITSET_WORD));
    int_types = calloc(BITSET_WORDS(impl->ssa_alloc),
                       sizeof(BITSET_WORD));
-   nir_gather_ssa_types(impl, float_types, int_types);
+   nir_gather_types(impl, float_types, int_types);
 
    nir_foreach_block(block, impl) {
       nir_foreach_instr_safe(instr, block) {
@@ -232,13 +286,13 @@ nir_lower_int_to_float_impl(nir_function_impl *impl)
          }
 
          case nir_instr_type_intrinsic:
-         case nir_instr_type_ssa_undef:
+         case nir_instr_type_undef:
          case nir_instr_type_phi:
          case nir_instr_type_tex:
             break;
 
          default:
-            nir_foreach_ssa_def(instr, assert_ssa_def_is_not_int, (void *)int_types);
+            nir_foreach_def(instr, assert_ssa_def_is_not_int, (void *)int_types);
             break;
          }
       }
@@ -246,7 +300,7 @@ nir_lower_int_to_float_impl(nir_function_impl *impl)
 
    if (progress) {
       nir_metadata_preserve(impl, nir_metadata_block_index |
-                                  nir_metadata_dominance);
+                                     nir_metadata_dominance);
    } else {
       nir_metadata_preserve(impl, nir_metadata_all);
    }
