@@ -43,407 +43,36 @@
 #include "util/libsync.h"
 #include "util/os_file.h"
 
+#include "main/glconfig.h"
 #include "egl_dri2.h"
+#include "eglglobals.h"
 #include "loader.h"
+#include "loader_dri_helper.h"
 #include "platform_android.h"
 
-#ifdef HAVE_DRM_GRALLOC
-#include "gralloc_drm.h"
-#include <gralloc_drm_handle.h>
-#endif /* HAVE_DRM_GRALLOC */
-
-#define ALIGN(val, align) (((val) + (align)-1) & ~((align)-1))
-
-enum chroma_order {
-   YCbCr,
-   YCrCb,
-};
-
-struct droid_yuv_format {
-   /* Lookup keys */
-   int native;                     /* HAL_PIXEL_FORMAT_ */
-   enum chroma_order chroma_order; /* chroma order is {Cb, Cr} or {Cr, Cb} */
-   int chroma_step; /* Distance in bytes between subsequent chroma pixels. */
-
-   /* Result */
-   int fourcc; /* DRM_FORMAT_ */
-};
-
-/* The following table is used to look up a DRI image FourCC based
- * on native format and information contained in android_ycbcr struct. */
-static const struct droid_yuv_format droid_yuv_formats[] = {
-   /* Native format, YCrCb, Chroma step, DRI image FourCC */
-   {HAL_PIXEL_FORMAT_YCbCr_420_888, YCbCr, 2, DRM_FORMAT_NV12},
-   {HAL_PIXEL_FORMAT_YCbCr_420_888, YCbCr, 1, DRM_FORMAT_YUV420},
-   {HAL_PIXEL_FORMAT_YCbCr_420_888, YCrCb, 1, DRM_FORMAT_YVU420},
-   {HAL_PIXEL_FORMAT_YV12, YCrCb, 1, DRM_FORMAT_YVU420},
-   /* HACK: See droid_create_image_from_prime_fds() and
-    * https://issuetracker.google.com/32077885. */
-   {HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED, YCbCr, 2, DRM_FORMAT_NV12},
-   {HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED, YCbCr, 1, DRM_FORMAT_YUV420},
-   {HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED, YCrCb, 1, DRM_FORMAT_YVU420},
-   {HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED, YCrCb, 1, DRM_FORMAT_AYUV},
-   {HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED, YCrCb, 1, DRM_FORMAT_XYUV8888},
-};
-
-static int
-get_fourcc_yuv(int native, enum chroma_order chroma_order, int chroma_step)
-{
-   for (int i = 0; i < ARRAY_SIZE(droid_yuv_formats); ++i)
-      if (droid_yuv_formats[i].native == native &&
-          droid_yuv_formats[i].chroma_order == chroma_order &&
-          droid_yuv_formats[i].chroma_step == chroma_step)
-         return droid_yuv_formats[i].fourcc;
-
-   return -1;
-}
-
-static bool
-is_yuv(int native)
-{
-   for (int i = 0; i < ARRAY_SIZE(droid_yuv_formats); ++i)
-      if (droid_yuv_formats[i].native == native)
-         return true;
-
-   return false;
-}
-
-static int
-get_format_bpp(int native)
-{
-   int bpp;
-
-   switch (native) {
-   case HAL_PIXEL_FORMAT_RGBA_FP16:
-      bpp = 8;
-      break;
-   case HAL_PIXEL_FORMAT_RGBA_8888:
-   case HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED:
-      /*
-       * HACK: Hardcode this to RGBX_8888 as per cros_gralloc hack.
-       * TODO: Remove this once https://issuetracker.google.com/32077885 is
-       * fixed.
-       */
-   case HAL_PIXEL_FORMAT_RGBX_8888:
-   case HAL_PIXEL_FORMAT_BGRA_8888:
-   case HAL_PIXEL_FORMAT_RGBA_1010102:
-      bpp = 4;
-      break;
-   case HAL_PIXEL_FORMAT_RGB_565:
-      bpp = 2;
-      break;
-   default:
-      bpp = 0;
-      break;
-   }
-
-   return bpp;
-}
-
-/* createImageFromFds requires fourcc format */
-static int
-get_fourcc(int native)
-{
-   switch (native) {
-   case HAL_PIXEL_FORMAT_RGB_565:
-      return DRM_FORMAT_RGB565;
-   case HAL_PIXEL_FORMAT_BGRA_8888:
-      return DRM_FORMAT_ARGB8888;
-   case HAL_PIXEL_FORMAT_RGBA_8888:
-      return DRM_FORMAT_ABGR8888;
-   case HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED:
-      /*
-       * HACK: Hardcode this to RGBX_8888 as per cros_gralloc hack.
-       * TODO: Remove this once https://issuetracker.google.com/32077885 is
-       * fixed.
-       */
-   case HAL_PIXEL_FORMAT_RGBX_8888:
-      return DRM_FORMAT_XBGR8888;
-   case HAL_PIXEL_FORMAT_RGBA_FP16:
-      return DRM_FORMAT_ABGR16161616F;
-   case HAL_PIXEL_FORMAT_RGBA_1010102:
-      return DRM_FORMAT_ABGR2101010;
-   default:
-      _eglLog(_EGL_WARNING, "unsupported native buffer format 0x%x", native);
-   }
-   return -1;
-}
-
-/* returns # of fds, and by reference the actual fds */
-static unsigned
-get_native_buffer_fds(struct ANativeWindowBuffer *buf, int fds[3])
-{
-   native_handle_t *handle = (native_handle_t *)buf->handle;
-
-   if (!handle)
-      return 0;
-
-   /*
-    * Various gralloc implementations exist, but the dma-buf fd tends
-    * to be first. Access it directly to avoid a dependency on specific
-    * gralloc versions.
-    */
-   for (int i = 0; i < handle->numFds; i++)
-      fds[i] = handle->data[i];
-
-   return handle->numFds;
-}
-
-#ifdef HAVE_DRM_GRALLOC
-static int
-get_native_buffer_name(struct ANativeWindowBuffer *buf)
-{
-   return gralloc_drm_get_gem_handle(buf->handle);
-}
-#endif /* HAVE_DRM_GRALLOC */
-
-static int
-get_yuv_buffer_info(struct dri2_egl_display *dri2_dpy,
-                    struct ANativeWindowBuffer *buf,
-                    struct buffer_info *out_buf_info)
-{
-   struct android_ycbcr ycbcr;
-   enum chroma_order chroma_order;
-   int drm_fourcc = 0;
-   int num_fds = 0;
-   int fds[3];
-   int ret;
-
-   num_fds = get_native_buffer_fds(buf, fds);
-   if (num_fds == 0)
-      return -EINVAL;
-
-   if (!dri2_dpy->gralloc->lock_ycbcr) {
-      _eglLog(_EGL_WARNING, "Gralloc does not support lock_ycbcr");
-      return -EINVAL;
-   }
-
-   memset(&ycbcr, 0, sizeof(ycbcr));
-   ret = dri2_dpy->gralloc->lock_ycbcr(dri2_dpy->gralloc, buf->handle, 0, 0, 0,
-                                       0, 0, &ycbcr);
-   if (ret) {
-      /* HACK: See native_window_buffer_get_buffer_info() and
-       * https://issuetracker.google.com/32077885.*/
-      if (buf->format == HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED)
-         return -EAGAIN;
-
-      _eglLog(_EGL_WARNING, "gralloc->lock_ycbcr failed: %d", ret);
-      return -EINVAL;
-   }
-   dri2_dpy->gralloc->unlock(dri2_dpy->gralloc, buf->handle);
-
-   chroma_order = ((size_t)ycbcr.cr < (size_t)ycbcr.cb) ? YCrCb : YCbCr;
-
-   /* .chroma_step is the byte distance between the same chroma channel
-    * values of subsequent pixels, assumed to be the same for Cb and Cr. */
-   drm_fourcc = get_fourcc_yuv(buf->format, chroma_order, ycbcr.chroma_step);
-   if (drm_fourcc == -1) {
-      _eglLog(_EGL_WARNING,
-              "unsupported YUV format, native = %x, chroma_order = %s, "
-              "chroma_step = %d",
-              buf->format, chroma_order == YCbCr ? "YCbCr" : "YCrCb",
-              ycbcr.chroma_step);
-      return -EINVAL;
-   }
-
-   *out_buf_info = (struct buffer_info){
-      .width = buf->width,
-      .height = buf->height,
-      .drm_fourcc = drm_fourcc,
-      .num_planes = ycbcr.chroma_step == 2 ? 2 : 3,
-      .fds = {-1, -1, -1, -1},
-      .modifier = DRM_FORMAT_MOD_INVALID,
-      .yuv_color_space = EGL_ITU_REC601_EXT,
-      .sample_range = EGL_YUV_NARROW_RANGE_EXT,
-      .horizontal_siting = EGL_YUV_CHROMA_SITING_0_EXT,
-      .vertical_siting = EGL_YUV_CHROMA_SITING_0_EXT,
-   };
-
-   /* When lock_ycbcr's usage argument contains no SW_READ/WRITE flags
-    * it will return the .y/.cb/.cr pointers based on a NULL pointer,
-    * so they can be interpreted as offsets. */
-   out_buf_info->offsets[0] = (size_t)ycbcr.y;
-   /* We assume here that all the planes are located in one DMA-buf. */
-   if (chroma_order == YCrCb) {
-      out_buf_info->offsets[1] = (size_t)ycbcr.cr;
-      out_buf_info->offsets[2] = (size_t)ycbcr.cb;
-   } else {
-      out_buf_info->offsets[1] = (size_t)ycbcr.cb;
-      out_buf_info->offsets[2] = (size_t)ycbcr.cr;
-   }
-
-   /* .ystride is the line length (in bytes) of the Y plane,
-    * .cstride is the line length (in bytes) of any of the remaining
-    * Cb/Cr/CbCr planes, assumed to be the same for Cb and Cr for fully
-    * planar formats. */
-   out_buf_info->pitches[0] = ycbcr.ystride;
-   out_buf_info->pitches[1] = out_buf_info->pitches[2] = ycbcr.cstride;
-
-   /*
-    * Since this is EGL_NATIVE_BUFFER_ANDROID don't assume that
-    * the single-fd case cannot happen.  So handle eithe single
-    * fd or fd-per-plane case:
-    */
-   if (num_fds == 1) {
-      out_buf_info->fds[1] = out_buf_info->fds[0] = fds[0];
-      if (out_buf_info->num_planes == 3)
-         out_buf_info->fds[2] = fds[0];
-   } else {
-      assert(num_fds == out_buf_info->num_planes);
-      out_buf_info->fds[0] = fds[0];
-      out_buf_info->fds[1] = fds[1];
-      out_buf_info->fds[2] = fds[2];
-   }
-
-   return 0;
-}
-
-static int
-native_window_buffer_get_buffer_info(struct dri2_egl_display *dri2_dpy,
-                                     struct ANativeWindowBuffer *buf,
-                                     struct buffer_info *out_buf_info)
-{
-   int num_planes = 0;
-   int drm_fourcc = 0;
-   int pitch = 0;
-   int fds[3];
-
-   if (is_yuv(buf->format)) {
-      int ret = get_yuv_buffer_info(dri2_dpy, buf, out_buf_info);
-      /*
-       * HACK: https://issuetracker.google.com/32077885
-       * There is no API available to properly query the IMPLEMENTATION_DEFINED
-       * format. As a workaround we rely here on gralloc allocating either
-       * an arbitrary YCbCr 4:2:0 or RGBX_8888, with the latter being recognized
-       * by lock_ycbcr failing.
-       */
-      if (ret != -EAGAIN)
-         return ret;
-   }
-
-   /*
-    * Non-YUV formats could *also* have multiple planes, such as ancillary
-    * color compression state buffer, but the rest of the code isn't ready
-    * yet to deal with modifiers:
-    */
-   num_planes = get_native_buffer_fds(buf, fds);
-   if (num_planes == 0)
-      return -EINVAL;
-
-   assert(num_planes == 1);
-
-   drm_fourcc = get_fourcc(buf->format);
-   if (drm_fourcc == -1) {
-      _eglError(EGL_BAD_PARAMETER, "eglCreateEGLImageKHR");
-      return -EINVAL;
-   }
-
-   pitch = buf->stride * get_format_bpp(buf->format);
-   if (pitch == 0) {
-      _eglError(EGL_BAD_PARAMETER, "eglCreateEGLImageKHR");
-      return -EINVAL;
-   }
-
-   *out_buf_info = (struct buffer_info){
-      .width = buf->width,
-      .height = buf->height,
-      .drm_fourcc = drm_fourcc,
-      .num_planes = num_planes,
-      .fds = {fds[0], -1, -1, -1},
-      .modifier = DRM_FORMAT_MOD_INVALID,
-      .offsets = {0, 0, 0, 0},
-      .pitches = {pitch, 0, 0, 0},
-      .yuv_color_space = EGL_ITU_REC601_EXT,
-      .sample_range = EGL_YUV_NARROW_RANGE_EXT,
-      .horizontal_siting = EGL_YUV_CHROMA_SITING_0_EXT,
-      .vertical_siting = EGL_YUV_CHROMA_SITING_0_EXT,
-   };
-
-   return 0;
-}
-
-/* More recent CrOS gralloc has a perform op that fills out the struct below
- * with canonical information about the buffer and its modifier, planes,
- * offsets and strides.  If we have this, we can skip straight to
- * createImageFromDmaBufs2() and avoid all the guessing and recalculations.
- * This also gives us the modifier and plane offsets/strides for multiplanar
- * compressed buffers (eg Intel CCS buffers) in order to make that work in
- * Android.
- */
-
-static const char cros_gralloc_module_name[] = "CrOS Gralloc";
-
-#define CROS_GRALLOC_DRM_GET_BUFFER_INFO               4
-#define CROS_GRALLOC_DRM_GET_USAGE                     5
-#define CROS_GRALLOC_DRM_GET_USAGE_FRONT_RENDERING_BIT 0x1
-
-struct cros_gralloc0_buffer_info {
-   uint32_t drm_fourcc;
-   int num_fds;
-   int fds[4];
-   uint64_t modifier;
-   int offset[4];
-   int stride[4];
-};
-
-static int
-cros_get_buffer_info(struct dri2_egl_display *dri2_dpy,
-                     struct ANativeWindowBuffer *buf,
-                     struct buffer_info *out_buf_info)
-{
-   struct cros_gralloc0_buffer_info info;
-
-   if (strcmp(dri2_dpy->gralloc->common.name, cros_gralloc_module_name) == 0 &&
-       dri2_dpy->gralloc->perform &&
-       dri2_dpy->gralloc->perform(dri2_dpy->gralloc,
-                                  CROS_GRALLOC_DRM_GET_BUFFER_INFO, buf->handle,
-                                  &info) == 0) {
-      *out_buf_info = (struct buffer_info){
-         .width = buf->width,
-         .height = buf->height,
-         .drm_fourcc = info.drm_fourcc,
-         .num_planes = info.num_fds,
-         .fds = {-1, -1, -1, -1},
-         .modifier = info.modifier,
-         .yuv_color_space = EGL_ITU_REC601_EXT,
-         .sample_range = EGL_YUV_NARROW_RANGE_EXT,
-         .horizontal_siting = EGL_YUV_CHROMA_SITING_0_EXT,
-         .vertical_siting = EGL_YUV_CHROMA_SITING_0_EXT,
-      };
-      for (int i = 0; i < out_buf_info->num_planes; i++) {
-         out_buf_info->fds[i] = info.fds[i];
-         out_buf_info->offsets[i] = info.offset[i];
-         out_buf_info->pitches[i] = info.stride[i];
-      }
-
-      return 0;
-   }
-
-   return -EINVAL;
-}
-
 static __DRIimage *
-droid_create_image_from_buffer_info(struct dri2_egl_display *dri2_dpy,
-                                    struct buffer_info *buf_info, void *priv)
+droid_create_image_from_buffer_info(
+   struct dri2_egl_display *dri2_dpy, int width, int height,
+   struct u_gralloc_buffer_basic_info *buf_info,
+   struct u_gralloc_buffer_color_info *color_info, void *priv)
 {
    unsigned error;
 
    if (dri2_dpy->image->base.version >= 15 &&
        dri2_dpy->image->createImageFromDmaBufs2 != NULL) {
       return dri2_dpy->image->createImageFromDmaBufs2(
-         dri2_dpy->dri_screen_render_gpu, buf_info->width, buf_info->height,
-         buf_info->drm_fourcc, buf_info->modifier, buf_info->fds,
-         buf_info->num_planes, buf_info->pitches, buf_info->offsets,
-         buf_info->yuv_color_space, buf_info->sample_range,
-         buf_info->horizontal_siting, buf_info->vertical_siting, &error, priv);
+         dri2_dpy->dri_screen_render_gpu, width, height, buf_info->drm_fourcc,
+         buf_info->modifier, buf_info->fds, buf_info->num_planes,
+         buf_info->strides, buf_info->offsets, color_info->yuv_color_space,
+         color_info->sample_range, color_info->horizontal_siting,
+         color_info->vertical_siting, &error, priv);
    }
 
    return dri2_dpy->image->createImageFromDmaBufs(
-      dri2_dpy->dri_screen_render_gpu, buf_info->width, buf_info->height,
-      buf_info->drm_fourcc, buf_info->fds, buf_info->num_planes,
-      buf_info->pitches, buf_info->offsets, buf_info->yuv_color_space,
-      buf_info->sample_range, buf_info->horizontal_siting,
-      buf_info->vertical_siting, &error, priv);
+      dri2_dpy->dri_screen_render_gpu, width, height, buf_info->drm_fourcc,
+      buf_info->fds, buf_info->num_planes, buf_info->strides, buf_info->offsets,
+      color_info->yuv_color_space, color_info->sample_range,
+      color_info->horizontal_siting, color_info->vertical_siting, &error, priv);
 }
 
 static __DRIimage *
@@ -452,23 +81,41 @@ droid_create_image_from_native_buffer(_EGLDisplay *disp,
                                       void *priv)
 {
    struct dri2_egl_display *dri2_dpy = dri2_egl_display(disp);
-   struct buffer_info buf_info;
+   struct u_gralloc_buffer_basic_info buf_info;
+   struct u_gralloc_buffer_color_info color_info = {
+      .yuv_color_space = __DRI_YUV_COLOR_SPACE_ITU_REC601,
+      .sample_range = __DRI_YUV_NARROW_RANGE,
+      .horizontal_siting = __DRI_YUV_CHROMA_SITING_0,
+      .vertical_siting = __DRI_YUV_CHROMA_SITING_0,
+   };
+   struct u_gralloc_buffer_handle gr_handle = {
+      .handle = buf->handle,
+      .hal_format = buf->format,
+      .pixel_stride = buf->stride,
+   };
    __DRIimage *img = NULL;
 
-   /* If dri driver is gallium virgl, real modifier info queried back from
-    * CrOS info (and potentially mapper metadata if integrated later) cannot
-    * get resolved and the buffer import will fail. Thus the fallback behavior
-    * is preserved down to native_window_buffer_get_buffer_info() so that the
-    * buffer can be imported without modifier info as a last resort.
-    */
-   if (!img && !mapper_metadata_get_buffer_info(buf, &buf_info))
-      img = droid_create_image_from_buffer_info(dri2_dpy, &buf_info, priv);
+   if (u_gralloc_get_buffer_basic_info(dri2_dpy->gralloc, &gr_handle,
+                                       &buf_info))
+      return 0;
 
-   if (!img && !cros_get_buffer_info(dri2_dpy, buf, &buf_info))
-      img = droid_create_image_from_buffer_info(dri2_dpy, &buf_info, priv);
+   /* May fail in some cases, defaults will be used in that case */
+   u_gralloc_get_buffer_color_info(dri2_dpy->gralloc, &gr_handle, &color_info);
 
-   if (!img && !native_window_buffer_get_buffer_info(dri2_dpy, buf, &buf_info))
-      img = droid_create_image_from_buffer_info(dri2_dpy, &buf_info, priv);
+   img = droid_create_image_from_buffer_info(dri2_dpy, buf->width, buf->height,
+                                             &buf_info, &color_info, priv);
+
+   if (!img) {
+      /* If dri driver is gallium virgl, real modifier info queried back from
+       * CrOS info (and potentially mapper metadata if integrated later) cannot
+       * get resolved and the buffer import will fail. Thus the fallback
+       * behavior is preserved so that the buffer can be imported without
+       * modifier info as a last resort.
+       */
+      buf_info.modifier = DRM_FORMAT_MOD_INVALID;
+      img = droid_create_image_from_buffer_info(
+         dri2_dpy, buf->width, buf->height, &buf_info, &color_info, priv);
+   }
 
    return img;
 }
@@ -1040,59 +687,6 @@ droid_swap_buffers(_EGLDisplay *disp, _EGLSurface *draw)
    return EGL_TRUE;
 }
 
-#ifdef HAVE_DRM_GRALLOC
-static int
-get_format(int format)
-{
-   switch (format) {
-   case HAL_PIXEL_FORMAT_BGRA_8888:
-      return __DRI_IMAGE_FORMAT_ARGB8888;
-   case HAL_PIXEL_FORMAT_RGB_565:
-      return __DRI_IMAGE_FORMAT_RGB565;
-   case HAL_PIXEL_FORMAT_RGBA_8888:
-      return __DRI_IMAGE_FORMAT_ABGR8888;
-   case HAL_PIXEL_FORMAT_IMPLEMENTATION_DEFINED:
-      /*
-       * HACK: Hardcode this to RGBX_8888 as per cros_gralloc hack.
-       * TODO: Revert this once https://issuetracker.google.com/32077885 is
-       * fixed.
-       */
-   case HAL_PIXEL_FORMAT_RGBX_8888:
-      return __DRI_IMAGE_FORMAT_XBGR8888;
-   case HAL_PIXEL_FORMAT_RGBA_FP16:
-      return __DRI_IMAGE_FORMAT_ABGR16161616F;
-   case HAL_PIXEL_FORMAT_RGBA_1010102:
-      return __DRI_IMAGE_FORMAT_ABGR2101010;
-   default:
-      _eglLog(_EGL_WARNING, "unsupported native buffer format 0x%x", format);
-   }
-   return -1;
-}
-
-static __DRIimage *
-droid_create_image_from_name(_EGLDisplay *disp, struct ANativeWindowBuffer *buf,
-                             void *priv)
-{
-   struct dri2_egl_display *dri2_dpy = dri2_egl_display(disp);
-   int name;
-   int format;
-
-   name = get_native_buffer_name(buf);
-   if (!name) {
-      _eglError(EGL_BAD_PARAMETER, "eglCreateEGLImageKHR");
-      return NULL;
-   }
-
-   format = get_format(buf->format);
-   if (format == -1)
-      return NULL;
-
-   return dri2_dpy->image->createImageFromName(dri2_dpy->dri_screen_render_gpu,
-                                               buf->width, buf->height, format,
-                                               name, buf->stride, priv);
-}
-#endif /* HAVE_DRM_GRALLOC */
-
 static EGLBoolean
 droid_query_surface(_EGLDisplay *disp, _EGLSurface *surf, EGLint attribute,
                     EGLint *value)
@@ -1145,11 +739,6 @@ dri2_create_image_android_native_buffer(_EGLDisplay *disp, _EGLContext *ctx,
    __DRIimage *dri_image =
       droid_create_image_from_native_buffer(disp, buf, buf);
 
-#ifdef HAVE_DRM_GRALLOC
-   if (dri_image == NULL)
-      dri_image = droid_create_image_from_name(disp, buf, buf);
-#endif
-
    if (dri_image) {
 #if ANDROID_API_LEVEL >= 26
       AHardwareBuffer_acquire(ANativeWindowBuffer_getHardwareBuffer(buf));
@@ -1178,84 +767,6 @@ droid_flush_front_buffer(__DRIdrawable *driDrawable, void *loaderPrivate)
 {
 }
 
-#ifdef HAVE_DRM_GRALLOC
-static int
-droid_get_buffers_parse_attachments(struct dri2_egl_surface *dri2_surf,
-                                    unsigned int *attachments, int count)
-{
-   int num_buffers = 0;
-
-   /* fill dri2_surf->buffers */
-   for (int i = 0; i < count * 2; i += 2) {
-      __DRIbuffer *buf, *local;
-
-      assert(num_buffers < ARRAY_SIZE(dri2_surf->buffers));
-      buf = &dri2_surf->buffers[num_buffers];
-
-      switch (attachments[i]) {
-      case __DRI_BUFFER_BACK_LEFT:
-         if (dri2_surf->base.Type == EGL_WINDOW_BIT) {
-            buf->attachment = attachments[i];
-            buf->name = get_native_buffer_name(dri2_surf->buffer);
-            buf->cpp = get_format_bpp(dri2_surf->buffer->format);
-            buf->pitch = dri2_surf->buffer->stride * buf->cpp;
-            buf->flags = 0;
-
-            if (buf->name)
-               num_buffers++;
-
-            break;
-         }
-         FALLTHROUGH; /* for pbuffers */
-      case __DRI_BUFFER_DEPTH:
-      case __DRI_BUFFER_STENCIL:
-      case __DRI_BUFFER_ACCUM:
-      case __DRI_BUFFER_DEPTH_STENCIL:
-      case __DRI_BUFFER_HIZ:
-         local = dri2_egl_surface_alloc_local_buffer(dri2_surf, attachments[i],
-                                                     attachments[i + 1]);
-
-         if (local) {
-            *buf = *local;
-            num_buffers++;
-         }
-         break;
-      case __DRI_BUFFER_FRONT_LEFT:
-      case __DRI_BUFFER_FRONT_RIGHT:
-      case __DRI_BUFFER_FAKE_FRONT_LEFT:
-      case __DRI_BUFFER_FAKE_FRONT_RIGHT:
-      case __DRI_BUFFER_BACK_RIGHT:
-      default:
-         /* no front or right buffers */
-         break;
-      }
-   }
-
-   return num_buffers;
-}
-
-static __DRIbuffer *
-droid_get_buffers_with_format(__DRIdrawable *driDrawable, int *width,
-                              int *height, unsigned int *attachments, int count,
-                              int *out_count, void *loaderPrivate)
-{
-   struct dri2_egl_surface *dri2_surf = loaderPrivate;
-
-   if (update_buffers(dri2_surf) < 0)
-      return NULL;
-
-   *out_count =
-      droid_get_buffers_parse_attachments(dri2_surf, attachments, count);
-
-   if (width)
-      *width = dri2_surf->base.Width;
-   if (height)
-      *height = dri2_surf->base.Height;
-
-   return dri2_surf->buffers;
-}
-#endif /* HAVE_DRM_GRALLOC */
-
 static unsigned
 droid_get_capability(void *loaderPrivate, enum dri_loader_cap cap)
 {
@@ -1279,26 +790,24 @@ droid_destroy_loader_image_state(void *loaderPrivate)
 #endif
 }
 
-static EGLBoolean
+static void
 droid_add_configs_for_visuals(_EGLDisplay *disp)
 {
    struct dri2_egl_display *dri2_dpy = dri2_egl_display(disp);
    static const struct {
-      int format;
-      int rgba_shifts[4];
-      unsigned int rgba_sizes[4];
+      int hal_format;
+      enum pipe_format pipe_format;
    } visuals[] = {
-      {HAL_PIXEL_FORMAT_RGBA_8888, {0, 8, 16, 24}, {8, 8, 8, 8}},
-      {HAL_PIXEL_FORMAT_RGBX_8888, {0, 8, 16, -1}, {8, 8, 8, 0}},
-      {HAL_PIXEL_FORMAT_RGB_565, {11, 5, 0, -1}, {5, 6, 5, 0}},
+      {HAL_PIXEL_FORMAT_RGBA_8888, PIPE_FORMAT_RGBA8888_UNORM},
+      {HAL_PIXEL_FORMAT_RGBX_8888, PIPE_FORMAT_RGBX8888_UNORM},
+      {HAL_PIXEL_FORMAT_RGB_565, PIPE_FORMAT_R5G6B5_UNORM},
       /* This must be after HAL_PIXEL_FORMAT_RGBA_8888, we only keep BGRA
        * visual if it turns out RGBA visual is not available.
        */
-      {HAL_PIXEL_FORMAT_BGRA_8888, {16, 8, 0, 24}, {8, 8, 8, 8}},
+      {HAL_PIXEL_FORMAT_BGRA_8888, PIPE_FORMAT_BGRA8888_UNORM},
    };
 
    unsigned int format_count[ARRAY_SIZE(visuals)] = {0};
-   int config_count = 0;
 
    /* The nesting of loops is significant here. Also significant is the order
     * of the HAL pixel formats. Many Android apps (such as Google's official
@@ -1323,16 +832,26 @@ droid_add_configs_for_visuals(_EGLDisplay *disp)
       /* Only enable BGRA configs when RGBA is not available. BGRA configs are
        * buggy on stock Android.
        */
-      if (visuals[i].format == HAL_PIXEL_FORMAT_BGRA_8888 && has_rgba)
+      if (visuals[i].hal_format == HAL_PIXEL_FORMAT_BGRA_8888 && has_rgba)
          continue;
       for (int j = 0; dri2_dpy->driver_configs[j]; j++) {
-         const EGLint surface_type = EGL_WINDOW_BIT | EGL_PBUFFER_BIT;
+         const struct gl_config *gl_config =
+            (struct gl_config *) dri2_dpy->driver_configs[j];
 
+         /* Rather than have duplicate table entries for _SRGB formats, just
+          * use the linear version of the format for the comparision:
+          */
+         enum pipe_format linear_format =
+            util_format_linear(gl_config->color_format);
+         if (linear_format != visuals[i].pipe_format)
+            continue;
+
+         const EGLint surface_type = EGL_WINDOW_BIT | EGL_PBUFFER_BIT;
          const EGLint config_attrs[] = {
             EGL_NATIVE_VISUAL_ID,
-            visuals[i].format,
+            visuals[i].hal_format,
             EGL_NATIVE_VISUAL_TYPE,
-            visuals[i].format,
+            visuals[i].hal_format,
             EGL_FRAMEBUFFER_TARGET_ANDROID,
             EGL_TRUE,
             EGL_RECORDABLE_ANDROID,
@@ -1341,26 +860,21 @@ droid_add_configs_for_visuals(_EGLDisplay *disp)
          };
 
          struct dri2_egl_config *dri2_conf = dri2_add_config(
-            disp, dri2_dpy->driver_configs[j], config_count + 1, surface_type,
-            config_attrs, visuals[i].rgba_shifts, visuals[i].rgba_sizes);
-         if (dri2_conf) {
-            if (dri2_conf->base.ConfigID == config_count + 1)
-               config_count++;
+            disp, dri2_dpy->driver_configs[j], surface_type, config_attrs);
+         if (dri2_conf)
             format_count[i]++;
-         }
       }
-      if (visuals[i].format == HAL_PIXEL_FORMAT_RGBA_8888 && format_count[i])
+
+      if (visuals[i].hal_format == HAL_PIXEL_FORMAT_RGBA_8888 && format_count[i])
          has_rgba = true;
    }
 
    for (int i = 0; i < ARRAY_SIZE(format_count); i++) {
       if (!format_count[i]) {
          _eglLog(_EGL_DEBUG, "No DRI config supports native format 0x%x",
-                 visuals[i].format);
+                 visuals[i].hal_format);
       }
    }
-
-   return (config_count != 0);
 }
 
 static const struct dri2_egl_display_vtbl droid_display_vtbl = {
@@ -1377,28 +891,6 @@ static const struct dri2_egl_display_vtbl droid_display_vtbl = {
    .set_shared_buffer_mode = droid_set_shared_buffer_mode,
 };
 
-#ifdef HAVE_DRM_GRALLOC
-static const __DRIdri2LoaderExtension droid_dri2_loader_extension = {
-   .base = {__DRI_DRI2_LOADER, 5},
-
-   .getBuffers = NULL,
-   .flushFrontBuffer = droid_flush_front_buffer,
-   .getBuffersWithFormat = droid_get_buffers_with_format,
-   .getCapability = droid_get_capability,
-   .destroyLoaderImageState = droid_destroy_loader_image_state,
-};
-
-static const __DRIextension *droid_dri2_loader_extensions[] = {
-   &droid_dri2_loader_extension.base,
-   &image_lookup_extension.base,
-   &use_invalidate.base,
-   /* No __DRI_MUTABLE_RENDER_BUFFER_LOADER because it requires
-    * __DRI_IMAGE_LOADER.
-    */
-   NULL,
-};
-#endif /* HAVE_DRM_GRALLOC */
-
 static const __DRIimageLoaderExtension droid_image_loader_extension = {
    .base = {__DRI_IMAGE_LOADER, 4},
 
@@ -1490,20 +982,14 @@ droid_load_driver(_EGLDisplay *disp, bool swrast)
 {
    struct dri2_egl_display *dri2_dpy = dri2_egl_display(disp);
 
-   dri2_dpy->driver_name = loader_get_driver_for_fd(dri2_dpy->fd_render_gpu);
+   if (disp->Options.Zink)
+      dri2_dpy->driver_name = strdup("zink");
+   else
+      dri2_dpy->driver_name = loader_get_driver_for_fd(dri2_dpy->fd_render_gpu);
    if (dri2_dpy->driver_name == NULL)
       return false;
 
-#ifdef HAVE_DRM_GRALLOC
-   /* Handle control nodes using __DRI_DRI2_LOADER extension and GEM names
-    * for backwards compatibility with drm_gralloc. (Do not use on new
-    * systems.) */
-   dri2_dpy->loader_extensions = droid_dri2_loader_extensions;
-   if (!dri2_load_driver(disp)) {
-      goto error;
-   }
-#else
-   if (swrast) {
+   if (swrast && !disp->Options.Zink) {
       /* Use kms swrast only with vgem / virtio_gpu.
        * virtio-gpu fallbacks to software rendering when 3D features
        * are unavailable since 6c5ab.
@@ -1521,7 +1007,6 @@ droid_load_driver(_EGLDisplay *disp, bool swrast)
    if (!dri2_load_driver_dri3(disp)) {
       goto error;
    }
-#endif
 
    return true;
 
@@ -1576,41 +1061,13 @@ droid_probe_device(_EGLDisplay *disp, bool swrast)
    return EGL_TRUE;
 }
 
-#ifdef HAVE_DRM_GRALLOC
-static EGLBoolean
-droid_open_device(_EGLDisplay *disp, bool swrast)
-{
-   struct dri2_egl_display *dri2_dpy = dri2_egl_display(disp);
-   int fd = -1, err = -EINVAL;
-
-   if (swrast)
-      return EGL_FALSE;
-
-   if (dri2_dpy->gralloc->perform)
-      err = dri2_dpy->gralloc->perform(dri2_dpy->gralloc,
-                                       GRALLOC_MODULE_PERFORM_GET_DRM_FD, &fd);
-   if (err || fd < 0) {
-      _eglLog(_EGL_WARNING, "fail to get drm fd");
-      return EGL_FALSE;
-   }
-
-   dri2_dpy->fd_render_gpu = os_dupfd_cloexec(fd);
-   if (dri2_dpy->fd_render_gpu < 0)
-      return EGL_FALSE;
-
-   if (drmGetNodeTypeFromFd(dri2_dpy->fd_render_gpu) == DRM_NODE_RENDER)
-      return EGL_FALSE;
-
-   return droid_probe_device(disp, swrast);
-}
-#else
 static EGLBoolean
 droid_open_device(_EGLDisplay *disp, bool swrast)
 {
 #define MAX_DRM_DEVICES 64
    struct dri2_egl_display *dri2_dpy = dri2_egl_display(disp);
-   drmDevicePtr device, devices[MAX_DRM_DEVICES] = {NULL};
-   int num_devices;
+   _EGLDevice *dev_list = _eglGlobal.DeviceList;
+   drmDevicePtr device;
 
    char *vendor_name = NULL;
    char vendor_buf[PROPERTY_VALUE_MAX];
@@ -1624,21 +1081,21 @@ droid_open_device(_EGLDisplay *disp, bool swrast)
    if (property_get("drm.gpu.vendor_name", vendor_buf, NULL) > 0)
       vendor_name = vendor_buf;
 
-   num_devices = drmGetDevices2(0, devices, ARRAY_SIZE(devices));
-   if (num_devices < 0)
-      return EGL_FALSE;
+   while (dev_list) {
+      if (!_eglDeviceSupports(dev_list, _EGL_DEVICE_DRM))
+         goto next;
 
-   for (int i = 0; i < num_devices; i++) {
-      device = devices[i];
+      device = _eglDeviceDrm(dev_list);
+      assert(device);
 
       if (!(device->available_nodes & (1 << node_type)))
-         continue;
+         goto next;
 
       dri2_dpy->fd_render_gpu = loader_open_device(device->nodes[node_type]);
       if (dri2_dpy->fd_render_gpu < 0) {
          _eglLog(_EGL_WARNING, "%s() Failed to open DRM device %s", __func__,
                  device->nodes[node_type]);
-         continue;
+         goto next;
       }
 
       /* If a vendor is explicitly provided, we use only that.
@@ -1649,7 +1106,7 @@ droid_open_device(_EGLDisplay *disp, bool swrast)
             /* Device does not match - try next device */
             close(dri2_dpy->fd_render_gpu);
             dri2_dpy->fd_render_gpu = -1;
-            continue;
+            goto next;
          }
          /* If the requested device matches - use it. Regardless if
           * init fails, do not fall-back to any other device.
@@ -1667,8 +1124,10 @@ droid_open_device(_EGLDisplay *disp, bool swrast)
       /* No explicit request - attempt the next device */
       close(dri2_dpy->fd_render_gpu);
       dri2_dpy->fd_render_gpu = -1;
+
+   next:
+      dev_list = _eglDeviceNext(dev_list);
    }
-   drmFreeDevices(devices, num_devices);
 
    if (dri2_dpy->fd_render_gpu < 0) {
       _eglLog(_EGL_WARNING, "Failed to open %s DRM device",
@@ -1677,19 +1136,14 @@ droid_open_device(_EGLDisplay *disp, bool swrast)
    }
 
    return EGL_TRUE;
-#undef MAX_DRM_DEVICES
 }
 
-#endif
-
 EGLBoolean
 dri2_initialize_android(_EGLDisplay *disp)
 {
-   _EGLDevice *dev;
    bool device_opened = false;
    struct dri2_egl_display *dri2_dpy;
    const char *err;
-   int ret;
 
    dri2_dpy = calloc(1, sizeof(*dri2_dpy));
    if (!dri2_dpy)
@@ -1697,10 +1151,10 @@ dri2_initialize_android(_EGLDisplay *disp)
 
    dri2_dpy->fd_render_gpu = -1;
    dri2_dpy->fd_display_gpu = -1;
-   ret = hw_get_module(GRALLOC_HARDWARE_MODULE_ID,
-                       (const hw_module_t **)&dri2_dpy->gralloc);
-   if (ret) {
-      err = "DRI2: failed to get gralloc module";
+
+   dri2_dpy->gralloc = u_gralloc_create(U_GRALLOC_TYPE_AUTO);
+   if (dri2_dpy->gralloc == NULL) {
+      err = "DRI2: failed to get gralloc";
       goto cleanup;
    }
 
@@ -1714,16 +1168,13 @@ dri2_initialize_android(_EGLDisplay *disp)
 
    dri2_dpy->fd_display_gpu = dri2_dpy->fd_render_gpu;
 
-   dev = _eglAddDevice(dri2_dpy->fd_render_gpu, false);
-   if (!dev) {
-      err = "DRI2: failed to find EGLDevice";
+   if (!dri2_setup_extensions(disp)) {
+      err = "DRI2: failed to setup extensions";
       goto cleanup;
    }
 
-   disp->Device = dev;
-
-   if (!dri2_setup_extensions(disp)) {
-      err = "DRI2: failed to setup extensions";
+   if (!dri2_setup_device(disp, false)) {
+      err = "DRI2: failed to setup EGLDevice";
       goto cleanup;
    }
 
@@ -1784,13 +1235,9 @@ dri2_initialize_android(_EGLDisplay *disp)
        *
        * So at least we can force BO_USE_LINEAR as the fallback.
        */
-      uint32_t front_rendering_usage = 0;
-      if (!strcmp(dri2_dpy->gralloc->common.name, cros_gralloc_module_name) &&
-          dri2_dpy->gralloc->perform &&
-          dri2_dpy->gralloc->perform(
-             dri2_dpy->gralloc, CROS_GRALLOC_DRM_GET_USAGE,
-             CROS_GRALLOC_DRM_GET_USAGE_FRONT_RENDERING_BIT,
-             &front_rendering_usage) == 0) {
+      uint64_t front_rendering_usage = 0;
+      if (!u_gralloc_get_front_rendering_usage(dri2_dpy->gralloc,
+                                               &front_rendering_usage)) {
          dri2_dpy->front_rendering_usage = front_rendering_usage;
          disp->Extensions.KHR_mutable_render_buffer = EGL_TRUE;
       }
@@ -1800,10 +1247,7 @@ dri2_initialize_android(_EGLDisplay *disp)
    /* Create configs *after* enabling extensions because presence of DRI
     * driver extensions can affect the capabilities of EGLConfigs.
     */
-   if (!droid_add_configs_for_visuals(disp)) {
-      err = "DRI2: failed to add configs";
-      goto cleanup;
-   }
+   droid_add_configs_for_visuals(disp);
 
    /* Fill vtbl last to prevent accidentally calling virtual function during
     * initialization.
