@@ -72,6 +72,9 @@ struct ir3_compiler_options {
   bool lower_base_vertex;
 
   bool shared_push_consts;
+
+   /* "dual_color_blend_by_location" workaround is enabled: */
+   bool dual_color_blend_by_location;
 };
 
 struct ir3_compiler {
@@ -214,6 +217,18 @@ struct ir3_compiler {
     */
    bool has_getfiberid;
 
+   /* Number of available predicate registers (p0.c) */
+   uint32_t num_predicates;
+
+   /* True if bitops (and.b, or.b, xor.b, not.b) can write to p0.c */
+   bool bitops_can_write_predicates;
+
+   /* True if braa/brao are available. */
+   bool has_branch_and_or;
+
+   /* True if predt/predf/prede are supported. */
+   bool has_predication;
+
    /* MAX_COMPUTE_VARIABLE_GROUP_INVOCATIONS_ARB */
    uint32_t max_variable_workgroup_size;
 
@@ -249,6 +264,9 @@ struct ir3_compiler {
    bool has_fs_tex_prefetch;
 
    bool stsc_duplication_quirk;
+
+   bool load_shader_consts_via_preamble;
+   bool load_inline_uniforms_via_preamble_ldgk;
 };
 
 void ir3_compiler_destroy(struct ir3_compiler *compiler);
@@ -299,8 +317,10 @@ enum ir3_shader_debug {
    IR3_DBG_SPILLALL = BITFIELD_BIT(12),
    IR3_DBG_NOPREAMBLE = BITFIELD_BIT(13),
    IR3_DBG_SHADER_INTERNAL = BITFIELD_BIT(14),
+   IR3_DBG_FULLSYNC = BITFIELD_BIT(15),
+   IR3_DBG_FULLNOP = BITFIELD_BIT(16),
 
-   /* DEBUG-only options: */
+   /* MESA_DEBUG-only options: */
    IR3_DBG_SCHEDMSGS = BITFIELD_BIT(20),
    IR3_DBG_RAMSGS = BITFIELD_BIT(21),
 
