@@ -24,6 +24,7 @@
 #include "tu_cs.h"
 #include "tu_device.h"
 #include "tu_dynamic_rendering.h"
+#include "tu_rmv.h"
 
 static int
 safe_ioctl(int fd, unsigned long request, void *arg)
@@ -38,7 +39,7 @@ safe_ioctl(int fd, unsigned long request, void *arg)
 }
 
 static int
-kgsl_submitqueue_new(const struct tu_device *dev,
+kgsl_submitqueue_new(struct tu_device *dev,
                      int priority,
                      uint32_t *queue_id)
 {
@@ -58,7 +59,7 @@ kgsl_submitqueue_new(const struct tu_device *dev,
 }
 
 static void
-kgsl_submitqueue_close(const struct tu_device *dev, uint32_t queue_id)
+kgsl_submitqueue_close(struct tu_device *dev, uint32_t queue_id)
 {
    struct kgsl_drawctxt_destroy req = {
       .drawctxt_id = queue_id,
@@ -117,6 +118,12 @@ kgsl_bo_init(struct tu_device *dev,
 
    *out_bo = bo;
 
+   TU_RMV(bo_allocate, dev, bo);
+   if (flags & TU_BO_ALLOC_INTERNAL_RESOURCE) {
+      TU_RMV(internal_resource_create, dev, bo);
+      TU_RMV(resource_name, dev, bo, name);
+   }
+
    return VK_SUCCESS;
 }
 
@@ -190,6 +197,7 @@ kgsl_bo_map(struct tu_device *dev, struct tu_bo *bo)
       return vk_error(dev, VK_ERROR_MEMORY_MAP_FAILED);
 
    bo->map = map;
+   TU_RMV(bo_map, dev, bo);
 
    return VK_SUCCESS;
 }
@@ -207,8 +215,12 @@ kgsl_bo_finish(struct tu_device *dev, struct tu_bo *bo)
    if (!p_atomic_dec_zero(&bo->refcnt))
       return;
 
-   if (bo->map)
+   if (bo->map) {
+      TU_RMV(bo_unmap, dev, bo);
       munmap(bo->map, bo->size);
+   }
+
+   TU_RMV(bo_destroy, dev, bo);
 
    struct kgsl_gpumem_free_id req = {
       .id = bo->gem_handle
@@ -226,7 +238,7 @@ kgsl_sync_cache(VkDevice _device,
                 uint32_t count,
                 const VkMappedMemoryRange *ranges)
 {
-   TU_FROM_HANDLE(tu_device, device, _device);
+   VK_FROM_HANDLE(tu_device, device, _device);
 
    struct kgsl_gpuobj_sync_obj *sync_list =
       (struct kgsl_gpuobj_sync_obj *) vk_zalloc(
@@ -240,7 +252,7 @@ kgsl_sync_cache(VkDevice _device,
    };
 
    for (uint32_t i = 0; i < count; i++) {
-      TU_FROM_HANDLE(tu_device_memory, mem, ranges[i].memory);
+      VK_FROM_HANDLE(tu_device_memory, mem, ranges[i].memory);
 
       sync_list[i].op = op;
       sync_list[i].id = mem->bo->gem_handle;
@@ -332,6 +344,12 @@ struct kgsl_syncobj
    int fd;
 };
 
+struct tu_u_trace_syncobj
+{
+   uint32_t msm_queue_id;
+   uint32_t timestamp;
+};
+
 static void
 kgsl_syncobj_init(struct kgsl_syncobj *s, bool signaled)
 {
@@ -523,7 +541,7 @@ kgsl_syncobj_wait(struct tu_device *device,
    }
 
    case KGSL_SYNCOBJ_STATE_FD: {
-      int ret = sync_wait(device->fd, get_relative_ms(abs_timeout_ns));
+      int ret = sync_wait(s->fd, get_relative_ms(abs_timeout_ns));
       if (ret) {
          assert(errno == ETIME);
          return VK_TIMEOUT;
@@ -930,6 +948,9 @@ kgsl_queue_submit(struct tu_queue *queue, struct vk_queue_submit *vk_submit)
 {
    MESA_TRACE_FUNC();
 
+   bool u_trace_enabled = u_trace_should_process(&queue->device->trace_context);
+   bool has_trace_points = false;
+
    if (vk_submit->command_buffer_count == 0) {
       pthread_mutex_lock(&queue->device->submit_mutex);
 
@@ -1006,6 +1027,14 @@ kgsl_queue_submit(struct tu_queue *queue, struct vk_queue_submit *vk_submit)
          entry_count++;
 
       entry_count += cmd_buffer->cs.entry_count;
+
+      if (u_trace_enabled && u_trace_has_points(&cmd_buffers[i]->trace)) {
+         if (!(cmd_buffers[i]->usage_flags &
+               VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT))
+            entry_count++;
+
+         has_trace_points = true;
+      }
    }
 
    if (tu_autotune_submit_requires_fence(cmd_buffers, cmdbuf_count))
@@ -1019,6 +1048,26 @@ kgsl_queue_submit(struct tu_queue *queue, struct vk_queue_submit *vk_submit)
       return vk_error(queue, VK_ERROR_OUT_OF_HOST_MEMORY);
    }
 
+   uint32_t obj_count = 0;
+   if (has_trace_points)
+      obj_count++;
+
+   struct kgsl_command_object *objs = (struct kgsl_command_object *)
+      vk_alloc(&queue->device->vk.alloc, sizeof(*objs) * obj_count,
+               alignof(*objs), VK_SYSTEM_ALLOCATION_SCOPE_COMMAND);
+
+   struct tu_u_trace_submission_data *u_trace_submission_data = NULL;
+   if (has_trace_points) {
+      tu_u_trace_submission_data_create(
+         queue->device, cmd_buffers, cmdbuf_count, &u_trace_submission_data);
+
+      mtx_lock(&queue->device->kgsl_profiling_mutex);
+      tu_suballoc_bo_alloc(&u_trace_submission_data->kgsl_timestamp_bo,
+                           &queue->device->kgsl_profiling_suballoc,
+                           sizeof(struct kgsl_cmdbatch_profiling_buffer), 4);
+      mtx_unlock(&queue->device->kgsl_profiling_mutex);
+   }
+
    uint32_t entry_idx = 0;
    for (uint32_t i = 0; i < cmdbuf_count; i++) {
       struct tu_cmd_buffer *cmd_buffer = cmd_buffers[i];
@@ -1044,6 +1093,36 @@ kgsl_queue_submit(struct tu_queue *queue, struct vk_queue_submit *vk_submit)
             .id = cs->entries[j].bo->gem_handle,
          };
       }
+
+      if (u_trace_submission_data &&
+          u_trace_submission_data->cmd_trace_data[i].timestamp_copy_cs) {
+         struct tu_cs_entry *trace_cs_entry =
+            &u_trace_submission_data->cmd_trace_data[i]
+                .timestamp_copy_cs->entries[0];
+         cmds[entry_idx++] = (struct kgsl_command_object) {
+            .offset = trace_cs_entry->offset,
+            .gpuaddr = trace_cs_entry->bo->iova,
+            .size = trace_cs_entry->size,
+            .flags = KGSL_CMDLIST_IB,
+            .id = trace_cs_entry->bo->gem_handle,
+         };
+      }
+   }
+
+   struct kgsl_cmdbatch_profiling_buffer *profiling_buffer = NULL;
+   uint32_t obj_idx = 0;
+   if (u_trace_submission_data) {
+      struct tu_suballoc_bo *bo = &u_trace_submission_data->kgsl_timestamp_bo;
+
+      objs[obj_idx++] = (struct kgsl_command_object) {
+         .offset = bo->iova - bo->bo->iova,
+         .gpuaddr = bo->iova,
+         .size = sizeof(struct kgsl_cmdbatch_profiling_buffer),
+         .flags = KGSL_OBJLIST_MEMOBJ | KGSL_OBJLIST_PROFILE,
+         .id = bo->bo->gem_handle,
+      };
+      profiling_buffer =
+         (struct kgsl_cmdbatch_profiling_buffer *) tu_suballoc_bo_map(bo);
    }
 
    if (tu_autotune_submit_requires_fence(cmd_buffers, cmdbuf_count)) {
@@ -1112,16 +1191,55 @@ kgsl_queue_submit(struct tu_queue *queue, struct vk_queue_submit *vk_submit)
       .context_id = queue->msm_queue_id,
    };
 
+   if (obj_idx) {
+      req.flags |= KGSL_CMDBATCH_PROFILING;
+      req.objlist = (uintptr_t) objs;
+      req.objsize = sizeof(struct kgsl_command_object);
+      req.numobjs = obj_idx;
+   }
+
    int ret = safe_ioctl(queue->device->physical_device->local_fd,
                         IOCTL_KGSL_GPU_COMMAND, &req);
 
+   uint64_t gpu_offset = 0;
+#if HAVE_PERFETTO
+   if (profiling_buffer && profiling_buffer->gpu_ticks_queued) {
+      struct kgsl_perfcounter_read_group perf = {
+         .groupid = KGSL_PERFCOUNTER_GROUP_ALWAYSON,
+         .countable = 0,
+         .value = 0
+      };
+
+      struct kgsl_perfcounter_read req = {
+         .reads = &perf,
+         .count = 1,
+      };
+
+      ret = safe_ioctl(queue->device->fd, IOCTL_KGSL_PERFCOUNTER_READ, &req);
+      /* Older KGSL has some kind of garbage in upper 32 bits */
+      uint64_t offseted_gpu_ts = perf.value & 0xffffffff;
+
+      gpu_offset = tu_device_ticks_to_ns(
+         queue->device, offseted_gpu_ts - profiling_buffer->gpu_ticks_queued);
+
+      struct tu_perfetto_clocks clocks = {
+         .cpu = profiling_buffer->wall_clock_ns,
+         .gpu_ts = tu_device_ticks_to_ns(queue->device,
+                                         profiling_buffer->gpu_ticks_queued),
+         .gpu_ts_offset = gpu_offset,
+      };
+
+      clocks = tu_perfetto_submit(queue->device, queue->device->submit_count, &clocks);
+      gpu_offset = clocks.gpu_ts_offset;
+   }
+#endif
+
    kgsl_syncobj_destroy(&wait_sync);
 
    if (ret) {
       result = vk_device_set_lost(&queue->device->vk, "submit failed: %s\n",
                                   strerror(errno));
-      pthread_mutex_unlock(&queue->device->submit_mutex);
-      return result;
+      goto fail_submit;
    }
 
    p_atomic_set(&queue->fence, req.timestamp);
@@ -1137,16 +1255,81 @@ kgsl_queue_submit(struct tu_queue *queue, struct vk_queue_submit *vk_submit)
       signal_sync->timestamp = req.timestamp;
    }
 
+   if (u_trace_submission_data) {
+      struct tu_u_trace_submission_data *submission_data =
+         u_trace_submission_data;
+      submission_data->submission_id = queue->device->submit_count;
+      submission_data->gpu_ts_offset = gpu_offset;
+      /* We have to allocate it here since it is different between drm/kgsl */
+      submission_data->syncobj = (struct tu_u_trace_syncobj *)
+         vk_alloc(&queue->device->vk.alloc, sizeof(struct tu_u_trace_syncobj),
+               8, VK_SYSTEM_ALLOCATION_SCOPE_DEVICE);
+         submission_data->syncobj->timestamp = req.timestamp;
+         submission_data->syncobj->msm_queue_id = queue->msm_queue_id;
+
+      u_trace_submission_data = NULL;
+
+      for (uint32_t i = 0; i < submission_data->cmd_buffer_count; i++) {
+         bool free_data = i == submission_data->last_buffer_with_tracepoints;
+         if (submission_data->cmd_trace_data[i].trace)
+            u_trace_flush(submission_data->cmd_trace_data[i].trace,
+                          submission_data, free_data);
+
+         if (!submission_data->cmd_trace_data[i].timestamp_copy_cs) {
+            /* u_trace is owned by cmd_buffer */
+            submission_data->cmd_trace_data[i].trace = NULL;
+         }
+      }
+   }
+
+   queue->device->submit_count++;
+
    pthread_mutex_unlock(&queue->device->submit_mutex);
    pthread_cond_broadcast(&queue->device->timeline_cond);
 
+   u_trace_context_process(&queue->device->trace_context, true);
+
+   if (cmd_buffers != (struct tu_cmd_buffer **) vk_submit->command_buffers)
+      vk_free(&queue->device->vk.alloc, cmd_buffers);
+
+   vk_free(&queue->device->vk.alloc, cmds);
+
+   return VK_SUCCESS;
+
+fail_submit:
+   pthread_mutex_unlock(&queue->device->submit_mutex);
+
+   if (result != VK_SUCCESS) {
+      mtx_lock(&queue->device->kgsl_profiling_mutex);
+      tu_suballoc_bo_free(&queue->device->kgsl_profiling_suballoc,
+                          &u_trace_submission_data->kgsl_timestamp_bo);
+      mtx_unlock(&queue->device->kgsl_profiling_mutex);
+   }
+
+   if (cmd_buffers != (struct tu_cmd_buffer **) vk_submit->command_buffers)
+      vk_free(&queue->device->vk.alloc, cmd_buffers);
+
+   vk_free(&queue->device->vk.alloc, cmds);
+
    return result;
 }
 
 static VkResult
 kgsl_device_wait_u_trace(struct tu_device *dev, struct tu_u_trace_syncobj *syncobj)
 {
-   tu_finishme("tu_device_wait_u_trace");
+   struct kgsl_device_waittimestamp_ctxtid req = {
+      .context_id = syncobj->msm_queue_id,
+      .timestamp = syncobj->timestamp,
+      .timeout = 5000, // 5s
+   };
+
+   int ret = safe_ioctl(dev->fd, IOCTL_KGSL_DEVICE_WAITTIMESTAMP_CTXTID, &req);
+
+   if (ret) {
+      assert(errno == ETIME);
+      return VK_TIMEOUT;
+   }
+
    return VK_SUCCESS;
 }
 
@@ -1166,7 +1349,7 @@ kgsl_device_finish(struct tu_device *dev)
 static int
 kgsl_device_get_gpu_timestamp(struct tu_device *dev, uint64_t *ts)
 {
-   tu_finishme("tu_device_get_gpu_timestamp");
+   unreachable("");
    return 0;
 }
 
@@ -1271,7 +1454,7 @@ tu_knl_kgsl_load(struct tu_instance *instance, int fd)
    device->sync_types[1] = &device->timeline_type.sync;
    device->sync_types[2] = NULL;
 
-   device->heap.size = tu_get_system_heap_size();
+   device->heap.size = tu_get_system_heap_size(device);
    device->heap.used = 0u;
    device->heap.flags = VK_MEMORY_HEAP_DEVICE_LOCAL_BIT;
 
