@@ -18,6 +18,9 @@
 #include "tu_cmd_buffer.h"
 #include "tu_cs.h"
 #include "tu_device.h"
+#include "tu_rmv.h"
+
+#include "common/freedreno_gpu_event.h"
 
 #define NSEC_PER_SEC 1000000000ull
 #define WAIT_TIMEOUT 5
@@ -27,19 +30,14 @@ struct PACKED query_slot {
    uint64_t available;
 };
 
-struct PACKED occlusion_slot_value {
-   /* Seems sample counters are placed to be 16-byte aligned
-    * even though this query needs an 8-byte slot. */
-   uint64_t value;
-   uint64_t _padding;
-};
-
 struct PACKED occlusion_query_slot {
    struct query_slot common;
-   uint64_t result;
+   uint64_t _padding0;
 
-   struct occlusion_slot_value begin;
-   struct occlusion_slot_value end;
+   uint64_t begin;
+   uint64_t result;
+   uint64_t end;
+   uint64_t _padding1;
 };
 
 struct PACKED timestamp_query_slot {
@@ -93,13 +91,18 @@ struct PACKED primitives_generated_query_slot {
    uint64_t end;
 };
 
-/* Returns the IOVA of a given uint64_t field in a given slot of a query
- * pool. */
+/* Returns the IOVA or mapped address of a given uint64_t field
+ * in a given slot of a query pool. */
 #define query_iova(type, pool, query, field)                         \
    pool->bo->iova + pool->stride * (query) + offsetof(type, field)
+#define query_addr(type, pool, query, field)                         \
+   (uint64_t *) ((char *) pool->bo->map + pool->stride * (query) +   \
+                 offsetof(type, field))
 
 #define occlusion_query_iova(pool, query, field)                     \
    query_iova(struct occlusion_query_slot, pool, query, field)
+#define occlusion_query_addr(pool, query, field)                     \
+   query_addr(struct occlusion_query_slot, pool, query, field)
 
 #define pipeline_stat_query_iova(pool, query, field, idx)                    \
    pool->bo->iova + pool->stride * (query) +                                 \
@@ -211,7 +214,7 @@ tu_CreateQueryPool(VkDevice _device,
                    const VkAllocationCallbacks *pAllocator,
                    VkQueryPool *pQueryPool)
 {
-   TU_FROM_HANDLE(tu_device, device, _device);
+   VK_FROM_HANDLE(tu_device, device, _device);
    assert(pCreateInfo->sType == VK_STRUCTURE_TYPE_QUERY_POOL_CREATE_INFO);
    assert(pCreateInfo->queryCount > 0);
 
@@ -331,6 +334,9 @@ tu_CreateQueryPool(VkDevice _device,
    pool->stride = slot_size;
    pool->size = pCreateInfo->queryCount;
    pool->pipeline_statistics = pCreateInfo->pipelineStatistics;
+
+   TU_RMV(query_pool_create, device, pool);
+
    *pQueryPool = tu_query_pool_to_handle(pool);
 
    return VK_SUCCESS;
@@ -341,12 +347,14 @@ tu_DestroyQueryPool(VkDevice _device,
                     VkQueryPool _pool,
                     const VkAllocationCallbacks *pAllocator)
 {
-   TU_FROM_HANDLE(tu_device, device, _device);
-   TU_FROM_HANDLE(tu_query_pool, pool, _pool);
+   VK_FROM_HANDLE(tu_device, device, _device);
+   VK_FROM_HANDLE(tu_query_pool, pool, _pool);
 
    if (!pool)
       return;
 
+   TU_RMV(resource_destroy, device, pool);
+
    tu_bo_finish(device, pool->bo);
    vk_object_free(&device->vk, pAllocator, pool);
 }
@@ -519,6 +527,9 @@ get_query_pool_results(struct tu_device *device,
                result = query_result_addr(pool, query, uint64_t, stat_idx);
             } else if (pool->type == VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR) {
                result = query_result_addr(pool, query, struct perfcntr_query_slot, k);
+            } else if (pool->type == VK_QUERY_TYPE_OCCLUSION) {
+               assert(k == 0);
+               result = occlusion_query_addr(pool, query, result);
             } else {
                result = query_result_addr(pool, query, uint64_t, k);
             }
@@ -561,8 +572,8 @@ tu_GetQueryPoolResults(VkDevice _device,
                        VkDeviceSize stride,
                        VkQueryResultFlags flags)
 {
-   TU_FROM_HANDLE(tu_device, device, _device);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_device, device, _device);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
    assert(firstQuery + queryCount <= pool->size);
 
    if (vk_device_is_lost(&device->vk))
@@ -603,6 +614,7 @@ copy_query_value_gpu(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit_qw(cs, src_iova);
 }
 
+template <chip CHIP>
 static void
 emit_copy_query_pool_results(struct tu_cmd_buffer *cmdbuf,
                              struct tu_cs *cs,
@@ -615,7 +627,7 @@ emit_copy_query_pool_results(struct tu_cmd_buffer *cmdbuf,
                              VkQueryResultFlags flags)
 {
    /* Flush cache for the buffer to copy to. */
-   tu_emit_cache_flush(cmdbuf);
+   tu_emit_cache_flush<CHIP>(cmdbuf);
 
    /* From the Vulkan 1.1.130 spec:
     *
@@ -656,6 +668,9 @@ emit_copy_query_pool_results(struct tu_cmd_buffer *cmdbuf,
          } else if (pool->type == VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR) {
             result_iova = query_result_iova(pool, query,
                                             struct perfcntr_query_slot, k);
+         } else if (pool->type == VK_QUERY_TYPE_OCCLUSION) {
+            assert(k == 0);
+            result_iova = occlusion_query_iova(pool, query, result);
          } else {
             result_iova = query_result_iova(pool, query, uint64_t, k);
          }
@@ -697,6 +712,7 @@ emit_copy_query_pool_results(struct tu_cmd_buffer *cmdbuf,
    }
 }
 
+template <chip CHIP>
 VKAPI_ATTR void VKAPI_CALL
 tu_CmdCopyQueryPoolResults(VkCommandBuffer commandBuffer,
                            VkQueryPool queryPool,
@@ -707,9 +723,9 @@ tu_CmdCopyQueryPoolResults(VkCommandBuffer commandBuffer,
                            VkDeviceSize stride,
                            VkQueryResultFlags flags)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
-   TU_FROM_HANDLE(tu_buffer, buffer, dstBuffer);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_buffer, buffer, dstBuffer);
    struct tu_cs *cs = &cmdbuf->cs;
    assert(firstQuery + queryCount <= pool->size);
 
@@ -719,14 +735,16 @@ tu_CmdCopyQueryPoolResults(VkCommandBuffer commandBuffer,
    case VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT:
    case VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT:
    case VK_QUERY_TYPE_PIPELINE_STATISTICS:
-      return emit_copy_query_pool_results(cmdbuf, cs, pool, firstQuery,
-               queryCount, buffer, dstOffset, stride, flags);
+      return emit_copy_query_pool_results<CHIP>(cmdbuf, cs, pool, firstQuery,
+                                                queryCount, buffer, dstOffset,
+                                                stride, flags);
    case VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR:
       unreachable("allowCommandBufferQueryCopies is false");
    default:
       assert(!"Invalid query type");
    }
 }
+TU_GENX(tu_CmdCopyQueryPoolResults);
 
 static void
 emit_reset_query_pool(struct tu_cmd_buffer *cmdbuf,
@@ -753,6 +771,9 @@ emit_reset_query_pool(struct tu_cmd_buffer *cmdbuf,
          } else if (pool->type == VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR) {
             result_iova = query_result_iova(pool, query,
                                             struct perfcntr_query_slot, k);
+         } else if (pool->type == VK_QUERY_TYPE_OCCLUSION) {
+            assert(k == 0);
+            result_iova = occlusion_query_iova(pool, query, result);
          } else {
             result_iova = query_result_iova(pool, query, uint64_t, k);
          }
@@ -771,8 +792,8 @@ tu_CmdResetQueryPool(VkCommandBuffer commandBuffer,
                      uint32_t firstQuery,
                      uint32_t queryCount)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
 
    switch (pool->type) {
    case VK_QUERY_TYPE_TIMESTAMP:
@@ -794,7 +815,7 @@ tu_ResetQueryPool(VkDevice device,
                   uint32_t firstQuery,
                   uint32_t queryCount)
 {
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
 
    for (uint32_t i = 0; i < queryCount; i++) {
       struct query_slot *slot = slot_address(pool, i + firstQuery);
@@ -806,6 +827,9 @@ tu_ResetQueryPool(VkDevice device,
          if (pool->type == VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR) {
             res = query_result_addr(pool, i + firstQuery,
                                     struct perfcntr_query_slot, k);
+         } else if (pool->type == VK_QUERY_TYPE_OCCLUSION) {
+            assert(k == 0);
+            res = occlusion_query_addr(pool, i + firstQuery, result);
          } else {
             res = query_result_addr(pool, i + firstQuery, uint64_t, k);
          }
@@ -815,6 +839,7 @@ tu_ResetQueryPool(VkDevice device,
    }
 }
 
+template <chip CHIP>
 static void
 emit_begin_occlusion_query(struct tu_cmd_buffer *cmdbuf,
                            struct tu_query_pool *pool,
@@ -840,13 +865,46 @@ emit_begin_occlusion_query(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit_regs(cs,
                    A6XX_RB_SAMPLE_COUNT_CONTROL(.copy = true));
 
-   tu_cs_emit_regs(cs,
-                   A6XX_RB_SAMPLE_COUNT_ADDR(.qword = begin_iova));
+   if (!cmdbuf->device->physical_device->info->a7xx.has_event_write_sample_count) {
+      tu_cs_emit_regs(cs,
+                        A6XX_RB_SAMPLE_COUNT_ADDR(.qword = begin_iova));
+      tu_cs_emit_pkt7(cs, CP_EVENT_WRITE, 1);
+      tu_cs_emit(cs, ZPASS_DONE);
+      if (CHIP == A7XX) {
+         /* Copied from blob's cmdstream, not sure why it is done. */
+         tu_cs_emit_pkt7(cs, CP_EVENT_WRITE, 1);
+         tu_cs_emit(cs, CCU_CLEAN_DEPTH);
+      }
+   } else {
+      tu_cs_emit_pkt7(cs, CP_EVENT_WRITE7, 3);
+      tu_cs_emit(cs, CP_EVENT_WRITE7_0(.event = ZPASS_DONE,
+                                       .write_sample_count = true).value);
+      tu_cs_emit_qw(cs, begin_iova);
 
-   tu_cs_emit_pkt7(cs, CP_EVENT_WRITE, 1);
-   tu_cs_emit(cs, ZPASS_DONE);
+      /* ZPASS_DONE events should come in begin-end pairs. When emitting and
+       * occlusion query outside of a renderpass, we emit a fake end event that
+       * closes the previous one since the autotuner's ZPASS_DONE use could end
+       * up causing problems. This events writes into the end field of the query
+       * slot, but it will be overwritten by events in emit_end_occlusion_query
+       * with the proper value.
+       * When inside a renderpass, the corresponding ZPASS_DONE event will be
+       * emitted in emit_end_occlusion_query. We note the use of ZPASS_DONE on
+       * the state object, enabling autotuner to optimize its own events.
+       */
+      if (!cmdbuf->state.pass) {
+         tu_cs_emit_pkt7(cs, CP_EVENT_WRITE7, 3);
+         tu_cs_emit(cs, CP_EVENT_WRITE7_0(.event = ZPASS_DONE,
+                                          .write_sample_count = true,
+                                          .sample_count_end_offset = true,
+                                          .write_accum_sample_count_diff = true).value);
+         tu_cs_emit_qw(cs, begin_iova);
+      } else {
+         cmdbuf->state.rp.has_zpass_done_sample_count_write_in_rp = true;
+      }
+   }
 }
 
+template <chip CHIP>
 static void
 emit_begin_stat_query(struct tu_cmd_buffer *cmdbuf,
                       struct tu_query_pool *pool,
@@ -868,7 +926,7 @@ emit_begin_stat_query(struct tu_cmd_buffer *cmdbuf,
                         CP_COND_REG_EXEC_0_BINNING);
       }
 
-      tu6_emit_event_write(cmdbuf, cs, START_PRIMITIVE_CTRS);
+      tu_emit_event_write<CHIP>(cmdbuf, cs, FD_START_PRIMITIVE_CTRS);
 
       tu_cs_emit_pkt7(cs, CP_MEM_WRITE, 3);
       tu_cs_emit_qw(cs, global_iova(cmdbuf, vtx_stats_query_not_running));
@@ -880,11 +938,11 @@ emit_begin_stat_query(struct tu_cmd_buffer *cmdbuf,
    }
 
    if (is_pipeline_query_with_fragment_stage(pool->pipeline_statistics)) {
-      tu6_emit_event_write(cmdbuf, cs, START_FRAGMENT_CTRS);
+      tu_emit_event_write<CHIP>(cmdbuf, cs, FD_START_FRAGMENT_CTRS);
    }
 
    if (is_pipeline_query_with_compute_stage(pool->pipeline_statistics)) {
-      tu6_emit_event_write(cmdbuf, cs, START_COMPUTE_CTRS);
+      tu_emit_event_write<CHIP>(cmdbuf, cs, FD_START_COMPUTE_CTRS);
    }
 
    tu_cs_emit_wfi(cs);
@@ -985,6 +1043,7 @@ emit_begin_perf_query(struct tu_cmd_buffer *cmdbuf,
    tu_cond_exec_end(cs);
 }
 
+template <chip CHIP>
 static void
 emit_begin_xfb_query(struct tu_cmd_buffer *cmdbuf,
                      struct tu_query_pool *pool,
@@ -995,9 +1054,10 @@ emit_begin_xfb_query(struct tu_cmd_buffer *cmdbuf,
    uint64_t begin_iova = primitive_query_iova(pool, query, begin, 0, 0);
 
    tu_cs_emit_regs(cs, A6XX_VPC_SO_STREAM_COUNTS(.qword = begin_iova));
-   tu6_emit_event_write(cmdbuf, cs, WRITE_PRIMITIVE_COUNTS);
+   tu_emit_event_write<CHIP>(cmdbuf, cs, FD_WRITE_PRIMITIVE_COUNTS);
 }
 
+template <chip CHIP>
 static void
 emit_begin_prim_generated_query(struct tu_cmd_buffer *cmdbuf,
                                 struct tu_query_pool *pool,
@@ -1023,7 +1083,7 @@ emit_begin_prim_generated_query(struct tu_cmd_buffer *cmdbuf,
                            CP_COND_REG_EXEC_0_BINNING);
    }
 
-   tu6_emit_event_write(cmdbuf, cs, START_PRIMITIVE_CTRS);
+   tu_emit_event_write<CHIP>(cmdbuf, cs, FD_START_PRIMITIVE_CTRS);
 
    tu_cs_emit_wfi(cs);
 
@@ -1038,14 +1098,15 @@ emit_begin_prim_generated_query(struct tu_cmd_buffer *cmdbuf,
    }
 }
 
+template <chip CHIP>
 VKAPI_ATTR void VKAPI_CALL
 tu_CmdBeginQuery(VkCommandBuffer commandBuffer,
                  VkQueryPool queryPool,
                  uint32_t query,
                  VkQueryControlFlags flags)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
    assert(query < pool->size);
 
    switch (pool->type) {
@@ -1054,19 +1115,19 @@ tu_CmdBeginQuery(VkCommandBuffer commandBuffer,
        * GL_SAMPLES_PASSED and GL_ANY_SAMPLES_PASSED, so we can similarly
        * ignore the VK_QUERY_CONTROL_PRECISE_BIT flag here.
        */
-      emit_begin_occlusion_query(cmdbuf, pool, query);
+      emit_begin_occlusion_query<CHIP>(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT:
-      emit_begin_xfb_query(cmdbuf, pool, query, 0);
+      emit_begin_xfb_query<CHIP>(cmdbuf, pool, query, 0);
       break;
    case VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT:
-      emit_begin_prim_generated_query(cmdbuf, pool, query);
+      emit_begin_prim_generated_query<CHIP>(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR:
       emit_begin_perf_query(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_PIPELINE_STATISTICS:
-      emit_begin_stat_query(cmdbuf, pool, query);
+      emit_begin_stat_query<CHIP>(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_TIMESTAMP:
       unreachable("Unimplemented query type");
@@ -1074,7 +1135,9 @@ tu_CmdBeginQuery(VkCommandBuffer commandBuffer,
       assert(!"Invalid query type");
    }
 }
+TU_GENX(tu_CmdBeginQuery);
 
+template <chip CHIP>
 VKAPI_ATTR void VKAPI_CALL
 tu_CmdBeginQueryIndexedEXT(VkCommandBuffer commandBuffer,
                            VkQueryPool queryPool,
@@ -1082,22 +1145,24 @@ tu_CmdBeginQueryIndexedEXT(VkCommandBuffer commandBuffer,
                            VkQueryControlFlags flags,
                            uint32_t index)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
    assert(query < pool->size);
 
    switch (pool->type) {
    case VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT:
-      emit_begin_xfb_query(cmdbuf, pool, query, index);
+      emit_begin_xfb_query<CHIP>(cmdbuf, pool, query, index);
       break;
    case VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT:
-      emit_begin_prim_generated_query(cmdbuf, pool, query);
+      emit_begin_prim_generated_query<CHIP>(cmdbuf, pool, query);
       break;
    default:
       assert(!"Invalid query type");
    }
 }
+TU_GENX(tu_CmdBeginQueryIndexedEXT);
 
+template <chip CHIP>
 static void
 emit_end_occlusion_query(struct tu_cmd_buffer *cmdbuf,
                          struct tu_query_pool *pool,
@@ -1122,8 +1187,8 @@ emit_end_occlusion_query(struct tu_cmd_buffer *cmdbuf,
 
    uint64_t available_iova = query_available_iova(pool, query);
    uint64_t begin_iova = occlusion_query_iova(pool, query, begin);
+   uint64_t result_iova = occlusion_query_iova(pool, query, result);
    uint64_t end_iova = occlusion_query_iova(pool, query, end);
-   uint64_t result_iova = query_result_iova(pool, query, uint64_t, 0);
    tu_cs_emit_pkt7(cs, CP_MEM_WRITE, 4);
    tu_cs_emit_qw(cs, end_iova);
    tu_cs_emit_qw(cs, 0xffffffffffffffffull);
@@ -1133,11 +1198,38 @@ emit_end_occlusion_query(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit_regs(cs,
                    A6XX_RB_SAMPLE_COUNT_CONTROL(.copy = true));
 
-   tu_cs_emit_regs(cs,
-                   A6XX_RB_SAMPLE_COUNT_ADDR(.qword = end_iova));
+   if (!cmdbuf->device->physical_device->info->a7xx.has_event_write_sample_count) {
+      tu_cs_emit_regs(cs,
+                        A6XX_RB_SAMPLE_COUNT_ADDR(.qword = end_iova));
+      tu_cs_emit_pkt7(cs, CP_EVENT_WRITE, 1);
+      tu_cs_emit(cs, ZPASS_DONE);
+      if (CHIP == A7XX) {
+         /* Copied from blob's cmdstream, not sure why it is done. */
+         tu_cs_emit_pkt7(cs, CP_EVENT_WRITE, 1);
+         tu_cs_emit(cs, CCU_CLEAN_DEPTH);
+      }
+   } else {
+      /* When outside of renderpass, potential autotuner activity can cause
+       * interference between ZPASS_DONE event pairs. In that case, like at the
+       * beginning of the occlusion query, a fake ZPASS_DONE event is emitted to
+       * compose a begin-end event pair. The first event will write into the end
+       * field, but that will be overwritten by the second ZPASS_DONE which will
+       * also handle the diff accumulation.
+       */
+      if (!cmdbuf->state.pass) {
+         tu_cs_emit_pkt7(cs, CP_EVENT_WRITE7, 3);
+         tu_cs_emit(cs, CP_EVENT_WRITE7_0(.event = ZPASS_DONE,
+                                          .write_sample_count = true).value);
+         tu_cs_emit_qw(cs, end_iova);
+      }
 
-   tu_cs_emit_pkt7(cs, CP_EVENT_WRITE, 1);
-   tu_cs_emit(cs, ZPASS_DONE);
+      tu_cs_emit_pkt7(cs, CP_EVENT_WRITE7, 3);
+      tu_cs_emit(cs, CP_EVENT_WRITE7_0(.event = ZPASS_DONE,
+                                       .write_sample_count = true,
+                                       .sample_count_end_offset = true,
+                                       .write_accum_sample_count_diff = true).value);
+      tu_cs_emit_qw(cs, begin_iova);
+   }
 
    tu_cs_emit_pkt7(cs, CP_WAIT_REG_MEM, 6);
    tu_cs_emit(cs, CP_WAIT_REG_MEM_0_FUNCTION(WRITE_NE) |
@@ -1147,13 +1239,15 @@ emit_end_occlusion_query(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit(cs, CP_WAIT_REG_MEM_4_MASK(~0));
    tu_cs_emit(cs, CP_WAIT_REG_MEM_5_DELAY_LOOP_CYCLES(16));
 
-   /* result (dst) = result (srcA) + end (srcB) - begin (srcC) */
-   tu_cs_emit_pkt7(cs, CP_MEM_TO_MEM, 9);
-   tu_cs_emit(cs, CP_MEM_TO_MEM_0_DOUBLE | CP_MEM_TO_MEM_0_NEG_C);
-   tu_cs_emit_qw(cs, result_iova);
-   tu_cs_emit_qw(cs, result_iova);
-   tu_cs_emit_qw(cs, end_iova);
-   tu_cs_emit_qw(cs, begin_iova);
+   if (!cmdbuf->device->physical_device->info->a7xx.has_event_write_sample_count) {
+      /* result (dst) = result (srcA) + end (srcB) - begin (srcC) */
+      tu_cs_emit_pkt7(cs, CP_MEM_TO_MEM, 9);
+      tu_cs_emit(cs, CP_MEM_TO_MEM_0_DOUBLE | CP_MEM_TO_MEM_0_NEG_C);
+      tu_cs_emit_qw(cs, result_iova);
+      tu_cs_emit_qw(cs, result_iova);
+      tu_cs_emit_qw(cs, end_iova);
+      tu_cs_emit_qw(cs, begin_iova);
+   }
 
    tu_cs_emit_pkt7(cs, CP_WAIT_MEM_WRITES, 0);
 
@@ -1181,6 +1275,7 @@ emit_end_occlusion_query(struct tu_cmd_buffer *cmdbuf,
  * query inside of secondary cmd buffer - for such case we ought to track
  * the status of pipeline stats query.
  */
+template <chip CHIP>
 static void
 emit_stop_primitive_ctrs(struct tu_cmd_buffer *cmdbuf,
                          struct tu_cs *cs,
@@ -1195,7 +1290,7 @@ emit_stop_primitive_ctrs(struct tu_cmd_buffer *cmdbuf,
          is_pipeline_query_with_vertex_stage(cmdbuf->inherited_pipeline_statistics);
 
       if (!need_cond_exec) {
-         tu6_emit_event_write(cmdbuf, cs, STOP_PRIMITIVE_CTRS);
+         tu_emit_event_write<CHIP>(cmdbuf, cs, FD_STOP_PRIMITIVE_CTRS);
       } else {
          tu_cs_reserve(cs, 7 + 2);
          /* Check that pipeline stats query is not running, only then
@@ -1207,7 +1302,7 @@ emit_stop_primitive_ctrs(struct tu_cmd_buffer *cmdbuf,
          tu_cs_emit(cs, CP_COND_EXEC_4_REF(0x2));
          tu_cs_emit(cs, 2); /* Cond execute the next 2 DWORDS */
 
-         tu6_emit_event_write(cmdbuf, cs, STOP_PRIMITIVE_CTRS);
+         tu_emit_event_write<CHIP>(cmdbuf, cs, FD_STOP_PRIMITIVE_CTRS);
       }
    }
 
@@ -1218,6 +1313,7 @@ emit_stop_primitive_ctrs(struct tu_cmd_buffer *cmdbuf,
    }
 }
 
+template <chip CHIP>
 static void
 emit_end_stat_query(struct tu_cmd_buffer *cmdbuf,
                     struct tu_query_pool *pool,
@@ -1235,15 +1331,15 @@ emit_end_stat_query(struct tu_cmd_buffer *cmdbuf,
        * we are inside VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT inside of a
        * renderpass, because it is already stopped.
        */
-      emit_stop_primitive_ctrs(cmdbuf, cs, VK_QUERY_TYPE_PIPELINE_STATISTICS);
+      emit_stop_primitive_ctrs<CHIP>(cmdbuf, cs, VK_QUERY_TYPE_PIPELINE_STATISTICS);
    }
 
    if (is_pipeline_query_with_fragment_stage(pool->pipeline_statistics)) {
-      tu6_emit_event_write(cmdbuf, cs, STOP_FRAGMENT_CTRS);
+      tu_emit_event_write<CHIP>(cmdbuf, cs, FD_STOP_FRAGMENT_CTRS);
    }
 
    if (is_pipeline_query_with_compute_stage(pool->pipeline_statistics)) {
-      tu6_emit_event_write(cmdbuf, cs, STOP_COMPUTE_CTRS);
+      tu_emit_event_write<CHIP>(cmdbuf, cs, FD_STOP_COMPUTE_CTRS);
    }
 
    tu_cs_emit_wfi(cs);
@@ -1360,6 +1456,7 @@ emit_end_perf_query(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit_qw(cs, 0x1);
 }
 
+template <chip CHIP>
 static void
 emit_end_xfb_query(struct tu_cmd_buffer *cmdbuf,
                    struct tu_query_pool *pool,
@@ -1378,10 +1475,10 @@ emit_end_xfb_query(struct tu_cmd_buffer *cmdbuf,
    uint64_t available_iova = query_available_iova(pool, query);
 
    tu_cs_emit_regs(cs, A6XX_VPC_SO_STREAM_COUNTS(.qword = end_iova));
-   tu6_emit_event_write(cmdbuf, cs, WRITE_PRIMITIVE_COUNTS);
+   tu_emit_event_write<CHIP>(cmdbuf, cs, FD_WRITE_PRIMITIVE_COUNTS);
 
    tu_cs_emit_wfi(cs);
-   tu6_emit_event_write(cmdbuf, cs, CACHE_FLUSH_TS);
+   tu_emit_event_write<CHIP>(cmdbuf, cs, FD_CACHE_FLUSH);
 
    /* Set the count of written primitives */
    tu_cs_emit_pkt7(cs, CP_MEM_TO_MEM, 9);
@@ -1392,7 +1489,7 @@ emit_end_xfb_query(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit_qw(cs, end_written_iova);
    tu_cs_emit_qw(cs, begin_written_iova);
 
-   tu6_emit_event_write(cmdbuf, cs, CACHE_FLUSH_TS);
+   tu_emit_event_write<CHIP>(cmdbuf, cs, FD_CACHE_FLUSH);
 
    /* Set the count of generated primitives */
    tu_cs_emit_pkt7(cs, CP_MEM_TO_MEM, 9);
@@ -1409,6 +1506,7 @@ emit_end_xfb_query(struct tu_cmd_buffer *cmdbuf,
    tu_cs_emit_qw(cs, 0x1);
 }
 
+template <chip CHIP>
 static void
 emit_end_prim_generated_query(struct tu_cmd_buffer *cmdbuf,
                               struct tu_query_pool *pool,
@@ -1452,7 +1550,7 @@ emit_end_prim_generated_query(struct tu_cmd_buffer *cmdbuf,
    /* Should be after waiting for mem writes to have up to date info
     * about which query is running.
     */
-   emit_stop_primitive_ctrs(cmdbuf, cs, VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT);
+   emit_stop_primitive_ctrs<CHIP>(cmdbuf, cs, VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT);
 
    if (cmdbuf->state.pass) {
       tu_cond_exec_end(cs);
@@ -1505,30 +1603,31 @@ handle_multiview_queries(struct tu_cmd_buffer *cmd,
    }
 }
 
+template <chip CHIP>
 VKAPI_ATTR void VKAPI_CALL
 tu_CmdEndQuery(VkCommandBuffer commandBuffer,
                VkQueryPool queryPool,
                uint32_t query)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
    assert(query < pool->size);
 
    switch (pool->type) {
    case VK_QUERY_TYPE_OCCLUSION:
-      emit_end_occlusion_query(cmdbuf, pool, query);
+      emit_end_occlusion_query<CHIP>(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT:
-      emit_end_xfb_query(cmdbuf, pool, query, 0);
+      emit_end_xfb_query<CHIP>(cmdbuf, pool, query, 0);
       break;
    case VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT:
-      emit_end_prim_generated_query(cmdbuf, pool, query);
+      emit_end_prim_generated_query<CHIP>(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR:
       emit_end_perf_query(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_PIPELINE_STATISTICS:
-      emit_end_stat_query(cmdbuf, pool, query);
+      emit_end_stat_query<CHIP>(cmdbuf, pool, query);
       break;
    case VK_QUERY_TYPE_TIMESTAMP:
       unreachable("Unimplemented query type");
@@ -1538,29 +1637,32 @@ tu_CmdEndQuery(VkCommandBuffer commandBuffer,
 
    handle_multiview_queries(cmdbuf, pool, query);
 }
+TU_GENX(tu_CmdEndQuery);
 
+template <chip CHIP>
 VKAPI_ATTR void VKAPI_CALL
 tu_CmdEndQueryIndexedEXT(VkCommandBuffer commandBuffer,
                          VkQueryPool queryPool,
                          uint32_t query,
                          uint32_t index)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmdbuf, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
    assert(query < pool->size);
 
    switch (pool->type) {
    case VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT:
       assert(index <= 4);
-      emit_end_xfb_query(cmdbuf, pool, query, index);
+      emit_end_xfb_query<CHIP>(cmdbuf, pool, query, index);
       break;
    case VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT:
-      emit_end_prim_generated_query(cmdbuf, pool, query);
+      emit_end_prim_generated_query<CHIP>(cmdbuf, pool, query);
       break;
    default:
       assert(!"Invalid query type");
    }
 }
+TU_GENX(tu_CmdEndQueryIndexedEXT);
 
 VKAPI_ATTR void VKAPI_CALL
 tu_CmdWriteTimestamp2(VkCommandBuffer commandBuffer,
@@ -1568,8 +1670,8 @@ tu_CmdWriteTimestamp2(VkCommandBuffer commandBuffer,
                       VkQueryPool queryPool,
                       uint32_t query)
 {
-   TU_FROM_HANDLE(tu_cmd_buffer, cmd, commandBuffer);
-   TU_FROM_HANDLE(tu_query_pool, pool, queryPool);
+   VK_FROM_HANDLE(tu_cmd_buffer, cmd, commandBuffer);
+   VK_FROM_HANDLE(tu_query_pool, pool, queryPool);
 
    /* Inside a render pass, just write the timestamp multiple times so that
     * the user gets the last one if we use GMEM. There isn't really much
@@ -1648,7 +1750,7 @@ tu_EnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR(
     VkPerformanceCounterKHR*                    pCounters,
     VkPerformanceCounterDescriptionKHR*         pCounterDescriptions)
 {
-   TU_FROM_HANDLE(tu_physical_device, phydev, physicalDevice);
+   VK_FROM_HANDLE(tu_physical_device, phydev, physicalDevice);
 
    uint32_t desc_count = *pCounterCount;
    uint32_t group_count;
@@ -1698,7 +1800,7 @@ tu_GetPhysicalDeviceQueueFamilyPerformanceQueryPassesKHR(
       const VkQueryPoolPerformanceCreateInfoKHR*  pPerformanceQueryCreateInfo,
       uint32_t*                                   pNumPasses)
 {
-   TU_FROM_HANDLE(tu_physical_device, phydev, physicalDevice);
+   VK_FROM_HANDLE(tu_physical_device, phydev, physicalDevice);
    uint32_t group_count = 0;
    uint32_t gid = 0, cid = 0, n_passes;
    const struct fd_perfcntr_group *group =
