@@ -43,7 +43,7 @@
 #include "gallivm/lp_bld_sample.h"
 #include "gallivm/lp_bld_ir_common.h"
 #include "lp_bld_type.h"
-#include "pipe/p_compiler.h"
+#include "util/compiler.h"
 #include "pipe/p_state.h"
 #include "tgsi/tgsi_exec.h"
 #include "tgsi/tgsi_scan.h"
@@ -55,8 +55,6 @@ extern "C" {
 
 #define LP_CHAN_ALL ~0u
 
-#define LP_MAX_INSTRUCTIONS 256
-
 struct tgsi_full_declaration;
 struct tgsi_full_immediate;
 struct tgsi_full_instruction;
@@ -174,17 +172,18 @@ struct lp_bld_tgsi_system_values {
    LLVMValueRef firstvertex;
    LLVMValueRef invocation_id;
    LLVMValueRef draw_id;
-   LLVMValueRef thread_id;
-   LLVMValueRef block_id;
-   LLVMValueRef grid_size;
+   LLVMValueRef thread_id[3];
+   LLVMValueRef block_id[3];
+   LLVMValueRef grid_size[3];
    LLVMValueRef front_facing;
    LLVMValueRef work_dim;
-   LLVMValueRef block_size;
+   LLVMValueRef block_size[3];
    LLVMValueRef tess_coord;
    LLVMValueRef tess_outer;
    LLVMValueRef tess_inner;
    LLVMValueRef vertices_in;
    LLVMValueRef sample_id;
+   LLVMTypeRef sample_pos_type;
    LLVMValueRef sample_pos;
    LLVMValueRef sample_mask_in;
    LLVMValueRef view_index;
@@ -203,49 +202,43 @@ struct lp_bld_tgsi_system_values {
  */
 struct lp_build_sampler_soa
 {
-   void
-   (*destroy)( struct lp_build_sampler_soa *sampler );
-
    void
    (*emit_tex_sample)(const struct lp_build_sampler_soa *sampler,
                       struct gallivm_state *gallivm,
                       const struct lp_sampler_params *params);
 
    void
-   (*emit_size_query)( const struct lp_build_sampler_soa *sampler,
-                       struct gallivm_state *gallivm,
-                       const struct lp_sampler_size_query_params *params);
+   (*emit_size_query)(const struct lp_build_sampler_soa *sampler,
+                      struct gallivm_state *gallivm,
+                      const struct lp_sampler_size_query_params *params);
 };
 
 
 struct lp_build_sampler_aos
 {
    LLVMValueRef
-   (*emit_fetch_texel)( const struct lp_build_sampler_aos *sampler,
-                        struct lp_build_context *bld,
-                        unsigned target, /* TGSI_TEXTURE_* */
-                        unsigned unit,
-                        LLVMValueRef coords,
-                        const struct lp_derivatives derivs,
-                        enum lp_build_tex_modifier modifier);
+   (*emit_fetch_texel)(const struct lp_build_sampler_aos *sampler,
+                       struct lp_build_context *bld,
+                       enum tgsi_texture_type target,
+                       unsigned unit,
+                       LLVMValueRef coords,
+                       const struct lp_derivatives derivs,
+                       enum lp_build_tex_modifier modifier);
 };
 
 struct lp_img_params;
 
 struct lp_build_image_soa
 {
-   void
-   (*destroy)( struct lp_build_image_soa *image );
-
    void
    (*emit_op)(const struct lp_build_image_soa *image,
               struct gallivm_state *gallivm,
               const struct lp_img_params *params);
 
    void
-   (*emit_size_query)( const struct lp_build_image_soa *sampler,
-                       struct gallivm_state *gallivm,
-                       const struct lp_sampler_size_query_params *params);
+   (*emit_size_query)(const struct lp_build_image_soa *sampler,
+                      struct gallivm_state *gallivm,
+                      const struct lp_sampler_size_query_params *params);
 };
 
 struct lp_build_fs_iface;
@@ -274,21 +267,33 @@ struct lp_build_tgsi_params {
    LLVMValueRef const_sizes_ptr;
    const struct lp_bld_tgsi_system_values *system_values;
    const LLVMValueRef (*inputs)[4];
+   int num_inputs;
+   LLVMTypeRef context_type;
    LLVMValueRef context_ptr;
+   LLVMTypeRef resources_type;
+   LLVMValueRef resources_ptr;
+   LLVMTypeRef thread_data_type;
    LLVMValueRef thread_data_ptr;
    const struct lp_build_sampler_soa *sampler;
    const struct tgsi_shader_info *info;
    const struct lp_build_gs_iface *gs_iface;
    const struct lp_build_tcs_iface *tcs_iface;
    const struct lp_build_tes_iface *tes_iface;
+   const struct lp_build_mesh_iface *mesh_iface;
    LLVMValueRef ssbo_ptr;
    LLVMValueRef ssbo_sizes_ptr;
    const struct lp_build_image_soa *image;
    LLVMValueRef shared_ptr;
+   LLVMValueRef payload_ptr;
    const struct lp_build_coro_suspend_info *coro;
    LLVMValueRef kernel_args;
    const struct lp_build_fs_iface *fs_iface;
    unsigned gs_vertex_streams;
+   LLVMValueRef aniso_filter_table;
+   LLVMValueRef current_func;
+   struct hash_table *fns;
+   LLVMValueRef scratch_ptr;
+   LLVMValueRef call_context_ptr;
 };
 
 void
@@ -297,23 +302,11 @@ lp_build_tgsi_soa(struct gallivm_state *gallivm,
                   const struct lp_build_tgsi_params *params,
                   LLVMValueRef (*outputs)[4]);
 
-void
-lp_build_tgsi_aos(struct gallivm_state *gallivm,
-                  const struct tgsi_token *tokens,
-                  struct lp_type type,
-                  const unsigned char swizzles[4],
-                  LLVMValueRef consts_ptr,
-                  const LLVMValueRef *inputs,
-                  LLVMValueRef *outputs,
-                  const struct lp_build_sampler_aos *sampler,
-                  const struct tgsi_shader_info *info);
-
-
 struct lp_build_tgsi_inst_list
 {
    struct tgsi_full_instruction *instructions;
-   uint max_instructions;
-   uint num_instructions;
+   unsigned max_instructions;
+   unsigned num_instructions;
 };
 
 unsigned lp_bld_tgsi_list_init(struct lp_build_tgsi_context * bld_base);
@@ -396,13 +389,13 @@ struct lp_build_tgsi_context
     * to every function. */
    void * userdata;
 
-   boolean soa;
+   bool soa;
 
    int pc;
 
    struct tgsi_full_instruction *instructions;
-   uint max_instructions;
-   uint num_instructions;
+   unsigned max_instructions;
+   unsigned num_instructions;
 
    /** This function allows the user to insert some instructions at the
      * beginning of the program.  It is optional and does not need to be
@@ -429,9 +422,9 @@ struct lp_build_gs_iface
 {
    LLVMValueRef (*fetch_input)(const struct lp_build_gs_iface *gs_iface,
                                struct lp_build_context * bld,
-                               boolean is_vindex_indirect,
+                               bool is_vindex_indirect,
                                LLVMValueRef vertex_index,
-                               boolean is_aindex_indirect,
+                               bool is_aindex_indirect,
                                LLVMValueRef attrib_index,
                                LLVMValueRef swizzle_index);
    void (*emit_vertex)(const struct lp_build_gs_iface *gs_iface,
@@ -459,31 +452,31 @@ struct lp_build_tcs_iface
    void (*emit_store_output)(const struct lp_build_tcs_iface *tcs_iface,
                              struct lp_build_context * bld,
                              unsigned name,
-                             boolean is_vindex_indirect,
+                             bool is_vindex_indirect,
                              LLVMValueRef vertex_index,
-                             boolean is_aindex_indirect,
+                             bool is_aindex_indirect,
                              LLVMValueRef attrib_index,
-                             boolean is_sindex_indirect,
+                             bool is_sindex_indirect,
                              LLVMValueRef swizzle_index,
                              LLVMValueRef value,
                              LLVMValueRef mask_vec);
 
    LLVMValueRef (*emit_fetch_input)(const struct lp_build_tcs_iface *tcs_iface,
                                     struct lp_build_context * bld,
-                                    boolean is_vindex_indirect,
+                                    bool is_vindex_indirect,
                                     LLVMValueRef vertex_index,
-                                    boolean is_aindex_indirect,
+                                    bool is_aindex_indirect,
                                     LLVMValueRef attrib_index,
-                                    boolean is_sindex_indirect,
+                                    bool is_sindex_indirect,
                                     LLVMValueRef swizzle_index);
 
    LLVMValueRef (*emit_fetch_output)(const struct lp_build_tcs_iface *tcs_iface,
                                     struct lp_build_context * bld,
-                                    boolean is_vindex_indirect,
+                                    bool is_vindex_indirect,
                                     LLVMValueRef vertex_index,
-                                    boolean is_aindex_indirect,
+                                    bool is_aindex_indirect,
                                     LLVMValueRef attrib_index,
-                                    boolean is_sindex_indirect,
+                                    bool is_sindex_indirect,
                                     LLVMValueRef swizzle_index,
                                     uint32_t name);
 };
@@ -492,20 +485,39 @@ struct lp_build_tes_iface
 {
    LLVMValueRef (*fetch_vertex_input)(const struct lp_build_tes_iface *tes_iface,
                                       struct lp_build_context * bld,
-                                      boolean is_vindex_indirect,
+                                      bool is_vindex_indirect,
                                       LLVMValueRef vertex_index,
-                                      boolean is_aindex_indirect,
+                                      bool is_aindex_indirect,
                                       LLVMValueRef attrib_index,
-                                      boolean is_sindex_indirect,
+                                      bool is_sindex_indirect,
                                       LLVMValueRef swizzle_index);
 
    LLVMValueRef (*fetch_patch_input)(const struct lp_build_tes_iface *tes_iface,
                                      struct lp_build_context * bld,
-                                     boolean is_aindex_indirect,
+                                     bool is_aindex_indirect,
                                      LLVMValueRef attrib_index,
                                      LLVMValueRef swizzle_index);
 };
 
+struct lp_build_mesh_iface
+{
+   void (*emit_store_output)(const struct lp_build_mesh_iface *mesh_iface,
+                             struct lp_build_context * bld,
+                             unsigned name,
+                             bool is_vindex_indirect,
+                             LLVMValueRef vertex_index,
+                             bool is_aindex_indirect,
+                             LLVMValueRef attrib_index,
+                             bool is_sindex_indirect,
+                             LLVMValueRef swizzle_index,
+                             LLVMValueRef value,
+                             LLVMValueRef mask_vec);
+   void (*emit_vertex_and_primitive_count)(const struct lp_build_mesh_iface *mesh_iface,
+                                           struct lp_build_context *bld,
+                                           LLVMValueRef vertices_count,
+                                           LLVMValueRef primitives_count);
+};
+
 struct lp_build_tgsi_soa_context
 {
    struct lp_build_tgsi_context bld_base;
@@ -523,16 +535,18 @@ struct lp_build_tgsi_soa_context
    LLVMValueRef max_output_vertices_vec;
 
    LLVMValueRef consts_ptr;
-   LLVMValueRef const_sizes_ptr;
    LLVMValueRef consts[LP_MAX_TGSI_CONST_BUFFERS];
    LLVMValueRef consts_sizes[LP_MAX_TGSI_CONST_BUFFERS];
    const LLVMValueRef (*inputs)[TGSI_NUM_CHANNELS];
    LLVMValueRef (*outputs)[TGSI_NUM_CHANNELS];
+   LLVMTypeRef context_type;
    LLVMValueRef context_ptr;
+   LLVMTypeRef resources_type;
+   LLVMValueRef resources_ptr;
+   LLVMTypeRef thread_data_type;
    LLVMValueRef thread_data_ptr;
 
    LLVMValueRef ssbo_ptr;
-   LLVMValueRef ssbo_sizes_ptr;
    LLVMValueRef ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
    LLVMValueRef ssbo_sizes[LP_MAX_TGSI_SHADER_BUFFERS];
 
@@ -553,12 +567,14 @@ struct lp_build_tgsi_soa_context
     * set in the indirect_files field.
     * The temps[] array above is unused then.
     */
+   LLVMTypeRef temps_array_type;
    LLVMValueRef temps_array;
 
    /* We allocate/use this array of output if (1 << TGSI_FILE_OUTPUT) is
     * set in the indirect_files field.
     * The outputs[] array above is unused then.
     */
+   LLVMTypeRef outputs_array_type;
    LLVMValueRef outputs_array;
 
    /* We allocate/use this array of inputs if (1 << TGSI_FILE_INPUT) is
@@ -581,8 +597,8 @@ struct lp_build_tgsi_soa_context
    struct lp_build_mask_context *mask;
    struct lp_exec_mask exec_mask;
 
-   uint num_immediates;
-   boolean use_immediates_array;
+   unsigned num_immediates;
+   bool use_immediates_array;
 };
 
 void
@@ -594,7 +610,7 @@ void lp_emit_immediate_soa(
    struct lp_build_tgsi_context *bld_base,
    const struct tgsi_full_immediate *imm);
 
-boolean
+bool
 lp_emit_instruction_soa(
    struct lp_build_tgsi_soa_context *bld,
    const struct tgsi_full_instruction *inst,
@@ -613,87 +629,16 @@ lp_get_output_ptr(
    unsigned index,
    unsigned chan);
 
-struct lp_build_tgsi_aos_context
-{
-   struct lp_build_tgsi_context bld_base;
-
-   /* Builder for integer masks and indices */
-   struct lp_build_context int_bld;
-
-   /*
-    * AoS swizzle used:
-    * - swizzles[0] = red index
-    * - swizzles[1] = green index
-    * - swizzles[2] = blue index
-    * - swizzles[3] = alpha index
-    */
-   unsigned char swizzles[4];
-   unsigned char inv_swizzles[4];
-
-   LLVMValueRef consts_ptr;
-   const LLVMValueRef *inputs;
-   LLVMValueRef *outputs;
-
-   const struct lp_build_sampler_aos *sampler;
-
-   struct tgsi_declaration_sampler_view sv[PIPE_MAX_SHADER_SAMPLER_VIEWS];
-
-   LLVMValueRef immediates[LP_MAX_INLINED_IMMEDIATES];
-   LLVMValueRef temps[LP_MAX_INLINED_TEMPS];
-   LLVMValueRef addr[LP_MAX_TGSI_ADDRS];
-
-   /* We allocate/use this array of temps if (1 << TGSI_FILE_TEMPORARY) is
-    * set in the indirect_files field.
-    * The temps[] array above is unused then.
-    */
-   LLVMValueRef temps_array;
-
-   /** bitmask indicating which register files are accessed indirectly */
-   unsigned indirect_files;
-
-};
-
 static inline struct lp_build_tgsi_soa_context *
 lp_soa_context(struct lp_build_tgsi_context *bld_base)
 {
    return (struct lp_build_tgsi_soa_context *)bld_base;
 }
 
-static inline struct lp_build_tgsi_aos_context *
-lp_aos_context(struct lp_build_tgsi_context *bld_base)
-{
-   return (struct lp_build_tgsi_aos_context *)bld_base;
-}
-
-void
-lp_emit_declaration_aos(
-   struct lp_build_tgsi_aos_context *bld,
-   const struct tgsi_full_declaration *decl);
-
-
-boolean
-lp_emit_instruction_aos(
-   struct lp_build_tgsi_aos_context *bld,
-   const struct tgsi_full_instruction *inst,
-   const struct tgsi_opcode_info *info,
-   int *pc);
-
-void
-lp_emit_store_aos(
-   struct lp_build_tgsi_aos_context *bld,
-   const struct tgsi_full_instruction *inst,
-   unsigned index,
-   LLVMValueRef value);
-
 void lp_build_fetch_args(
    struct lp_build_tgsi_context * bld_base,
    struct lp_build_emit_data * emit_data);
 
-LLVMValueRef
-lp_build_tgsi_inst_llvm_aos(
-   struct lp_build_tgsi_context * bld_base,
-   const struct tgsi_full_instruction *inst);
-
 void
 lp_build_tgsi_intrinsic(
  const struct lp_build_tgsi_action * action,
@@ -727,7 +672,7 @@ lp_build_emit_llvm_ternary(
    LLVMValueRef arg1,
    LLVMValueRef arg2);
 
-boolean
+bool
 lp_build_tgsi_inst_llvm(
    struct lp_build_tgsi_context * bld_base,
    const struct tgsi_full_instruction *inst);
@@ -754,7 +699,7 @@ lp_build_emit_fetch_texoffset(
    unsigned tex_off_op,
    const unsigned chan_index);
 
-boolean
+bool
 lp_build_tgsi_llvm(
    struct lp_build_tgsi_context * bld_base,
    const struct tgsi_token *tokens);
