@@ -1,24 +1,7 @@
 /*
  * Copyright 2009 Nicolai Hähnle <nhaehnle@gmail.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * on the rights to use, copy, modify, merge, publish, distribute, sub
- * license, and/or sell copies of the Software, and to permit persons to whom
- * the Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
- * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
- * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
- * USE OR OTHER DEALINGS IN THE SOFTWARE. */
+ * SPDX-License-Identifier: MIT
+ */
 
 #include "radeon_compiler.h"
 
@@ -108,6 +91,25 @@ int rc_if_fail_helper(struct radeon_compiler * c, const char * file, int line, c
 	return 1;
 }
 
+void rc_mark_unused_channels(struct radeon_compiler * c, void *user)
+{
+	unsigned int srcmasks[3];
+
+	for(struct rc_instruction * inst = c->Program.Instructions.Next;
+	    inst != &c->Program.Instructions;
+	    inst = inst->Next) {
+
+		rc_compute_sources_for_writemask(inst, inst->U.I.DstReg.WriteMask, srcmasks);
+
+		for(unsigned int src = 0; src < 3; ++src) {
+			for(unsigned int chan = 0; chan < 4; ++chan) {
+				if (!GET_BIT(srcmasks[src], chan))
+					SET_SWZ(inst->U.I.SrcReg[src].Swizzle, chan, RC_SWIZZLE_UNUSED);
+			}
+		}
+	}
+}
+
 /**
  * Recompute c->Program.InputsRead and c->Program.OutputsWritten
  * based on which inputs and outputs are actually referenced
@@ -357,17 +359,24 @@ void rc_get_stats(struct radeon_compiler *c, struct rc_program_stats *s)
 {
 	struct rc_instruction * tmp;
 	memset(s, 0, sizeof(*s));
+	unsigned ip = 0;
+	int last_begintex = -1;
 
 	for(tmp = c->Program.Instructions.Next; tmp != &c->Program.Instructions;
-							tmp = tmp->Next){
+							tmp = tmp->Next, ip++){
 		const struct rc_opcode_info * info;
 		rc_for_all_reads_mask(tmp, reg_count_callback, s);
 		if (tmp->Type == RC_INSTRUCTION_NORMAL) {
 			info = rc_get_opcode_info(tmp->U.I.Opcode);
-			if (info->Opcode == RC_OPCODE_BEGIN_TEX)
+			if (info->Opcode == RC_OPCODE_BEGIN_TEX) {
+				/* The R5xx docs mention ~30 cycles in section 8.3.1 */
+				s->num_cycles += 30;
+				last_begintex = ip;
 				continue;
-			if (tmp->U.I.PreSub.Opcode != RC_PRESUB_NONE)
-				s->num_presub_ops++;
+			}
+			if (info->Opcode == RC_OPCODE_MAD &&
+				rc_inst_has_three_diff_temp_srcs(tmp))
+				s->num_cycles++;
 		} else {
 			if (tmp->U.P.RGB.Src[RC_PAIR_PRESUB_SRC].Used)
 				s->num_presub_ops++;
@@ -387,6 +396,15 @@ void rc_get_stats(struct radeon_compiler *c, struct rc_program_stats *s)
 				tmp->U.P.Alpha.Omod != RC_OMOD_DISABLE) {
 				s->num_omod_ops++;
 			}
+			if (tmp->U.P.Nop)
+				s->num_cycles++;
+			/* SemWait has effect only on R500, the more instructions we can put
+			 * between the tex block and the first texture semaphore, the better.
+			 */
+			if (tmp->U.P.SemWait && c->is_r500 && last_begintex != -1) {
+				s->num_cycles -= MIN2(30, ip - last_begintex);
+				last_begintex = -1;
+			}
 			info = rc_get_opcode_info(tmp->U.P.RGB.Opcode);
 		}
 		if (info->IsFlowControl) {
@@ -402,6 +420,7 @@ void rc_get_stats(struct radeon_compiler *c, struct rc_program_stats *s)
 		if (info->HasTexture)
 			s->num_tex_insts++;
 		s->num_insts++;
+		s->num_cycles++;
 	}
 	/* Increment here because the reg_count_callback store the max
 	 * temporary reg index in s->nun_temp_regs. */
@@ -418,11 +437,14 @@ static void print_stats(struct radeon_compiler * c)
 	 * only the FS has, because shader-db's report.py wants all shaders to
 	 * have the same set.
 	 */
-	util_debug_message(c->debug, SHADER_INFO, "%s shader: %u inst, %u vinst, %u sinst, %u predicate, %u flowcontrol, %u loops, %u tex, %u presub, %u omod, %u temps, %u consts, %u lits",
+	util_debug_message(c->debug, SHADER_INFO,
+	                   "%s shader: %u inst, %u vinst, %u sinst, %u predicate, %u flowcontrol,"
+	                   "%u loops, %u tex, %u presub, %u omod, %u temps, %u consts, %u lits, %u cycles",
 	                   c->type == RC_VERTEX_PROGRAM ? "VS" : "FS",
 	                   s.num_insts, s.num_rgb_insts, s.num_alpha_insts, s.num_pred_insts,
 	                   s.num_fc_insts, s.num_loops, s.num_tex_insts, s.num_presub_ops,
-	                   s.num_omod_ops, s.num_temp_regs, s.num_consts, s.num_inline_literals);
+	                   s.num_omod_ops, s.num_temp_regs, s.num_consts, s.num_inline_literals,
+	                   s.num_cycles);
 }
 
 static const char *shader_name[RC_NUM_PROGRAM_TYPES] = {
