@@ -1,28 +1,10 @@
 /*
  * Copyright 2010 Jerome Glisse <glisse@freedesktop.org>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * on the rights to use, copy, modify, merge, publish, distribute, sub
- * license, and/or sell copies of the Software, and to permit persons to whom
- * the Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
- * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
- * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
- * USE OR OTHER DEALINGS IN THE SOFTWARE.
- *
  * Authors:
  *      Jerome Glisse
+ * SPDX-License-Identifier: MIT
  */
+
 #ifndef R600_PIPE_H
 #define R600_PIPE_H
 
@@ -75,7 +57,6 @@
 #define R600_MAX_USER_CONST_BUFFERS 15
 #define R600_MAX_DRIVER_CONST_BUFFERS 3
 #define R600_MAX_CONST_BUFFERS (R600_MAX_USER_CONST_BUFFERS + R600_MAX_DRIVER_CONST_BUFFERS)
-#define R600_MAX_HW_CONST_BUFFERS 16
 
 /* start driver buffers after user buffers */
 #define R600_BUFFER_INFO_CONST_BUFFER (R600_MAX_USER_CONST_BUFFERS)
@@ -259,15 +240,6 @@ struct r600_gs_rings_state {
 /* This must start from 16. */
 /* features */
 #define DBG_NO_CP_DMA		(1 << 30)
-/* shader backend */
-#define DBG_NO_SB		(1 << 21)
-#define DBG_SB_DRY_RUN	(1 << 23)
-#define DBG_SB_STAT		(1 << 24)
-#define DBG_SB_DUMP		(1 << 25)
-#define DBG_SB_NO_FALLBACK	(1 << 26)
-#define DBG_SB_DISASM	(1 << 27)
-#define DBG_SB_SAFEMATH	(1 << 28)
-#define DBG_NIR_SB	(1 << 28)
 
 struct r600_screen {
 	struct r600_common_screen	b;
@@ -443,6 +415,8 @@ struct r600_cso_state
 struct r600_fetch_shader {
 	struct r600_resource		*buffer;
 	unsigned			offset;
+	uint32_t                        buffer_mask;
+	unsigned                        strides[PIPE_MAX_ATTRIBS];
 };
 
 struct r600_shader_state {
@@ -595,7 +569,6 @@ struct r600_context {
 	enum mesa_prim		last_rast_prim;
 	unsigned			last_start_instance;
 
-	void				*sb_context;
 	struct r600_isa		*isa;
 	float sample_positions[4 * 16];
 	float tess_state[8];
