@@ -8,16 +8,17 @@
 #include "nir_xfb_info.h"
 #include "si_pipe.h"
 #include "ac_nir.h"
+#include "aco_interface.h"
 
 
 bool si_alu_to_scalar_packed_math_filter(const nir_instr *instr, const void *data)
 {
    if (instr->type == nir_instr_type_alu) {
       nir_alu_instr *alu = nir_instr_as_alu(instr);
+      bool use_aco = (bool)data;
 
-      if (alu->dest.dest.is_ssa &&
-          alu->dest.dest.ssa.bit_size == 16 &&
-          alu->dest.dest.ssa.num_components == 2)
+      if (alu->def.bit_size == 16 && alu->def.num_components == 2 &&
+          (!use_aco || aco_nir_op_supports_packed_math_16bit(alu)))
          return false;
    }
 
@@ -30,7 +31,14 @@ static uint8_t si_vectorize_callback(const nir_instr *instr, const void *data)
       return 0;
 
    nir_alu_instr *alu = nir_instr_as_alu(instr);
-   if (nir_dest_bit_size(alu->dest.dest) == 16) {
+   if (alu->def.bit_size != 16)
+      return 1;
+
+   bool use_aco = (bool)data;
+
+   if (use_aco) {
+      return aco_nir_op_supports_packed_math_16bit(alu) ? 2 : 1;
+   } else {
       switch (alu->op) {
       case nir_op_unpack_32_2x16_split_x:
       case nir_op_unpack_32_2x16_split_y:
@@ -39,8 +47,6 @@ static uint8_t si_vectorize_callback(const nir_instr *instr, const void *data)
          return 2;
       }
    }
-
-   return 1;
 }
 
 static unsigned si_lower_bit_size_callback(const nir_instr *instr, void *data)
@@ -53,7 +59,7 @@ static unsigned si_lower_bit_size_callback(const nir_instr *instr, void *data)
    switch (alu->op) {
    case nir_op_imul_high:
    case nir_op_umul_high:
-      if (nir_dest_bit_size(alu->dest.dest) < 32)
+      if (alu->def.bit_size < 32)
          return 32;
       break;
    default:
@@ -74,7 +80,7 @@ void si_nir_opts(struct si_screen *sscreen, struct nir_shader *nir, bool first)
 
       NIR_PASS(progress, nir, nir_lower_vars_to_ssa);
       NIR_PASS(progress, nir, nir_lower_alu_to_scalar,
-               nir->options->lower_to_scalar_filter, NULL);
+               nir->options->lower_to_scalar_filter, (void *)sscreen->use_aco);
       NIR_PASS(progress, nir, nir_lower_phis_to_scalar, false);
 
       if (first) {
@@ -85,20 +91,19 @@ void si_nir_opts(struct si_screen *sscreen, struct nir_shader *nir, bool first)
       NIR_PASS(progress, nir, nir_opt_copy_prop_vars);
       NIR_PASS(progress, nir, nir_opt_dead_write_vars);
 
-      NIR_PASS(lower_alu_to_scalar, nir, nir_opt_trivial_continues);
+      NIR_PASS(lower_alu_to_scalar, nir, nir_opt_loop);
       /* (Constant) copy propagation is needed for txf with offsets. */
       NIR_PASS(progress, nir, nir_copy_prop);
       NIR_PASS(progress, nir, nir_opt_remove_phis);
       NIR_PASS(progress, nir, nir_opt_dce);
       /* nir_opt_if_optimize_phi_true_false is disabled on LLVM14 (#6976) */
       NIR_PASS(lower_phis_to_scalar, nir, nir_opt_if,
-               nir_opt_if_aggressive_last_continue |
                nir_opt_if_optimize_phi_true_false);
       NIR_PASS(progress, nir, nir_opt_dead_cf);
 
       if (lower_alu_to_scalar) {
          NIR_PASS_V(nir, nir_lower_alu_to_scalar,
-                    nir->options->lower_to_scalar_filter, NULL);
+                    nir->options->lower_to_scalar_filter, (void *)sscreen->use_aco);
       }
       if (lower_phis_to_scalar)
          NIR_PASS_V(nir, nir_lower_phis_to_scalar, false);
@@ -140,8 +145,10 @@ void si_nir_opts(struct si_screen *sscreen, struct nir_shader *nir, bool first)
       if (nir->info.stage == MESA_SHADER_FRAGMENT)
          NIR_PASS_V(nir, nir_opt_move_discards_to_top);
 
-      if (sscreen->info.has_packed_math_16bit)
-         NIR_PASS(progress, nir, nir_opt_vectorize, si_vectorize_callback, NULL);
+      if (sscreen->info.has_packed_math_16bit) {
+         NIR_PASS(progress, nir, nir_opt_vectorize, si_vectorize_callback,
+                  (void *)sscreen->use_aco);
+      }
    } while (progress);
 
    NIR_PASS_V(nir, nir_lower_var_copies);
@@ -186,7 +193,7 @@ static void si_late_optimize_16bit_samplers(struct si_screen *sscreen, nir_shade
     * We only use a16/g16 if all of the affected sources are 16bit.
     */
    bool has_g16 = sscreen->info.gfx_level >= GFX10;
-   struct nir_fold_tex_srcs_options fold_srcs_options[] = {
+   struct nir_opt_tex_srcs_options opt_srcs_options[] = {
       {
          .sampler_dims =
             ~(BITFIELD_BIT(GLSL_SAMPLER_DIM_CUBE) | BITFIELD_BIT(GLSL_SAMPLER_DIM_BUF)),
@@ -200,16 +207,17 @@ static void si_late_optimize_16bit_samplers(struct si_screen *sscreen, nir_shade
          .src_types = (1 << nir_tex_src_ddx) | (1 << nir_tex_src_ddy),
       },
    };
-   struct nir_fold_16bit_tex_image_options fold_16bit_options = {
-      .rounding_mode = nir_rounding_mode_rtz,
-      .fold_tex_dest_types = nir_type_float,
-      .fold_image_dest_types = nir_type_float,
-      .fold_image_store_data = true,
-      .fold_srcs_options_count = has_g16 ? 2 : 1,
-      .fold_srcs_options = fold_srcs_options,
+   struct nir_opt_16bit_tex_image_options opt_16bit_options = {
+      .rounding_mode = nir_rounding_mode_undef,
+      .opt_tex_dest_types = nir_type_float,
+      .opt_image_dest_types = nir_type_float,
+      .opt_image_store_data = true,
+      .opt_image_srcs = true,
+      .opt_srcs_options_count = has_g16 ? 2 : 1,
+      .opt_srcs_options = opt_srcs_options,
    };
    bool changed = false;
-   NIR_PASS(changed, nir, nir_fold_16bit_tex_image, &fold_16bit_options);
+   NIR_PASS(changed, nir, nir_opt_16bit_tex_image, &opt_16bit_options);
 
    if (changed) {
       si_nir_opts(sscreen, nir, false);
@@ -223,7 +231,7 @@ lower_intrinsic_filter(const nir_instr *instr, const void *dummy)
    return instr->type == nir_instr_type_intrinsic;
 }
 
-static nir_ssa_def *
+static nir_def *
 lower_intrinsic_instr(nir_builder *b, nir_instr *instr, void *dummy)
 {
    nir_intrinsic_instr *intrin = nir_instr_as_intrinsic(instr);
@@ -247,15 +255,16 @@ static bool si_lower_intrinsics(nir_shader *nir)
                                         NULL);
 }
 
-const nir_lower_subgroups_options si_nir_subgroups_options = {
-   .subgroup_size = 64,
-   .ballot_bit_size = 64,
-   .ballot_components = 1,
-   .lower_to_scalar = true,
-   .lower_subgroup_masks = true,
-   .lower_vote_trivial = false,
-   .lower_vote_eq = true,
-};
+void si_lower_mediump_io(nir_shader *nir)
+{
+   NIR_PASS_V(nir, nir_lower_mediump_io,
+              /* TODO: LLVM fails to compile this test if VS inputs are 16-bit:
+               * dEQP-GLES31.functional.shaders.builtin_functions.integer.bitfieldinsert.uvec3_lowp_geometry
+               */
+              (nir->info.stage != MESA_SHADER_VERTEX ? nir_var_shader_in : 0) | nir_var_shader_out,
+              BITFIELD64_BIT(VARYING_SLOT_PNTC) | BITFIELD64_RANGE(VARYING_SLOT_VAR0, 32),
+              true);
+}
 
 /**
  * Perform "lowering" operations on the NIR that are run once when the shader
@@ -270,6 +279,7 @@ static void si_lower_nir(struct si_screen *sscreen, struct nir_shader *nir)
     * - ensure constant offsets for texture instructions are folded
     *   and copy-propagated
     */
+   NIR_PASS_V(nir, nir_lower_int64);
 
    const struct nir_lower_tex_options lower_tex_options = {
       .lower_txp = ~0u,
@@ -278,12 +288,14 @@ static void si_lower_nir(struct si_screen *sscreen, struct nir_shader *nir)
       .lower_invalid_implicit_lod = true,
       .lower_tg4_offsets = true,
       .lower_to_fragment_fetch_amd = sscreen->info.gfx_level < GFX11,
+      .lower_1d = sscreen->info.gfx_level == GFX9,
    };
    NIR_PASS_V(nir, nir_lower_tex, &lower_tex_options);
 
    const struct nir_lower_image_options lower_image_options = {
       .lower_cube_size = true,
-      .lower_to_fragment_mask_load_amd = sscreen->info.gfx_level < GFX11,
+      .lower_to_fragment_mask_load_amd = sscreen->info.gfx_level < GFX11 &&
+                                         !(sscreen->debug_flags & DBG(NO_FMASK)),
    };
    NIR_PASS_V(nir, nir_lower_image, &lower_image_options);
 
@@ -291,7 +303,7 @@ static void si_lower_nir(struct si_screen *sscreen, struct nir_shader *nir)
 
    NIR_PASS_V(nir, ac_nir_lower_sin_cos);
 
-   NIR_PASS_V(nir, nir_lower_subgroups, &si_nir_subgroups_options);
+   NIR_PASS_V(nir, nir_lower_subgroups, sscreen->nir_lower_subgroups_options);
 
    NIR_PASS_V(nir, nir_lower_discard_or_demote, true);
 
@@ -300,13 +312,12 @@ static void si_lower_nir(struct si_screen *sscreen, struct nir_shader *nir)
    NIR_PASS_V(nir, nir_lower_var_copies);
    NIR_PASS_V(nir, nir_opt_intrinsics);
    NIR_PASS_V(nir, nir_lower_system_values);
-   NIR_PASS_V(nir, nir_lower_compute_system_values, NULL);
 
    /* si_nir_kill_outputs and ac_nir_optimize_outputs require outputs to be scalar. */
    if (nir->info.stage == MESA_SHADER_VERTEX ||
        nir->info.stage == MESA_SHADER_TESS_EVAL ||
        nir->info.stage == MESA_SHADER_GEOMETRY)
-      NIR_PASS_V(nir, nir_lower_io_to_scalar, nir_var_shader_out);
+      NIR_PASS_V(nir, nir_lower_io_to_scalar, nir_var_shader_out, NULL, NULL);
 
    if (nir->info.stage == MESA_SHADER_GEOMETRY) {
       unsigned flags = nir_lower_gs_intrinsics_per_stream;
@@ -320,41 +331,34 @@ static void si_lower_nir(struct si_screen *sscreen, struct nir_shader *nir)
    }
 
    if (nir->info.stage == MESA_SHADER_COMPUTE) {
+      nir_lower_compute_system_values_options options = {0};
+
+      /* gl_LocalInvocationIndex must be derived from gl_LocalInvocationID.xyz to make it correct
+       * with quad derivatives. Using gl_SubgroupID for that (which is what we do by default) is
+       * incorrect with a non-linear thread order.
+       */
+      options.lower_local_invocation_index =
+         nir->info.cs.derivative_group == DERIVATIVE_GROUP_QUADS;
+      NIR_PASS_V(nir, nir_lower_compute_system_values, &options);
+
       if (nir->info.cs.derivative_group == DERIVATIVE_GROUP_QUADS) {
-         /* If we are shuffling local_invocation_id for quad derivatives, we
-          * need to derive local_invocation_index from local_invocation_id
-          * first, so that the value corresponds to the shuffled
-          * local_invocation_id.
-          */
-         nir_lower_compute_system_values_options options = {0};
-         options.lower_local_invocation_index = true;
+         nir_opt_cse(nir); /* CSE load_local_invocation_id */
+         memset(&options, 0, sizeof(options));
+         options.shuffle_local_ids_for_quad_derivatives = true;
          NIR_PASS_V(nir, nir_lower_compute_system_values, &options);
       }
-
-      nir_opt_cse(nir); /* CSE load_local_invocation_id */
-      nir_lower_compute_system_values_options options = {0};
-      options.shuffle_local_ids_for_quad_derivatives = true;
-      NIR_PASS_V(nir, nir_lower_compute_system_values, &options);
-   }
-
-   if (sscreen->b.get_shader_param(&sscreen->b, PIPE_SHADER_FRAGMENT, PIPE_SHADER_CAP_FP16)) {
-      NIR_PASS_V(nir, nir_lower_mediump_io,
-                 /* TODO: LLVM fails to compile this test if VS inputs are 16-bit:
-                  * dEQP-GLES31.functional.shaders.builtin_functions.integer.bitfieldinsert.uvec3_lowp_geometry
-                  */
-                 (nir->info.stage != MESA_SHADER_VERTEX ? nir_var_shader_in : 0) | nir_var_shader_out,
-                 BITFIELD64_BIT(VARYING_SLOT_PNTC) | BITFIELD64_RANGE(VARYING_SLOT_VAR0, 32),
-                 true);
    }
 
    si_nir_opts(sscreen, nir, true);
    /* Run late optimizations to fuse ffma and eliminate 16-bit conversions. */
    si_nir_late_opts(nir);
 
-   if (sscreen->b.get_shader_param(&sscreen->b, PIPE_SHADER_FRAGMENT, PIPE_SHADER_CAP_FP16))
+   if (sscreen->info.gfx_level >= GFX9)
       si_late_optimize_16bit_samplers(sscreen, nir);
 
    NIR_PASS_V(nir, nir_remove_dead_variables, nir_var_function_temp, NULL);
+
+   NIR_PASS_V(nir, nir_lower_fp16_casts, nir_lower_fp16_split_fp64);
 }
 
 static bool si_mark_divergent_texture_non_uniform(struct nir_shader *nir)
@@ -403,7 +407,7 @@ static bool si_mark_divergent_texture_non_uniform(struct nir_shader *nir)
          }
 
          /* If dest is already divergent, divergence won't change. */
-         divergence_changed |= !tex->dest.ssa.divergent &&
+         divergence_changed |= !tex->def.divergent &&
             (tex->texture_non_uniform || tex->sampler_non_uniform);
       }
    }
