@@ -36,6 +36,7 @@
 #include "broadcom/compiler/v3d_compiler.h"
 #include "broadcom/common/v3d_macros.h"
 #include "broadcom/common/v3d_util.h"
+#include "broadcom/common/v3d_csd.h"
 #include "broadcom/cle/v3dx_pack.h"
 
 void
@@ -75,7 +76,7 @@ v3dX(start_binning)(struct v3d_context *v3d, struct v3d_job *job)
 
         job->tile_alloc = v3d_bo_alloc(v3d->screen, tile_alloc_size,
                                        "tile_alloc");
-        uint32_t tsda_per_tile_size = v3d->screen->devinfo.ver >= 40 ? 256 : 64;
+        uint32_t tsda_per_tile_size = 256;
         job->tile_state = v3d_bo_alloc(v3d->screen,
                                        MAX2(job->num_layers, 1) *
                                        job->draw_tiles_y *
@@ -83,7 +84,6 @@ v3dX(start_binning)(struct v3d_context *v3d, struct v3d_job *job)
                                        tsda_per_tile_size,
                                        "TSDA");
 
-#if V3D_VERSION >= 41
         /* This must go before the binning mode configuration. It is
          * required for layered framebuffers to work.
          */
@@ -92,10 +92,27 @@ v3dX(start_binning)(struct v3d_context *v3d, struct v3d_job *job)
                         config.number_of_layers = job->num_layers;
                 }
         }
-#endif
 
         assert(!job->msaa || !job->double_buffer);
-#if V3D_VERSION >= 40
+#if V3D_VERSION >= 71
+        cl_emit(&job->bcl, TILE_BINNING_MODE_CFG, config) {
+                config.width_in_pixels = job->draw_width;
+                config.height_in_pixels = job->draw_height;
+
+                config.log2_tile_width = log2_tile_size(job->tile_width);
+                config.log2_tile_height = log2_tile_size(job->tile_height);
+
+                /* FIXME: ideallly we would like next assert on the packet header (as is
+                 * general, so also applies to GL). We would need to expand
+                 * gen_pack_header for that.
+                 */
+                assert(config.log2_tile_width == config.log2_tile_height ||
+                       config.log2_tile_width == config.log2_tile_height + 1);
+        }
+
+#endif
+
+#if V3D_VERSION == 42
         cl_emit(&job->bcl, TILE_BINNING_MODE_CFG, config) {
                 config.width_in_pixels = job->draw_width;
                 config.height_in_pixels = job->draw_height;
@@ -107,34 +124,7 @@ v3dX(start_binning)(struct v3d_context *v3d, struct v3d_job *job)
 
                 config.maximum_bpp_of_all_render_targets = job->internal_bpp;
         }
-#else /* V3D_VERSION < 40 */
-        /* "Binning mode lists start with a Tile Binning Mode Configuration
-         * item (120)"
-         *
-         * Part1 signals the end of binning config setup.
-         */
-        cl_emit(&job->bcl, TILE_BINNING_MODE_CFG_PART2, config) {
-                config.tile_allocation_memory_address =
-                        cl_address(job->tile_alloc, 0);
-                config.tile_allocation_memory_size = job->tile_alloc->size;
-        }
-
-        cl_emit(&job->bcl, TILE_BINNING_MODE_CFG_PART1, config) {
-                config.tile_state_data_array_base_address =
-                        cl_address(job->tile_state, 0);
-
-                config.width_in_tiles = job->draw_tiles_x;
-                config.height_in_tiles = job->draw_tiles_y;
-                /* Must be >= 1 */
-                config.number_of_render_targets =
-                        MAX2(job->nr_cbufs, 1);
-
-                config.multisample_mode_4x = job->msaa;
-                config.double_buffer_in_non_ms_mode = job->double_buffer;
-
-                config.maximum_bpp_of_all_render_targets = job->internal_bpp;
-        }
-#endif /* V3D_VERSION < 40 */
+#endif
 
         /* There's definitely nothing in the VCD cache we want. */
         cl_emit(&job->bcl, FLUSH_VCD_CACHE, bin);
@@ -184,7 +174,7 @@ v3d_predraw_check_stage_inputs(struct pipe_context *pctx,
                         v3d_update_shadow_texture(pctx, &view->base);
 
                 v3d_flush_jobs_writing_resource(v3d, view->texture,
-                                                V3D_FLUSH_DEFAULT,
+                                                V3D_FLUSH_NOT_CURRENT_JOB,
                                                 s == PIPE_SHADER_COMPUTE);
         }
 
@@ -361,7 +351,6 @@ v3d_emit_wait_for_tf_if_needed(struct v3d_context *v3d, struct v3d_job *job)
         }
 }
 
-#if V3D_VERSION >= 41
 static void
 v3d_emit_gs_state_record(struct v3d_job *job,
                          struct v3d_compiled_shader *gs_bin,
@@ -377,7 +366,9 @@ v3d_emit_gs_state_record(struct v3d_job *job,
                         gs_bin->prog_data.gs->base.threads == 4;
                 shader.geometry_bin_mode_shader_start_in_final_thread_section =
                         gs_bin->prog_data.gs->base.single_seg;
+#if V3D_VERSION == 42
                 shader.geometry_bin_mode_shader_propagate_nans = true;
+#endif
                 shader.geometry_bin_mode_shader_uniforms_address =
                         gs_bin_uniforms;
 
@@ -387,7 +378,9 @@ v3d_emit_gs_state_record(struct v3d_job *job,
                         gs->prog_data.gs->base.threads == 4;
                 shader.geometry_render_mode_shader_start_in_final_thread_section =
                         gs->prog_data.gs->base.single_seg;
+#if V3D_VERSION == 42
                 shader.geometry_render_mode_shader_propagate_nans = true;
+#endif
                 shader.geometry_render_mode_shader_uniforms_address =
                         gs_render_uniforms;
         }
@@ -477,7 +470,6 @@ v3d_emit_tes_gs_shader_params(struct v3d_job *job,
                 shader.gbg_min_gs_output_segments_required_in_play = 1;
         }
 }
-#endif
 
 static void
 v3d_emit_gl_shader_state(struct v3d_context *v3d,
@@ -536,14 +528,12 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
 
         uint32_t shader_state_record_length =
                 cl_packet_length(GL_SHADER_STATE_RECORD);
-#if V3D_VERSION >= 41
         if (v3d->prog.gs) {
                 shader_state_record_length +=
                         cl_packet_length(GEOMETRY_SHADER_STATE_RECORD) +
                         cl_packet_length(TESSELLATION_GEOMETRY_COMMON_PARAMS) +
                         2 * cl_packet_length(TESSELLATION_GEOMETRY_SHADER_PARAMS);
         }
-#endif
 
         /* See GFXH-930 workaround below */
         uint32_t shader_rec_offset =
@@ -559,8 +549,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
          */
 
         struct vpm_config vpm_cfg_bin, vpm_cfg;
-
-        assert(v3d->screen->devinfo.ver >= 41 || !v3d->prog.gs);
         v3d_compute_vpm_config(&v3d->screen->devinfo,
                                v3d->prog.cs->prog_data.vs,
                                v3d->prog.vs->prog_data.vs,
@@ -570,7 +558,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                                &vpm_cfg);
 
         if (v3d->prog.gs) {
-#if V3D_VERSION >= 41
                 v3d_emit_gs_state_record(v3d->job,
                                          v3d->prog.gs_bin, gs_bin_uniforms,
                                          v3d->prog.gs, gs_uniforms);
@@ -591,9 +578,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                                               vpm_cfg.gs_width,
                                               vpm_cfg.Gd,
                                               vpm_cfg.Gv);
-#else
-                unreachable("No GS support pre-4.1");
-#endif
         }
 
         cl_emit(&job->indirect, GL_SHADER_STATE_RECORD, shader) {
@@ -620,28 +604,20 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                 shader.fragment_shader_uses_real_pixel_centre_w_in_addition_to_centroid_w2 =
                         v3d->prog.fs->prog_data.fs->uses_center_w;
 
-#if V3D_VERSION >= 41
                 shader.any_shader_reads_hardware_written_primitive_id =
                         (v3d->prog.gs && v3d->prog.gs->prog_data.gs->uses_pid) ||
                         v3d->prog.fs->prog_data.fs->uses_pid;
                 shader.insert_primitive_id_as_first_varying_to_fragment_shader =
                         !v3d->prog.gs && v3d->prog.fs->prog_data.fs->uses_pid;
-#endif
 
-#if V3D_VERSION >= 40
-               shader.do_scoreboard_wait_on_first_thread_switch =
+                shader.do_scoreboard_wait_on_first_thread_switch =
                         v3d->prog.fs->prog_data.fs->lock_scoreboard_on_first_thrsw;
-               shader.disable_implicit_point_line_varyings =
+                shader.disable_implicit_point_line_varyings =
                         !v3d->prog.fs->prog_data.fs->uses_implicit_point_line_varyings;
-#endif
 
                 shader.number_of_varyings_in_fragment_shader =
                         v3d->prog.fs->prog_data.fs->num_inputs;
 
-                shader.coordinate_shader_propagate_nans = true;
-                shader.vertex_shader_propagate_nans = true;
-                shader.fragment_shader_propagate_nans = true;
-
                 shader.coordinate_shader_code_address =
                         cl_address(v3d_resource(v3d->prog.cs->resource)->bo,
                                    v3d->prog.cs->offset);
@@ -652,6 +628,11 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                         cl_address(v3d_resource(v3d->prog.fs->resource)->bo,
                                    v3d->prog.fs->offset);
 
+#if V3D_VERSION == 42
+                shader.coordinate_shader_propagate_nans = true;
+                shader.vertex_shader_propagate_nans = true;
+                shader.fragment_shader_propagate_nans = true;
+
                 /* XXX: Use combined input/output size flag in the common
                  * case.
                  */
@@ -659,13 +640,24 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                         v3d->prog.cs->prog_data.vs->separate_segments;
                 shader.vertex_shader_has_separate_input_and_output_vpm_blocks =
                         v3d->prog.vs->prog_data.vs->separate_segments;
-
                 shader.coordinate_shader_input_vpm_segment_size =
                         v3d->prog.cs->prog_data.vs->separate_segments ?
                         v3d->prog.cs->prog_data.vs->vpm_input_size : 1;
                 shader.vertex_shader_input_vpm_segment_size =
                         v3d->prog.vs->prog_data.vs->separate_segments ?
                         v3d->prog.vs->prog_data.vs->vpm_input_size : 1;
+#endif
+                /* On V3D 7.1 there isn't a specific flag to set if we are using
+                 * shared/separate segments or not. We just set the value of
+                 * vpm_input_size to 0, and set output to the max needed. That should be
+                 * already properly set on prog_data_vs_bin
+                 */
+#if V3D_VERSION == 71
+                shader.coordinate_shader_input_vpm_segment_size =
+                        v3d->prog.cs->prog_data.vs->vpm_input_size;
+                shader.vertex_shader_input_vpm_segment_size =
+                        v3d->prog.vs->prog_data.vs->vpm_input_size;
+#endif
 
                 shader.coordinate_shader_output_vpm_segment_size =
                         v3d->prog.cs->prog_data.vs->vpm_output_size;
@@ -676,7 +668,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                 shader.vertex_shader_uniforms_address = vs_uniforms;
                 shader.fragment_shader_uniforms_address = fs_uniforms;
 
-#if V3D_VERSION >= 41
                 shader.min_coord_shader_input_segments_required_in_play =
                         vpm_cfg_bin.As;
                 shader.min_vertex_shader_input_segments_required_in_play =
@@ -700,20 +691,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                         v3d->prog.vs->prog_data.vs->base.single_seg;
                 shader.fragment_shader_start_in_final_thread_section =
                         v3d->prog.fs->prog_data.fs->base.single_seg;
-#else
-                shader.coordinate_shader_4_way_threadable =
-                        v3d->prog.cs->prog_data.vs->base.threads == 4;
-                shader.coordinate_shader_2_way_threadable =
-                        v3d->prog.cs->prog_data.vs->base.threads == 2;
-                shader.vertex_shader_4_way_threadable =
-                        v3d->prog.vs->prog_data.vs->base.threads == 4;
-                shader.vertex_shader_2_way_threadable =
-                        v3d->prog.vs->prog_data.vs->base.threads == 2;
-                shader.fragment_shader_4_way_threadable =
-                        v3d->prog.fs->prog_data.fs->base.threads == 4;
-                shader.fragment_shader_2_way_threadable =
-                        v3d->prog.fs->prog_data.fs->base.threads == 2;
-#endif
 
                 shader.vertex_id_read_by_coordinate_shader =
                         v3d->prog.cs->prog_data.vs->uses_vid;
@@ -724,12 +701,17 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                 shader.instance_id_read_by_vertex_shader =
                         v3d->prog.vs->prog_data.vs->uses_iid;
 
+#if V3D_VERSION == 42
                 shader.address_of_default_attribute_values =
                         cl_address(v3d_resource(vtx->defaults)->bo,
                                    vtx->defaults_offset);
+#endif
         }
 
         bool cs_loaded_any = false;
+        const bool cs_uses_builtins = v3d->prog.cs->prog_data.vs->uses_iid ||
+                                      v3d->prog.cs->prog_data.vs->uses_biid ||
+                                      v3d->prog.cs->prog_data.vs->uses_vid;
         for (int i = 0; i < vtx->num_elements; i++) {
                 struct pipe_vertex_element *elem = &vtx->pipe[i];
                 struct pipe_vertex_buffer *vb =
@@ -743,7 +725,7 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                 cl_emit_with_prepacked(&job->indirect,
                                        GL_SHADER_STATE_ATTRIBUTE_RECORD,
                                        &vtx->attrs[i * size], attr) {
-                        attr.stride = vb->stride;
+                        attr.stride = elem->src_stride;
                         attr.address = cl_address(rsc->bo,
                                                   vb->buffer_offset +
                                                   elem->src_offset);
@@ -759,15 +741,20 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                          * inputs.  (Since CS is just dead-code-elimination
                          * compared to VS, we can't have CS loading but not
                          * VS).
+                         *
+                         * GFXH-1602: first attribute must be active if using
+                         * builtins.
                          */
                         if (v3d->prog.cs->prog_data.vs->vattr_sizes[i])
                                 cs_loaded_any = true;
-                        if (i == vtx->num_elements - 1 && !cs_loaded_any) {
+                        if (i == 0 && cs_uses_builtins && !cs_loaded_any) {
                                 attr.number_of_values_read_by_coordinate_shader = 1;
+                                cs_loaded_any = true;
+                        } else if (i == vtx->num_elements - 1 && !cs_loaded_any) {
+                                attr.number_of_values_read_by_coordinate_shader = 1;
+                                cs_loaded_any = true;
                         }
-#if V3D_VERSION >= 41
                         attr.maximum_index = 0xffffff;
-#endif
                 }
                 STATIC_ASSERT(sizeof(vtx->attrs) >= V3D_MAX_VS_INPUTS / 4 * size);
         }
@@ -796,7 +783,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                 vcm.number_of_16_vertex_batches_for_rendering = vpm_cfg.Vc;
         }
 
-#if V3D_VERSION >= 41
         if (v3d->prog.gs) {
                 cl_emit(&job->bcl, GL_SHADER_STATE_INCLUDING_GS, state) {
                         state.address = cl_address(job->indirect.bo,
@@ -810,13 +796,6 @@ v3d_emit_gl_shader_state(struct v3d_context *v3d,
                         state.number_of_attribute_arrays = num_elements_to_emit;
                 }
         }
-#else
-        assert(!v3d->prog.gs);
-        cl_emit(&job->bcl, GL_SHADER_STATE, state) {
-                state.address = cl_address(job->indirect.bo, shader_rec_offset);
-                state.number_of_attribute_arrays = num_elements_to_emit;
-        }
-#endif
 
         v3d_bo_unreference(&cs_uniforms.bo);
         v3d_bo_unreference(&vs_uniforms.bo);
@@ -863,13 +842,15 @@ v3d_update_job_ez(struct v3d_context *v3d, struct v3d_job *job)
                 return;
         }
 
-        /* If this is the first time we update EZ state for this job we first
-         * check if there is anything that requires disabling it completely
-         * for the entire job (based on state that is not related to the
-         * current draw call and pipeline state).
+        /* When we update the EZ state we first check if there is anything
+         * that requires disabling it completely for the entire job (based on
+         * state that is not related to the current draw call and pipeline
+         * state).
          */
-        if (!job->decided_global_ez_enable) {
+        if (!job->decided_global_ez_enable ||
+            job->global_ez_zsa_decision_state != v3d->zsa) {
                 job->decided_global_ez_enable = true;
+                job->global_ez_zsa_decision_state = v3d->zsa;
 
                 if (!job->zsbuf) {
                         job->first_ez_state = V3D_EZ_DISABLED;
@@ -878,19 +859,29 @@ v3d_update_job_ez(struct v3d_context *v3d, struct v3d_job *job)
                 }
 
                 /* GFXH-1918: the early-Z buffer may load incorrect depth
-                 * values if the frame has odd width or height. Disable early-Z
-                 * in this case.
+                 * values if the frame has odd width or height, or if the
+                 * buffer is 16-bit and multisampled. Disable early-Z in these
+                 * cases.
                  */
                 bool needs_depth_load = v3d->zsa && job->zsbuf &&
                         v3d->zsa->base.depth_enabled &&
                         (PIPE_CLEAR_DEPTH & ~job->clear);
-                if (needs_depth_load &&
-                     ((job->draw_width % 2 != 0) || (job->draw_height % 2 != 0))) {
-                        perf_debug("Loading depth buffer for framebuffer with odd width "
-                                   "or height disables early-Z tests\n");
-                        job->first_ez_state = V3D_EZ_DISABLED;
-                        job->ez_state = V3D_EZ_DISABLED;
-                        return;
+                if (needs_depth_load) {
+                        if (job->zsbuf->texture->format == PIPE_FORMAT_Z16_UNORM &&
+                            job->zsbuf->texture->nr_samples > 0) {
+                                perf_debug("Loading 16-bit multisampled depth buffer "
+                                           "disables early-Z tests\n");
+                                job->first_ez_state = V3D_EZ_DISABLED;
+                                job->ez_state = V3D_EZ_DISABLED;
+                                return;
+                        }
+                        if ((job->draw_width % 2 != 0) || (job->draw_height % 2 != 0)) {
+                                perf_debug("Loading depth buffer for framebuffer with "
+                                           "odd width or height disables early-Z tests\n");
+                                job->first_ez_state = V3D_EZ_DISABLED;
+                                job->ez_state = V3D_EZ_DISABLED;
+                                return;
+                        }
                 }
         }
 
@@ -1057,12 +1048,16 @@ v3d_draw_vbo(struct pipe_context *pctx, const struct pipe_draw_info *info,
                 u_foreach_bit(i, v3d->ssbo[s].enabled_mask) {
                         v3d_job_add_write_resource(job,
                                                    v3d->ssbo[s].sb[i].buffer);
+                        struct v3d_resource *rsc= v3d_resource(v3d->ssbo[s].sb[i].buffer);
+                        rsc->graphics_written = true;
                         job->tmu_dirty_rcl = true;
                 }
 
                 u_foreach_bit(i, v3d->shaderimg[s].enabled_mask) {
                         v3d_job_add_write_resource(job,
                                                    v3d->shaderimg[s].si[i].base.resource);
+                        struct v3d_resource *rsc= v3d_resource(v3d->shaderimg[s].si[i].base.resource);
+                        rsc->graphics_written = true;
                         job->tmu_dirty_rcl = true;
                 }
         }
@@ -1127,13 +1122,6 @@ v3d_draw_vbo(struct pipe_context *pctx, const struct pipe_draw_info *info,
         }
 
         uint32_t prim_tf_enable = 0;
-#if V3D_VERSION < 40
-        /* V3D 3.x: The HW only processes transform feedback on primitives
-         * with the flag set.
-         */
-        if (v3d->streamout.num_targets)
-                prim_tf_enable = (V3D_PRIM_POINTS_TF - V3D_PRIM_POINTS);
-#endif
 
         v3d->prim_restart = info->primitive_restart;
 
@@ -1157,20 +1145,14 @@ v3d_draw_vbo(struct pipe_context *pctx, const struct pipe_draw_info *info,
                 }
                 struct v3d_resource *rsc = v3d_resource(prsc);
 
-#if V3D_VERSION >= 40
                 cl_emit(&job->bcl, INDEX_BUFFER_SETUP, ib) {
                         ib.address = cl_address(rsc->bo, 0);
                         ib.size = rsc->bo->size;
                 }
-#endif
 
                 if (indirect && indirect->buffer) {
                         cl_emit(&job->bcl, INDIRECT_INDEXED_INSTANCED_PRIM_LIST, prim) {
                                 prim.index_type = ffs(info->index_size) - 1;
-#if V3D_VERSION < 40
-                                prim.address_of_indices_list =
-                                        cl_address(rsc->bo, offset);
-#endif /* V3D_VERSION < 40 */
                                 prim.mode = hw_prim_type | prim_tf_enable;
                                 prim.enable_primitive_restarts = info->primitive_restart;
 
@@ -1183,13 +1165,7 @@ v3d_draw_vbo(struct pipe_context *pctx, const struct pipe_draw_info *info,
                 } else if (info->instance_count > 1) {
                         cl_emit(&job->bcl, INDEXED_INSTANCED_PRIM_LIST, prim) {
                                 prim.index_type = ffs(info->index_size) - 1;
-#if V3D_VERSION >= 40
                                 prim.index_offset = offset;
-#else /* V3D_VERSION < 40 */
-                                prim.maximum_index = (1u << 31) - 1; /* XXX */
-                                prim.address_of_indices_list =
-                                        cl_address(rsc->bo, offset);
-#endif /* V3D_VERSION < 40 */
                                 prim.mode = hw_prim_type | prim_tf_enable;
                                 prim.enable_primitive_restarts = info->primitive_restart;
 
@@ -1200,13 +1176,7 @@ v3d_draw_vbo(struct pipe_context *pctx, const struct pipe_draw_info *info,
                         cl_emit(&job->bcl, INDEXED_PRIM_LIST, prim) {
                                 prim.index_type = ffs(info->index_size) - 1;
                                 prim.length = draws[0].count;
-#if V3D_VERSION >= 40
                                 prim.index_offset = offset;
-#else /* V3D_VERSION < 40 */
-                                prim.maximum_index = (1u << 31) - 1; /* XXX */
-                                prim.address_of_indices_list =
-                                        cl_address(rsc->bo, offset);
-#endif /* V3D_VERSION < 40 */
                                 prim.mode = hw_prim_type | prim_tf_enable;
                                 prim.enable_primitive_restarts = info->primitive_restart;
                         }
@@ -1324,23 +1294,6 @@ v3d_draw_vbo(struct pipe_context *pctx, const struct pipe_draw_info *info,
                 v3d_flush(pctx);
 }
 
-#if V3D_VERSION >= 41
-#define V3D_CSD_CFG012_WG_COUNT_SHIFT 16
-#define V3D_CSD_CFG012_WG_OFFSET_SHIFT 0
-/* Allow this dispatch to start while the last one is still running. */
-#define V3D_CSD_CFG3_OVERLAP_WITH_PREV (1 << 26)
-/* Maximum supergroup ID.  6 bits. */
-#define V3D_CSD_CFG3_MAX_SG_ID_SHIFT 20
-/* Batches per supergroup minus 1.  8 bits. */
-#define V3D_CSD_CFG3_BATCHES_PER_SG_M1_SHIFT 12
-/* Workgroups per supergroup, 0 means 16 */
-#define V3D_CSD_CFG3_WGS_PER_SG_SHIFT 8
-#define V3D_CSD_CFG3_WG_SIZE_SHIFT 0
-
-#define V3D_CSD_CFG5_PROPAGATE_NANS (1 << 2)
-#define V3D_CSD_CFG5_SINGLE_SEG (1 << 1)
-#define V3D_CSD_CFG5_THREADING (1 << 0)
-
 static void
 v3d_launch_grid(struct pipe_context *pctx, const struct pipe_grid_info *info)
 {
@@ -1436,8 +1389,15 @@ v3d_launch_grid(struct pipe_context *pctx, const struct pipe_grid_info *info)
         submit.cfg[3] |= (wg_size & 0xff) << V3D_CSD_CFG3_WG_SIZE_SHIFT;
 
 
-        /* Number of batches the dispatch will invoke (minus 1). */
-        submit.cfg[4] = num_batches - 1;
+        /* Number of batches the dispatch will invoke.
+         * V3D 7.1.6 and later don't subtract 1 from the number of batches
+         */
+        if (v3d->screen->devinfo.ver < 71 ||
+            (v3d->screen->devinfo.ver == 71 && v3d->screen->devinfo.rev < 6)) {
+                submit.cfg[4] = num_batches - 1;
+        } else {
+                submit.cfg[4] = num_batches;
+        }
 
         /* Make sure we didn't accidentally underflow. */
         assert(submit.cfg[4] != ~0);
@@ -1445,7 +1405,8 @@ v3d_launch_grid(struct pipe_context *pctx, const struct pipe_grid_info *info)
         v3d_job_add_bo(job, v3d_resource(v3d->prog.compute->resource)->bo);
         submit.cfg[5] = (v3d_resource(v3d->prog.compute->resource)->bo->offset +
                          v3d->prog.compute->offset);
-        submit.cfg[5] |= V3D_CSD_CFG5_PROPAGATE_NANS;
+        if (v3d->screen->devinfo.ver < 71)
+                submit.cfg[5] |= V3D_CSD_CFG5_PROPAGATE_NANS;
         if (v3d->prog.compute->prog_data.base->single_seg)
                 submit.cfg[5] |= V3D_CSD_CFG5_SINGLE_SEG;
         if (v3d->prog.compute->prog_data.base->threads == 4)
@@ -1455,7 +1416,7 @@ v3d_launch_grid(struct pipe_context *pctx, const struct pipe_grid_info *info)
                 v3d->compute_shared_memory =
                         v3d_bo_alloc(v3d->screen,
                                      v3d->prog.compute->prog_data.compute->shared_size *
-                                     wgs_per_sg,
+                                     num_wgs,
                                      "shared_vars");
         }
 
@@ -1518,7 +1479,6 @@ v3d_launch_grid(struct pipe_context *pctx, const struct pipe_grid_info *info)
         v3d_bo_unreference(&uniforms.bo);
         v3d_bo_unreference(&v3d->compute_shared_memory);
 }
-#endif
 
 /**
  * Implements gallium's clear() hook (glClear()) by drawing a pair of triangles.
@@ -1560,9 +1520,10 @@ v3d_tlb_clear(struct v3d_job *job, unsigned buffers,
         /* GFXH-1461: If we were to emit a load of just depth or just stencil,
          * then the clear for the other may get lost.  We need to decide now
          * if it would be possible to need to emit a load of just one after
-         * we've set up our TLB clears.
+         * we've set up our TLB clears. This issue is fixed since V3D 4.3.18.
          */
-        if (buffers & PIPE_CLEAR_DEPTHSTENCIL &&
+        if (v3d->screen->devinfo.ver == 42 &&
+            buffers & PIPE_CLEAR_DEPTHSTENCIL &&
             (buffers & PIPE_CLEAR_DEPTHSTENCIL) != PIPE_CLEAR_DEPTHSTENCIL &&
             job->zsbuf &&
             util_format_is_depth_and_stencil(job->zsbuf->texture->format)) {
@@ -1716,8 +1677,6 @@ v3dX(draw_init)(struct pipe_context *pctx)
         pctx->clear = v3d_clear;
         pctx->clear_render_target = v3d_clear_render_target;
         pctx->clear_depth_stencil = v3d_clear_depth_stencil;
-#if V3D_VERSION >= 41
         if (v3d_context(pctx)->screen->has_csd)
                 pctx->launch_grid = v3d_launch_grid;
-#endif
 }
