@@ -6,12 +6,12 @@ use crate::core::platform::*;
 use crate::impl_cl_type_trait;
 
 use mesa_rust::pipe::context::PipeContext;
-use mesa_rust::pipe::resource::PipeResource;
-use mesa_rust::pipe::screen::ResourceType;
 use mesa_rust_util::properties::*;
 use rusticl_opencl_gen::*;
 
+use std::cmp;
 use std::mem;
+use std::mem::ManuallyDrop;
 use std::ops::Deref;
 use std::sync::mpsc;
 use std::sync::Arc;
@@ -25,7 +25,7 @@ use std::thread::JoinHandle;
 /// Used for tracking bound GPU state to lower CPU overhead and centralize state tracking
 pub struct QueueContext {
     ctx: PipeContext,
-    cb0: Option<PipeResource>,
+    use_stream: bool,
 }
 
 impl QueueContext {
@@ -34,34 +34,25 @@ impl QueueContext {
             .screen()
             .create_context()
             .ok_or(CL_OUT_OF_HOST_MEMORY)?;
-        let size = device.param_max_size() as u32;
-        let cb0 = if device.prefers_real_buffer_in_cb0() {
-            device
-                .screen()
-                .resource_create_buffer(size, ResourceType::Cb0, 0)
-        } else {
-            None
-        };
 
-        if let Some(cb0) = &cb0 {
-            ctx.bind_constant_buffer(0, cb0);
-        }
-
-        Ok(Self { ctx: ctx, cb0: cb0 })
+        Ok(Self {
+            ctx: ctx,
+            use_stream: device.prefers_real_buffer_in_cb0(),
+        })
     }
 
-    pub fn update_cb0(&self, data: &[u8]) {
+    pub fn update_cb0(&self, data: &[u8]) -> CLResult<()> {
         // only update if we actually bind data
         if !data.is_empty() {
-            // if we have a real buffer, update that, otherwise just set the data directly
-            if let Some(cb) = &self.cb0 {
-                debug_assert!(data.len() <= cb.width() as usize);
-                self.ctx
-                    .buffer_subdata(cb, 0, data.as_ptr().cast(), data.len() as u32);
+            if self.use_stream {
+                if !self.ctx.set_constant_buffer_stream(0, data) {
+                    return Err(CL_OUT_OF_RESOURCES);
+                }
             } else {
                 self.ctx.set_constant_buffer(0, data);
             }
         }
+        Ok(())
     }
 }
 
@@ -85,7 +76,7 @@ struct QueueState {
     last: Weak<Event>,
     // `Sync` on `Sender` was stabilized in 1.72, until then, put it into our Mutex.
     // see https://github.com/rust-lang/rust/commit/5f56956b3c7edb9801585850d1f41b0aeb1888ff
-    chan_in: mpsc::Sender<Vec<Arc<Event>>>,
+    chan_in: ManuallyDrop<mpsc::Sender<Vec<Arc<Event>>>>,
 }
 
 pub struct Queue {
@@ -95,7 +86,7 @@ pub struct Queue {
     pub props: cl_command_queue_properties,
     pub props_v2: Option<Properties<cl_queue_properties>>,
     state: Mutex<QueueState>,
-    _thrd: JoinHandle<()>,
+    thrd: ManuallyDrop<JoinHandle<()>>,
 }
 
 impl_cl_type_trait!(cl_command_queue, Queue, CL_INVALID_COMMAND_QUEUE);
@@ -119,7 +110,7 @@ impl Queue {
         let ctx = QueueContext::new_for(device)?;
         let (tx_q, rx_t) = mpsc::channel::<Vec<Arc<Event>>>();
         Ok(Arc::new(Self {
-            base: CLObjectBase::new(),
+            base: CLObjectBase::new(RusticlTypes::Queue),
             context: context,
             device: device,
             props: props,
@@ -127,11 +118,26 @@ impl Queue {
             state: Mutex::new(QueueState {
                 pending: Vec::new(),
                 last: Weak::new(),
-                chan_in: tx_q,
+                chan_in: ManuallyDrop::new(tx_q),
             }),
-            _thrd: thread::Builder::new()
+            thrd: ManuallyDrop::new(thread::Builder::new()
                 .name("rusticl queue thread".into())
-                .spawn(move || loop {
+                .spawn(move || {
+                    // Track the error of all executed events. This is only needed for in-order
+                    // queues, so for out of order we'll need to update this.
+                    // Also, the OpenCL specification gives us enough freedom to do whatever we want
+                    // in case of any event running into an error while executing:
+                    //
+                    //   Unsuccessful completion results in abnormal termination of the command
+                    //   which is indicated by setting the event status to a negative value. In this
+                    //   case, the command-queue associated with the abnormally terminated command
+                    //   and all other command-queues in the same context may no longer be available
+                    //   and their behavior is implementation-defined.
+                    //
+                    // TODO: use pipe_context::set_device_reset_callback to get notified about gone
+                    //       GPU contexts
+                    let mut last_err = CL_SUCCESS as cl_int;
+                    loop {
                     let r = rx_t.recv();
                     if r.is_err() {
                         break;
@@ -147,21 +153,31 @@ impl Queue {
                             flush_events(&mut flushed, &ctx);
                         }
 
-                        // We have to wait on user events or events from other queues.
-                        let err = e
-                            .deps
-                            .iter()
-                            .filter(|ev| ev.is_user() || ev.queue != e.queue)
-                            .map(|e| e.wait())
-                            .find(|s| *s < 0);
+                        // check if any dependency has an error
+                        for dep in &e.deps {
+                            // We have to wait on user events or events from other queues.
+                            let dep_err = if dep.is_user() || dep.queue != e.queue {
+                                dep.wait()
+                            } else {
+                                dep.status()
+                            };
 
-                        if let Some(err) = err {
+                            last_err = cmp::min(last_err, dep_err);
+                        }
+
+                        if last_err < 0 {
                             // If a dependency failed, fail this event as well.
-                            e.set_user_status(err);
+                            e.set_user_status(last_err);
                             continue;
                         }
 
-                        e.call(&ctx);
+                        // if there is an execution error don't bother signaling it as the  context
+                        // might be in a broken state. How queues behave after any event hit an
+                        // error is entirely implementation defined.
+                        last_err = e.call(&ctx);
+                        if last_err < 0 {
+                            continue;
+                        }
 
                         if e.is_user() {
                             // On each user event we flush our events as application might
@@ -180,8 +196,9 @@ impl Queue {
                     }
 
                     flush_events(&mut flushed, &ctx);
+                    }
                 })
-                .unwrap(),
+                .unwrap()),
         }))
     }
 
@@ -244,5 +261,15 @@ impl Drop for Queue {
         // commands in command_queue.
         // TODO: maybe we have to do it on every release?
         let _ = self.flush(true);
+
+        let state = self.state.get_mut().unwrap();
+
+        unsafe {
+            // disconnect the channel
+            ManuallyDrop::drop(&mut state.chan_in);
+
+            // and now explicitly wait on the thread to quit, because it won't happen implicitly.
+            ManuallyDrop::take(&mut self.thrd).join().unwrap();
+        }
     }
 }
