@@ -227,7 +227,7 @@ vlVaDeriveImage(VADriverContextP ctx, VASurfaceID surface, VAImage *image)
    VAImage *img;
    VAStatus status;
    struct pipe_screen *screen;
-   struct pipe_surface **surfaces;
+   struct pipe_resource *buf_resources[VL_NUM_COMPONENTS];
    struct pipe_video_buffer *new_buffer = NULL;
    int w;
    int h;
@@ -277,14 +277,17 @@ vlVaDeriveImage(VADriverContextP ctx, VASurfaceID surface, VAImage *image)
                                    PIPE_VIDEO_CAP_SUPPORTS_PROGRESSIVE))
          return VA_STATUS_ERROR_OPERATION_FAILED;
    } else if (util_format_get_num_planes(surf->buffer->buffer_format) >= 2 &&
-              !screen->get_video_param(screen, PIPE_VIDEO_PROFILE_UNKNOWN,
+              (!screen->get_video_param(screen, PIPE_VIDEO_PROFILE_UNKNOWN,
                                        PIPE_VIDEO_ENTRYPOINT_BITSTREAM,
-                                       PIPE_VIDEO_CAP_SUPPORTS_CONTIGUOUS_PLANES_MAP)) {
+                                       PIPE_VIDEO_CAP_SUPPORTS_CONTIGUOUS_PLANES_MAP) ||
+               !surf->buffer->contiguous_planes)) {
       return VA_STATUS_ERROR_OPERATION_FAILED;
    }
 
-   surfaces = surf->buffer->get_surfaces(surf->buffer);
-   if (!surfaces || !surfaces[0]->texture)
+   memset(buf_resources, 0, sizeof(buf_resources));
+   surf->buffer->get_resources(surf->buffer, buf_resources);
+
+   if (!buf_resources[0])
       return VA_STATUS_ERROR_ALLOCATION_FAILED;
 
    img = CALLOC(1, sizeof(VAImage));
@@ -311,7 +314,7 @@ vlVaDeriveImage(VADriverContextP ctx, VASurfaceID surface, VAImage *image)
 
    mtx_lock(&drv->mutex);
    if (screen->resource_get_info) {
-      screen->resource_get_info(screen, surfaces[0]->texture, &stride,
+      screen->resource_get_info(screen, buf_resources[0], &stride,
                                 &offset);
       if (!stride)
          offset = 0;
@@ -344,14 +347,14 @@ vlVaDeriveImage(VADriverContextP ctx, VASurfaceID surface, VAImage *image)
       /* In some gallium platforms, the stride and offset are different*/
       /* for the Y and UV planes, query them independently.*/
       if (screen->resource_get_info) {
-         /* resource_get_info is called above for surfaces[0]->texture and */
+         /* resource_get_info is called above for buf_resources[0] and */
          /* saved results in stride, offset, reuse those values to avoid a new call to: */
-         /* screen->resource_get_info(screen, surfaces[0]->texture, &img->pitches[0],*/
+         /* screen->resource_get_info(screen, buf_resources[0], &img->pitches[0],*/
          /*                         &img->offsets[0]);*/
          img->pitches[0] = stride;
          img->offsets[0] = offset;
 
-         screen->resource_get_info(screen, surfaces[1]->texture, &img->pitches[1],
+         screen->resource_get_info(screen, buf_resources[1], &img->pitches[1],
                                  &img->offsets[1]);
          if (!img->pitches[1])
                img->offsets[1] = 0;
@@ -383,14 +386,15 @@ vlVaDeriveImage(VADriverContextP ctx, VASurfaceID surface, VAImage *image)
                            VL_COMPOSITOR_WEAVE);
 
          /* recalculate the values now that we have a new surface */
-         surfaces = surf->buffer->get_surfaces(new_buffer);
+         memset(buf_resources, 0, sizeof(buf_resources));
+         new_buffer->get_resources(new_buffer, buf_resources);
          if (screen->resource_get_info) {
-            screen->resource_get_info(screen, surfaces[0]->texture, &img->pitches[0],
+            screen->resource_get_info(screen, buf_resources[0], &img->pitches[0],
                                     &img->offsets[0]);
             if (!img->pitches[0])
                img->offsets[0] = 0;
 
-            screen->resource_get_info(screen, surfaces[1]->texture, &img->pitches[1],
+            screen->resource_get_info(screen, buf_resources[1], &img->pitches[1],
                                     &img->offsets[1]);
             if (!img->pitches[1])
                img->offsets[1] = 0;
@@ -432,7 +436,7 @@ vlVaDeriveImage(VADriverContextP ctx, VASurfaceID surface, VAImage *image)
    img_buf->size = img->data_size;
    img_buf->num_elements = 1;
 
-   pipe_resource_reference(&img_buf->derived_surface.resource, surfaces[0]->texture);
+   pipe_resource_reference(&img_buf->derived_surface.resource, buf_resources[0]);
    img_buf->derived_image_buffer = new_buffer;
 
    if (surf->ctx)
@@ -597,7 +601,8 @@ vlVaGetImage(VADriverContextP ctx, VASurfaceID surface, int x, int y,
                                   pipe_format_to_chroma_format(surf->templat.buffer_format),
                                   surf->templat.interlaced);
       for (j = 0; j < view_resources[i]->array_size; ++j) {
-         struct pipe_box box = {box_x, box_y, j, box_w, box_h, 1};
+         struct pipe_box box;
+         u_box_3d(box_x, box_y, j, box_w, box_h, 1, &box);
          struct pipe_transfer *transfer;
          uint8_t *map;
          map = drv->pipe->texture_map(drv->pipe, view_resources[i], 0,
@@ -724,7 +729,8 @@ vlVaPutImage(VADriverContextP ctx, VASurfaceID surface, VAImageID image,
 
       vlVaVideoSurfaceSize(surf, i, &width, &height);
       for (j = 0; j < tex->array_size; ++j) {
-         struct pipe_box dst_box = {0, 0, j, width, height, 1};
+         struct pipe_box dst_box;
+         u_box_3d(0, 0, j, width, height, 1, &dst_box);
 
          if (((format == PIPE_FORMAT_YV12) || (format == PIPE_FORMAT_IYUV))
              && (surf->buffer->buffer_format == PIPE_FORMAT_NV12)
