@@ -29,7 +29,7 @@
 #define PIPE_VIDEO_STATE_H
 
 #include "pipe/p_defines.h"
-#include "pipe/p_format.h"
+#include "util/format/u_formats.h"
 #include "pipe/p_state.h"
 #include "pipe/p_screen.h"
 #include "util/u_hash_table.h"
@@ -40,10 +40,16 @@
 extern "C" {
 #endif
 
+#define PIPE_H264_MAX_REFERENCES      16
 #define PIPE_H265_MAX_REFERENCES      15
+#define PIPE_H265_MAX_SLICES          128
+#define PIPE_AV1_MAX_REFERENCES       8
 #define PIPE_DEFAULT_FRAME_RATE_DEN   1
 #define PIPE_DEFAULT_FRAME_RATE_NUM   30
+#define PIPE_DEFAULT_INTRA_IDR_PERIOD 30
 #define PIPE_H2645_EXTENDED_SAR       255
+#define PIPE_ENC_ROI_REGION_NUM_MAX   32
+#define PIPE_DEFAULT_DECODER_FEEDBACK_TIMEOUT_NS 1000000000
 
 /*
  * see table 6-12 in the spec
@@ -126,6 +132,14 @@ enum pipe_h265_slice_type
    PIPE_H265_SLICE_TYPE_I = 0x2,
 };
 
+/* To be used on each encoding feature bit field */
+enum pipe_enc_feature
+{
+   PIPE_ENC_FEATURE_NOT_SUPPORTED = 0x0,
+   PIPE_ENC_FEATURE_SUPPORTED = 0x1,
+   PIPE_ENC_FEATURE_REQUIRED = 0x2,
+};
+
 /* Same enum for h264/h265 */
 enum pipe_h2645_enc_picture_type
 {
@@ -136,13 +150,23 @@ enum pipe_h2645_enc_picture_type
    PIPE_H2645_ENC_PICTURE_TYPE_SKIP = 0x04
 };
 
+enum pipe_av1_enc_frame_type
+{
+   PIPE_AV1_ENC_FRAME_TYPE_KEY = 0x00,
+   PIPE_AV1_ENC_FRAME_TYPE_INTER = 0x01,
+   PIPE_AV1_ENC_FRAME_TYPE_INTRA_ONLY = 0x02,
+   PIPE_AV1_ENC_FRAME_TYPE_SWITCH = 0x03,
+   PIPE_AV1_ENC_FRAME_TYPE_SHOW_EXISTING = 0x04
+};
+
 enum pipe_h2645_enc_rate_control_method
 {
    PIPE_H2645_ENC_RATE_CONTROL_METHOD_DISABLE = 0x00,
    PIPE_H2645_ENC_RATE_CONTROL_METHOD_CONSTANT_SKIP = 0x01,
    PIPE_H2645_ENC_RATE_CONTROL_METHOD_VARIABLE_SKIP = 0x02,
    PIPE_H2645_ENC_RATE_CONTROL_METHOD_CONSTANT = 0x03,
-   PIPE_H2645_ENC_RATE_CONTROL_METHOD_VARIABLE = 0x04
+   PIPE_H2645_ENC_RATE_CONTROL_METHOD_VARIABLE = 0x04,
+   PIPE_H2645_ENC_RATE_CONTROL_METHOD_QUALITY_VARIABLE = 0x05
 };
 
 enum pipe_slice_buffer_placement_type
@@ -165,7 +189,10 @@ struct pipe_picture_desc
    uint8_t *decrypt_key;
    uint32_t key_size;
    enum pipe_format input_format;
+   bool input_full_range;
    enum pipe_format output_format;
+   /* A fence used on PIPE_VIDEO_ENTRYPOINT_DECODE/PROCESSING to signal job completion */
+   struct pipe_fence_handle **fence;
 };
 
 struct pipe_quant_matrix
@@ -354,10 +381,6 @@ struct pipe_h264_pps
    int8_t   second_chroma_qp_index_offset;
 };
 
-struct h264_private {
-   struct pipe_video_buffer *past_ref[16];
-};
-
 struct pipe_h264_picture_desc
 {
    struct pipe_picture_desc base;
@@ -384,8 +407,15 @@ struct pipe_h264_picture_desc
 
    struct pipe_video_buffer *ref[16];
 
-   /* using private as a parameter name conflicts with C++ keywords */
-   void    *priv;
+   struct
+   {
+      bool slice_info_present;
+      uint32_t slice_count;
+      uint8_t slice_type[128];
+      uint32_t slice_data_size[128];
+      uint32_t slice_data_offset[128];
+      enum pipe_slice_buffer_placement_type slice_data_flag[128];
+   } slice_parameter;
 };
 
 struct pipe_enc_quality_modes
@@ -396,6 +426,57 @@ struct pipe_enc_quality_modes
    unsigned int vbaq_mode;
 };
 
+/*
+ * intra refresh supports row or column only, it doens't support
+ * row and column mixed, if mixed it will pick up column mode.
+ * Also the assumption is the first row/column since the offset
+ * is zero, and it marks the start of intra-refresh, it will need
+ * to have headers at this point.
+ */
+struct pipe_enc_intra_refresh
+{
+   unsigned int mode;
+   unsigned int region_size;
+   unsigned int offset;
+   unsigned int need_sequence_header;
+};
+
+/*
+ * In AVC, unit is MB, HEVC (CTB) and AV1(SB)
+ */
+enum
+{
+   INTRA_REFRESH_MODE_NONE,
+   INTRA_REFRESH_MODE_UNIT_ROWS,
+   INTRA_REFRESH_MODE_UNIT_COLUMNS,
+};
+
+/* All the values are in pixels, driver converts it into
+ * different units for different codecs, for example: h264
+ * is in 16x16 block, hevc/av1 is in 64x64 block.
+ * x, y means the location of region start, width/height defines
+ * the region size; the qp value carries the qp_delta.
+ */
+struct pipe_enc_region_in_roi
+{
+   bool    valid;
+   int32_t qp_value;
+   unsigned int x, y;
+   unsigned int width, height;
+};
+/* It does not support prioirty only qp_delta.
+ * The priority is implied by the region sequence number.
+ * Region 0 is most significant one, and region 1 is less
+ * significant, and lesser significant when region number
+ * grows. It allows region overlapping, and lower
+ * priority region would be overwritten by the higher one.
+ */
+struct pipe_enc_roi
+{
+   unsigned int num;
+   struct pipe_enc_region_in_roi region[PIPE_ENC_ROI_REGION_NUM_MAX];
+};
+
 struct pipe_h264_enc_rate_control
 {
    enum pipe_h2645_enc_rate_control_method rate_ctrl_method;
@@ -405,6 +486,8 @@ struct pipe_h264_enc_rate_control
    unsigned frame_rate_den;
    unsigned vbv_buffer_size;
    unsigned vbv_buf_lv;
+   unsigned vbv_buf_initial_size;
+   bool app_requested_hrd_buffer;
    unsigned target_bits_picture;
    unsigned peak_bits_picture_integer;
    unsigned peak_bits_picture_fraction;
@@ -414,6 +497,10 @@ struct pipe_h264_enc_rate_control
    unsigned max_au_size;
    unsigned max_qp;
    unsigned min_qp;
+   bool app_requested_qp_range;
+
+   /* Used with PIPE_H2645_ENC_RATE_CONTROL_METHOD_QUALITY_VARIABLE */
+   unsigned vbr_quality_factor;
 };
 
 struct pipe_h264_enc_motion_estimation
@@ -466,6 +553,20 @@ struct h265_slice_descriptor
    enum pipe_h265_slice_type slice_type;
 };
 
+typedef struct pipe_h264_enc_hrd_params
+{
+   uint32_t cpb_cnt_minus1;
+   uint32_t bit_rate_scale;
+   uint32_t cpb_size_scale;
+   uint32_t bit_rate_value_minus1[32];
+   uint32_t cpb_size_value_minus1[32];
+   uint32_t cbr_flag[32];
+   uint32_t initial_cpb_removal_delay_length_minus1;
+   uint32_t cpb_removal_delay_length_minus1;
+   uint32_t dpb_output_delay_length_minus1;
+   uint32_t time_offset_length;
+} pipe_h264_enc_hrd_params;
+
 struct pipe_h264_enc_seq_param
 {
    unsigned enc_constraint_set_flags;
@@ -475,17 +576,46 @@ struct pipe_h264_enc_seq_param
    unsigned enc_frame_crop_top_offset;
    unsigned enc_frame_crop_bottom_offset;
    unsigned pic_order_cnt_type;
+   unsigned log2_max_frame_num_minus4;
+   unsigned log2_max_pic_order_cnt_lsb_minus4;
    unsigned num_temporal_layers;
    uint32_t vui_parameters_present_flag;
    struct {
       uint32_t aspect_ratio_info_present_flag: 1;
       uint32_t timing_info_present_flag: 1;
+      uint32_t video_signal_type_present_flag: 1;
+      uint32_t colour_description_present_flag: 1;
+      uint32_t chroma_loc_info_present_flag: 1;
+      uint32_t overscan_info_present_flag: 1;
+      uint32_t overscan_appropriate_flag: 1;
+      uint32_t fixed_frame_rate_flag: 1;
+      uint32_t nal_hrd_parameters_present_flag: 1;
+      uint32_t vcl_hrd_parameters_present_flag: 1;
+      uint32_t low_delay_hrd_flag: 1;
+      uint32_t pic_struct_present_flag: 1;
+      uint32_t bitstream_restriction_flag: 1;
+      uint32_t motion_vectors_over_pic_boundaries_flag: 1;
    } vui_flags;
    uint32_t aspect_ratio_idc;
    uint32_t sar_width;
    uint32_t sar_height;
    uint32_t num_units_in_tick;
    uint32_t time_scale;
+   uint32_t video_format;
+   uint32_t video_full_range_flag;
+   uint32_t colour_primaries;
+   uint32_t transfer_characteristics;
+   uint32_t matrix_coefficients;
+   uint32_t chroma_sample_loc_type_top_field;
+   uint32_t chroma_sample_loc_type_bottom_field;
+   uint32_t max_num_reorder_frames;
+   pipe_h264_enc_hrd_params nal_hrd_parameters;
+   pipe_h264_enc_hrd_params vcl_hrd_parameters;
+   uint32_t max_bytes_per_pic_denom;
+   uint32_t max_bits_per_mb_denom;
+   uint32_t log2_max_mv_length_vertical;
+   uint32_t log2_max_mv_length_horizontal;
+   uint32_t max_dec_frame_buffering;
 };
 
 struct pipe_h264_enc_picture_desc
@@ -499,6 +629,10 @@ struct pipe_h264_enc_picture_desc
    struct pipe_h264_enc_pic_control pic_ctrl;
    struct pipe_h264_enc_dbk_param dbk;
 
+   unsigned intra_idr_period;
+   unsigned ip_period;
+
+   unsigned init_qp;
    unsigned quant_i_frames;
    unsigned quant_p_frames;
    unsigned quant_b_frames;
@@ -519,6 +653,8 @@ struct pipe_h264_enc_picture_desc
    bool l1_is_long_term[32];
    unsigned gop_size;
    struct pipe_enc_quality_modes quality_modes;
+   struct pipe_enc_intra_refresh intra_refresh;
+   struct pipe_enc_roi roi;
 
    bool not_referenced;
    bool is_ltr;
@@ -526,8 +662,57 @@ struct pipe_h264_enc_picture_desc
    bool enable_vui;
    struct hash_table *frame_idx;
 
+   enum pipe_video_slice_mode slice_mode;
+
+   /* Use with PIPE_VIDEO_SLICE_MODE_BLOCKS */
    unsigned num_slice_descriptors;
    struct h264_slice_descriptor slices_descriptors[128];
+
+   /* Use with PIPE_VIDEO_SLICE_MODE_MAX_SLICE_SIZE */
+   unsigned max_slice_bytes;
+
+   bool insert_aud_nalu;
+   enum pipe_video_feedback_metadata_type requested_metadata;
+   bool renew_headers_on_idr;
+};
+
+struct pipe_h265_st_ref_pic_set
+{
+   uint32_t num_neg_pics;
+   uint32_t num_pos_pics;
+};
+
+struct pipe_h265_enc_sublayer_hrd_params
+{
+    uint32_t bit_rate_value_minus1[32];
+    uint32_t cpb_size_value_minus1[32];
+    uint32_t cpb_size_du_value_minus1[32];
+    uint32_t bit_rate_du_value_minus1[32];
+    uint32_t cbr_flag[32];
+};
+
+struct pipe_h265_enc_hrd_params
+{
+   uint32_t nal_hrd_parameters_present_flag;
+   uint32_t vcl_hrd_parameters_present_flag;
+   uint32_t sub_pic_hrd_params_present_flag;
+   uint32_t tick_divisor_minus2;
+   uint32_t du_cpb_removal_delay_increment_length_minus1;
+   uint32_t sub_pic_cpb_params_in_pic_timing_sei_flag;
+   uint32_t dpb_output_delay_du_length_minus1;
+   uint32_t bit_rate_scale;
+   uint32_t cpb_rate_scale;
+   uint32_t cpb_size_du_scale;
+   uint32_t initial_cpb_removal_delay_length_minus1;
+   uint32_t au_cpb_removal_delay_length_minus1;
+   uint32_t dpb_output_delay_length_minus1;
+   uint32_t fixed_pic_rate_general_flag[7];
+   uint32_t fixed_pic_rate_within_cvs_flag[7];
+   uint32_t elemental_duration_in_tc_minus1[7];
+   uint32_t low_delay_hrd_flag[7];
+   uint32_t cpb_cnt_minus1[7];
+   struct pipe_h265_enc_sublayer_hrd_params nal_hrd_parameters[7];
+   struct pipe_h265_enc_sublayer_hrd_params vlc_hrd_parameters[7];
 };
 
 struct pipe_h265_enc_seq_param
@@ -562,12 +747,45 @@ struct pipe_h265_enc_seq_param
    struct {
       uint32_t aspect_ratio_info_present_flag: 1;
       uint32_t timing_info_present_flag: 1;
+      uint32_t video_signal_type_present_flag: 1;
+      uint32_t colour_description_present_flag: 1;
+      uint32_t chroma_loc_info_present_flag: 1;
+      uint32_t overscan_info_present_flag: 1;
+      uint32_t overscan_appropriate_flag: 1;
+      uint32_t neutral_chroma_indication_flag: 1;
+      uint32_t field_seq_flag: 1;
+      uint32_t frame_field_info_present_flag: 1;
+      uint32_t default_display_window_flag: 1;
+      uint32_t poc_proportional_to_timing_flag: 1;
+      uint32_t hrd_parameters_present_flag: 1;
+      uint32_t bitstream_restriction_flag: 1;
+      uint32_t tiles_fixed_structure_flag: 1;
+      uint32_t motion_vectors_over_pic_boundaries_flag: 1;
+      uint32_t restricted_ref_pic_lists_flag: 1;
    } vui_flags;
    uint32_t aspect_ratio_idc;
    uint32_t sar_width;
    uint32_t sar_height;
    uint32_t num_units_in_tick;
    uint32_t time_scale;
+   uint32_t video_format;
+   uint32_t video_full_range_flag;
+   uint32_t colour_primaries;
+   uint32_t transfer_characteristics;
+   uint32_t matrix_coefficients;
+   uint32_t chroma_sample_loc_type_top_field;
+   uint32_t chroma_sample_loc_type_bottom_field;
+   uint32_t def_disp_win_left_offset;
+   uint32_t def_disp_win_right_offset;
+   uint32_t def_disp_win_top_offset;
+   uint32_t def_disp_win_bottom_offset;
+   uint32_t num_ticks_poc_diff_one_minus1;
+   uint32_t min_spatial_segmentation_idc;
+   uint32_t max_bytes_per_pic_denom;
+   uint32_t max_bits_per_min_cu_denom;
+   uint32_t log2_max_mv_length_horizontal;
+   uint32_t log2_max_mv_length_vertical;
+   struct pipe_h265_enc_hrd_params hrd_parameters;
 };
 
 struct pipe_h265_enc_pic_param
@@ -598,11 +816,14 @@ struct pipe_h265_enc_rate_control
    unsigned peak_bitrate;
    unsigned frame_rate_num;
    unsigned frame_rate_den;
+   unsigned init_qp;
    unsigned quant_i_frames;
    unsigned quant_p_frames;
    unsigned quant_b_frames;
    unsigned vbv_buffer_size;
    unsigned vbv_buf_lv;
+   unsigned vbv_buf_initial_size;
+   bool app_requested_hrd_buffer;
    unsigned target_bits_picture;
    unsigned peak_bits_picture_integer;
    unsigned peak_bits_picture_fraction;
@@ -612,6 +833,10 @@ struct pipe_h265_enc_rate_control
    unsigned max_au_size;
    unsigned max_qp;
    unsigned min_qp;
+   bool app_requested_qp_range;
+
+   /* Used with PIPE_H2645_ENC_RATE_CONTROL_METHOD_QUALITY_VARIABLE */
+   unsigned vbr_quality_factor;
 };
 
 struct pipe_h265_enc_picture_desc
@@ -630,6 +855,8 @@ struct pipe_h265_enc_picture_desc
    unsigned pic_order_cnt;
    unsigned pic_order_cnt_type;
    struct pipe_enc_quality_modes quality_modes;
+   struct pipe_enc_intra_refresh intra_refresh;
+   struct pipe_enc_roi roi;
    unsigned num_ref_idx_l0_active_minus1;
    unsigned num_ref_idx_l1_active_minus1;
    unsigned ref_idx_l0_list[PIPE_H265_MAX_REFERENCES];
@@ -637,8 +864,232 @@ struct pipe_h265_enc_picture_desc
    bool not_referenced;
    struct hash_table *frame_idx;
 
+   enum pipe_video_slice_mode slice_mode;
+
+   /* Use with PIPE_VIDEO_SLICE_MODE_BLOCKS */
    unsigned num_slice_descriptors;
    struct h265_slice_descriptor slices_descriptors[128];
+
+   /* Use with PIPE_VIDEO_SLICE_MODE_MAX_SLICE_SIZE */
+   unsigned max_slice_bytes;
+   enum pipe_video_feedback_metadata_type requested_metadata;
+   bool renew_headers_on_idr;
+};
+
+struct pipe_av1_enc_rate_control
+{
+   enum pipe_h2645_enc_rate_control_method rate_ctrl_method;
+   unsigned target_bitrate;
+   unsigned peak_bitrate;
+   unsigned frame_rate_num;
+   unsigned frame_rate_den;
+   unsigned vbv_buffer_size;
+   unsigned vbv_buf_lv;
+   unsigned vbv_buf_initial_size;
+   bool app_requested_hrd_buffer;
+   unsigned target_bits_picture;
+   unsigned peak_bits_picture_integer;
+   unsigned peak_bits_picture_fraction;
+   unsigned fill_data_enable;
+   unsigned skip_frame_enable;
+   unsigned enforce_hrd;
+   unsigned max_au_size;
+   unsigned qp; /* Initial QP */
+   unsigned qp_inter;
+   unsigned max_qp;
+   unsigned min_qp;
+   bool app_requested_qp_range;
+   bool app_requested_initial_qp;
+
+   /* Used with PIPE_H2645_ENC_RATE_CONTROL_METHOD_QUALITY_VARIABLE */
+   unsigned vbr_quality_factor;
+};
+
+struct pipe_av1_enc_decoder_model_info
+{
+   uint32_t buffer_delay_length_minus1;
+   uint32_t num_units_in_decoding_tick;
+   uint32_t buffer_removal_time_length_minus1;
+   uint32_t frame_presentation_time_length_minus1;
+};
+
+struct pipe_av1_enc_color_description
+{
+   uint32_t color_primaries;
+   uint32_t transfer_characteristics;
+   uint32_t matrix_coefficients;
+   uint32_t color_range;
+   uint32_t chroma_sample_position;
+};
+struct pipe_av1_enc_seq_param
+{
+   uint32_t profile;
+   uint32_t level;
+   uint32_t tier;
+   uint32_t num_temporal_layers;
+   uint32_t intra_period;
+   uint32_t ip_period;
+   uint32_t bit_depth_minus8;
+   uint32_t pic_width_in_luma_samples;
+   uint32_t pic_height_in_luma_samples;
+   struct
+   {
+      uint32_t use_128x128_superblock:1;
+      uint32_t enable_filter_intra :1;
+      uint32_t enable_intra_edge_filter :1;
+      uint32_t enable_interintra_compound :1;
+      uint32_t enable_masked_compound :1;
+      uint32_t enable_warped_motion :1;
+      uint32_t enable_dual_filter :1;
+      uint32_t enable_cdef:1;
+      uint32_t enable_restoration:1;
+      uint32_t enable_superres:1;
+      uint32_t enable_order_hint:1;
+      uint32_t enable_jnt_comp:1;
+      uint32_t color_description_present_flag:1;
+      uint32_t enable_ref_frame_mvs:1;
+      uint32_t frame_id_number_present_flag:1;
+      uint32_t disable_screen_content_tools:1;
+      uint32_t timing_info_present_flag:1;
+      uint32_t equal_picture_interval:1;
+      uint32_t decoder_model_info_present_flag:1;
+      uint32_t force_screen_content_tools:2;
+      uint32_t force_integer_mv:2;
+   } seq_bits;
+
+   /* timing info params */
+   uint32_t num_units_in_display_tick;
+   uint32_t time_scale;
+   uint32_t num_tick_per_picture_minus1;
+   uint32_t delta_frame_id_length;
+   uint32_t additional_frame_id_length;
+   uint32_t order_hint_bits;
+   struct pipe_av1_enc_decoder_model_info decoder_model_info;
+   struct pipe_av1_enc_color_description color_config;
+   uint16_t frame_width_bits_minus1;
+   uint16_t frame_height_bits_minus1;
+   uint16_t operating_point_idc[32];
+   uint8_t decoder_model_present_for_this_op[32];
+};
+
+struct pipe_av1_tile_group {
+   uint8_t tile_group_start;
+   uint8_t tile_group_end;
+};
+
+struct pipe_av1_enc_picture_desc
+{
+   struct pipe_picture_desc base;
+   enum pipe_av1_enc_frame_type frame_type;
+   struct pipe_av1_enc_seq_param seq;
+   struct pipe_av1_enc_rate_control rc[4];
+   struct {
+      uint32_t enable_frame_obu:1;
+      uint32_t error_resilient_mode:1;
+      uint32_t disable_cdf_update:1;
+      uint32_t frame_size_override_flag:1;
+      uint32_t allow_screen_content_tools:1;
+      uint32_t allow_intrabc:1;
+      uint32_t force_integer_mv:1;
+      uint32_t disable_frame_end_update_cdf:1;
+      uint32_t palette_mode_enable:1;
+      uint32_t allow_high_precision_mv:1;
+      uint32_t use_ref_frame_mvs;
+      uint32_t show_existing_frame:1;
+      uint32_t enable_render_size:1;
+      uint32_t use_superres:1;
+      uint32_t reduced_tx_set:1;
+      uint32_t skip_mode_present:1;
+      uint32_t long_term_reference:1;
+   };
+   struct pipe_enc_quality_modes quality_modes;
+   struct pipe_enc_intra_refresh intra_refresh;
+   struct pipe_enc_roi roi;
+   uint32_t num_tiles_in_pic; /* [1, 32], */
+   uint32_t tile_rows;
+   uint32_t tile_cols;
+   unsigned num_tile_groups;
+   struct pipe_av1_tile_group tile_groups[256];
+   uint32_t context_update_tile_id;
+   uint16_t width_in_sbs_minus_1[63];
+   uint16_t height_in_sbs_minus_1[63];
+   uint32_t frame_num;
+   uint32_t last_key_frame_num;
+   uint32_t number_of_skips;
+   uint32_t temporal_id;
+   uint32_t spatial_id;
+   uint16_t frame_width;
+   uint16_t frame_height;
+   uint16_t frame_width_sb;
+   uint16_t frame_height_sb;
+   uint16_t upscaled_width;
+   uint16_t render_width;
+   uint16_t render_height;
+   uint32_t interpolation_filter;
+   uint8_t tx_mode;
+   uint8_t compound_reference_mode;
+   uint32_t order_hint;
+   uint8_t superres_scale_denominator;
+   uint32_t primary_ref_frame;
+   uint8_t refresh_frame_flags;
+   uint8_t ref_frame_idx[7];
+   uint32_t ref_frame_ctrl_l0;            /* forward prediction only */
+   void *ref_list[8];                     /* for tracking ref frames */
+   void *recon_frame;
+
+   struct {
+      uint8_t cdef_damping_minus_3;
+      uint8_t cdef_bits;
+      uint8_t cdef_y_strengths[8];
+      uint8_t cdef_uv_strengths[8];
+   } cdef;
+
+   struct {
+      uint8_t yframe_restoration_type;
+      uint8_t cbframe_restoration_type;
+      uint8_t crframe_restoration_type;
+      uint8_t lr_unit_shift;
+      uint8_t lr_uv_shift;
+   } restoration;
+
+   struct {
+      uint8_t filter_level[2];
+      uint8_t filter_level_u;
+      uint8_t filter_level_v;
+      uint8_t sharpness_level;
+      uint8_t mode_ref_delta_enabled;
+      uint8_t mode_ref_delta_update;
+      int8_t  ref_deltas[8];
+      int8_t  mode_deltas[2];
+      uint8_t delta_lf_present;
+      uint8_t delta_lf_res;
+      uint8_t delta_lf_multi;
+   } loop_filter;
+
+   struct {
+      uint8_t base_qindex;
+      int8_t y_dc_delta_q;
+      int8_t u_dc_delta_q;
+      int8_t u_ac_delta_q;
+      int8_t v_dc_delta_q;
+      int8_t v_ac_delta_q;
+      uint8_t min_base_qindex;
+      uint8_t max_base_qindex;
+      uint8_t using_qmatrix;
+      uint8_t qm_y;
+      uint8_t qm_u;
+      uint8_t qm_v;
+      uint8_t delta_q_present;
+      uint8_t delta_q_res;
+   } quantization;
+
+   struct {
+      uint8_t obu_extension_flag;
+      uint8_t obu_has_size_field;
+      uint8_t temporal_id;
+      uint8_t spatial_id;
+   } tg_obu_header;
+   enum pipe_video_feedback_metadata_type requested_metadata;
 };
 
 struct pipe_h265_sps
@@ -757,7 +1208,7 @@ struct pipe_h265_picture_desc
    uint8_t RefPicSetStCurrBefore[8];
    uint8_t RefPicSetStCurrAfter[8];
    uint8_t RefPicSetLtCurr[8];
-   uint8_t RefPicList[2][15];
+   uint8_t RefPicList[PIPE_H265_MAX_SLICES][2][15];
    bool UseRefPicList;
    bool UseStRpsBits;
 
@@ -765,9 +1216,9 @@ struct pipe_h265_picture_desc
    {
       bool slice_info_present;
       uint32_t slice_count;
-      uint32_t slice_data_size[128];
-      uint32_t slice_data_offset[128];
-      enum pipe_slice_buffer_placement_type slice_data_flag[128];
+      uint32_t slice_data_size[PIPE_H265_MAX_SLICES];
+      uint32_t slice_data_offset[PIPE_H265_MAX_SLICES];
+      enum pipe_slice_buffer_placement_type slice_data_flag[PIPE_H265_MAX_SLICES];
    } slice_parameter;
 };
 
@@ -788,6 +1239,10 @@ struct pipe_mjpeg_picture_desc
       } components[255];
 
       uint8_t num_components;
+      uint16_t crop_x;
+      uint16_t crop_y;
+      uint16_t crop_width;
+      uint16_t crop_height;
    } picture_parameter;
 
    struct
@@ -862,12 +1317,15 @@ struct pipe_vp9_picture_desc
    struct {
       uint16_t frame_width;
       uint16_t frame_height;
+      uint16_t prev_frame_width;
+      uint16_t prev_frame_height;
 
       struct {
          uint32_t  subsampling_x:1;
          uint32_t  subsampling_y:1;
          uint32_t  frame_type:1;
          uint32_t  show_frame:1;
+         uint32_t  prev_show_frame:1;
          uint32_t  error_resilient_mode:1;
          uint32_t  intra_only:1;
          uint32_t  allow_high_precision_mv:1;
@@ -1033,6 +1491,7 @@ struct pipe_av1_picture_desc
          uint32_t disable_frame_end_update_cdf:1;
          uint32_t uniform_tile_spacing_flag:1;
          uint32_t allow_warped_motion:1;
+         uint32_t large_scale_tile:1;
       } pic_info_fields;
 
       uint8_t superres_scale_denominator;
@@ -1107,6 +1566,7 @@ struct pipe_av1_picture_desc
       uint16_t slice_data_row[256];
       uint16_t slice_data_col[256];
       uint8_t slice_data_anchor_frame_idx[256];
+      uint16_t slice_count;
    } slice_parameter;
 };
 
@@ -1124,6 +1584,17 @@ struct pipe_vpp_desc
    struct u_rect dst_region;
    enum pipe_video_vpp_orientation orientation;
    struct pipe_vpp_blend blend;
+
+   /* Fence to wait on for the src surface */
+   struct pipe_fence_handle *src_surface_fence;
+
+   uint32_t background_color;
+   enum pipe_video_vpp_color_standard_type in_colors_standard;
+   enum pipe_video_vpp_color_range in_color_range;
+   enum pipe_video_vpp_chroma_siting in_chroma_siting;
+   enum pipe_video_vpp_color_standard_type out_colors_standard;
+   enum pipe_video_vpp_color_range out_color_range;
+   enum pipe_video_vpp_chroma_siting out_chroma_siting;
 };
 
 
@@ -1140,16 +1611,6 @@ enum pipe_h265_enc_pred_direction
    PIPE_H265_PRED_DIRECTION_BI_NOT_EMPTY = 0x4,
 };
 
-/* To be used on each h265 feature bit field
-   defined in pipe_h265_enc_cap_features
-*/
-enum pipe_h265_enc_feature
-{
-   PIPE_H265_ENC_FEATURE_NOT_SUPPORTED = 0x0,
-   PIPE_H265_ENC_FEATURE_SUPPORTED = 0x1,
-   PIPE_H265_ENC_FEATURE_REQUIRED = 0x2,
-};
-
 /* To be used with PIPE_VIDEO_CAP_ENC_HEVC_FEATURE_FLAGS
    the config_supported bit is used to differenciate a supported
    config with all bits as zero and unsupported by driver with value=0
@@ -1341,6 +1802,235 @@ union pipe_h265_enc_cap_block_sizes {
       uint32_t value;
 };
 
+union pipe_av1_enc_cap_features {
+    struct {
+        /**
+         * Use 128x128 superblock.
+         *
+         * Allows setting use_128x128_superblock in the SPS.
+         */
+        uint32_t support_128x128_superblock     : 2;
+        /**
+         * Intra  filter.
+         * Allows setting enable_filter_intra in the SPS.
+         */
+        uint32_t support_filter_intra           : 2;
+        /**
+         *  Intra edge filter.
+         * Allows setting enable_intra_edge_filter in the SPS.
+         */
+        uint32_t support_intra_edge_filter      : 2;
+        /**
+         *  Interintra compound.
+         * Allows setting enable_interintra_compound in the SPS.
+         */
+        uint32_t support_interintra_compound    : 2;
+        /**
+         *  Masked compound.
+         * Allows setting enable_masked_compound in the SPS.
+         */
+        uint32_t support_masked_compound        : 2;
+        /**
+         *  Warped motion.
+         * Allows setting enable_warped_motion in the SPS.
+         */
+        uint32_t support_warped_motion          : 2;
+        /**
+         *  Palette mode.
+         * Allows setting palette_mode in the PPS.
+         */
+        uint32_t support_palette_mode           : 2;
+        /**
+         *  Dual filter.
+         * Allows setting enable_dual_filter in the SPS.
+         */
+        uint32_t support_dual_filter            : 2;
+        /**
+         *  Jnt compound.
+         * Allows setting enable_jnt_comp in the SPS.
+         */
+        uint32_t support_jnt_comp               : 2;
+        /**
+         *  Refrence frame mvs.
+         * Allows setting enable_ref_frame_mvs in the SPS.
+         */
+        uint32_t support_ref_frame_mvs          : 2;
+        /**
+         *  Super resolution.
+         * Allows setting enable_superres in the SPS.
+         */
+        uint32_t support_superres               : 2;
+        /**
+         *  Restoration.
+         * Allows setting enable_restoration in the SPS.
+         */
+        uint32_t support_restoration            : 2;
+        /**
+         *  Allow intraBC.
+         * Allows setting allow_intrabc in the PPS.
+         */
+        uint32_t support_allow_intrabc          : 2;
+        /**
+         *  Cdef channel strength.
+         * Allows setting cdef_y_strengths and cdef_uv_strengths in PPS.
+         */
+        uint32_t support_cdef_channel_strength  : 2;
+        /** Reserved bits for future, must be zero. */
+        uint32_t reserved                       : 4;
+    } bits;
+    uint32_t value;
+};
+
+union pipe_av1_enc_cap_features_ext1 {
+    struct {
+        /**
+         * Fields indicate which types of interpolation filter are supported.
+         * (interpolation_filter & 0x01) == 1: eight_tap filter is supported, 0: not.
+         * (interpolation_filter & 0x02) == 1: eight_tap_smooth filter is supported, 0: not.
+         * (interpolation_filter & 0x04) == 1: eight_sharp filter is supported, 0: not.
+         * (interpolation_filter & 0x08) == 1: bilinear filter is supported, 0: not.
+         * (interpolation_filter & 0x10) == 1: switchable filter is supported, 0: not.
+         */
+        uint32_t interpolation_filter          : 5;
+        /**
+         * Min segmentId block size accepted.
+         * Application need to send seg_id_block_size in PPS equal or larger than this value.
+         */
+        uint32_t min_segid_block_size_accepted : 8;
+        /**
+         * Type of segment feature supported.
+         * (segment_feature_support & 0x01) == 1: SEG_LVL_ALT_Q is supported, 0: not.
+         * (segment_feature_support & 0x02) == 1: SEG_LVL_ALT_LF_Y_V is supported, 0: not.
+         * (segment_feature_support & 0x04) == 1: SEG_LVL_ALT_LF_Y_H is supported, 0: not.
+         * (segment_feature_support & 0x08) == 1: SEG_LVL_ALT_LF_U is supported, 0: not.
+         * (segment_feature_support & 0x10) == 1: SEG_LVL_ALT_LF_V is supported, 0: not.
+         * (segment_feature_support & 0x20) == 1: SEG_LVL_REF_FRAME is supported, 0: not.
+         * (segment_feature_support & 0x40) == 1: SEG_LVL_SKIP is supported, 0: not.
+         * (segment_feature_support & 0x80) == 1: SEG_LVL_GLOBALMV is supported, 0: not.
+         */
+        uint32_t segment_feature_support       : 8;
+        /** Reserved bits for future, must be zero. */
+        uint32_t reserved                      : 11;
+    } bits;
+    uint32_t value;
+};
+
+union pipe_av1_enc_cap_features_ext2 {
+    struct {
+        /**
+        * Tile size bytes minus1.
+        * Specify the number of bytes needed to code tile size supported.
+        * This value need to be set in frame header obu.
+        */
+        uint32_t tile_size_bytes_minus1        : 2;
+        /**
+        * Tile size bytes minus1.
+        * Specify the fixed number of bytes needed to code syntax obu_size.
+        */
+        uint32_t obu_size_bytes_minus1         : 2;
+        /**
+         * tx_mode supported.
+         * (tx_mode_support & 0x01) == 1: ONLY_4X4 is supported, 0: not.
+         * (tx_mode_support & 0x02) == 1: TX_MODE_LARGEST is supported, 0: not.
+         * (tx_mode_support & 0x04) == 1: TX_MODE_SELECT is supported, 0: not.
+         */
+        uint32_t tx_mode_support               : 3;
+        /**
+         * Max tile num minus1.
+         * Specify the max number of tile supported by driver.
+         */
+        uint32_t max_tile_num_minus1           : 13;
+        /** Reserved bits for future, must be zero. */
+        uint32_t reserved                      : 12;
+    } bits;
+    uint32_t value;
+};
+
+struct codec_unit_location_t
+{
+   uint64_t offset;
+   uint64_t size;
+   enum codec_unit_location_flags flags;
+};
+
+struct pipe_enc_feedback_metadata
+{
+   /*
+   * Driver writes the metadata types present in this struct
+   */
+   enum pipe_video_feedback_metadata_type present_metadata;
+
+   /*
+    * Driver writes the result of encoding the associated frame.
+    * Requires PIPE_VIDEO_FEEDBACK_METADATA_TYPE_ENCODE_RESULT
+    */
+   enum pipe_video_feedback_encode_result_flags encode_result;
+
+   /*
+    * Driver fills in with coded headers information
+    * and a number codec_unit_metadata_count of valid entries
+    * Requires PIPE_VIDEO_FEEDBACK_METADATA_TYPE_CODEC_UNIT_LOCATION
+    */
+   struct codec_unit_location_t codec_unit_metadata[256];
+   unsigned codec_unit_metadata_count;
+
+   /*
+   * Driver writes the average QP used to encode this frame
+   */
+   unsigned int average_frame_qp;
+};
+
+union pipe_enc_cap_roi {
+   struct {
+      /**
+       * The number of ROI regions supported, 0 if ROI is not supported
+       */
+      uint32_t num_roi_regions                 : 8;
+      /**
+       * A flag indicates whether ROI priority is supported
+       *
+       * roi_rc_priority_support equal to 1 specifies the underlying driver supports
+       * ROI priority when VAConfigAttribRateControl != VA_RC_CQP, user can use roi_value
+       * in #VAEncROI to set ROI priority. roi_rc_priority_support equal to 0 specifies
+       * the underlying driver doesn't support ROI priority.
+       *
+       * User should ignore roi_rc_priority_support when VAConfigAttribRateControl == VA_RC_CQP
+       * because ROI delta QP is always required when VAConfigAttribRateControl == VA_RC_CQP.
+       */
+      uint32_t roi_rc_priority_support         : 1;
+      /**
+       * A flag indicates whether ROI delta QP is supported
+       *
+       * roi_rc_qp_delta_support equal to 1 specifies the underlying driver supports
+       * ROI delta QP when VAConfigAttribRateControl != VA_RC_CQP, user can use roi_value
+       * in #VAEncROI to set ROI delta QP. roi_rc_qp_delta_support equal to 0 specifies
+       * the underlying driver doesn't support ROI delta QP.
+       *
+       * User should ignore roi_rc_qp_delta_support when VAConfigAttribRateControl == VA_RC_CQP
+       * because ROI delta QP is always required when VAConfigAttribRateControl == VA_RC_CQP.
+       */
+      uint32_t roi_rc_qp_delta_support         : 1;
+      uint32_t reserved                        : 22;
+
+   } bits;
+   uint32_t value;
+};
+
+union pipe_enc_cap_surface_alignment {
+   struct {
+      /**
+       * log2_width_alignment
+       */
+      uint32_t log2_width_alignment                 : 4;
+      /**
+       * log2_height_alignment
+       */
+      uint32_t log2_height_alignment                : 4;
+      uint32_t reserved                             : 24;
+   } bits;
+   uint32_t value;
+};
+
 #ifdef __cplusplus
 }
 #endif
