@@ -59,13 +59,6 @@
 #define GET_BITS(data, high, low) ((data & INTEL_MASK((high), (low))) >> (low))
 #define GET_FIELD(word, field) (((word)  & field ## _MASK) >> field ## _SHIFT)
 
-/* Bitfields for the URB_WRITE message, DW2 of message header: */
-#define URB_WRITE_PRIM_END		0x1
-#define URB_WRITE_PRIM_START		0x2
-#define URB_WRITE_PRIM_TYPE_SHIFT	2
-
-#define BRW_SPRITE_POINT_ENABLE  16
-
 # define GFX7_GS_CONTROL_DATA_FORMAT_GSCTL_CUT		0
 # define GFX7_GS_CONTROL_DATA_FORMAT_GSCTL_SID		1
 
@@ -114,11 +107,6 @@ enum ENUM_PACKED brw_conditional_mod {
 #define BRW_DEBUG_NONE        0
 #define BRW_DEBUG_BREAKPOINT  1
 
-#define BRW_DEPENDENCY_NORMAL         0
-#define BRW_DEPENDENCY_NOTCLEARED     1
-#define BRW_DEPENDENCY_NOTCHECKED     2
-#define BRW_DEPENDENCY_DISABLE        3
-
 enum ENUM_PACKED brw_execution_size {
    BRW_EXECUTE_1  = 0,
    BRW_EXECUTE_2  = 1,
@@ -180,55 +168,45 @@ enum opcode {
    BRW_OPCODE_SYNC,
    BRW_OPCODE_MOV,
    BRW_OPCODE_SEL,
-   BRW_OPCODE_MOVI, /**< G45+ */
+   BRW_OPCODE_MOVI,
    BRW_OPCODE_NOT,
    BRW_OPCODE_AND,
    BRW_OPCODE_OR,
    BRW_OPCODE_XOR,
    BRW_OPCODE_SHR,
    BRW_OPCODE_SHL,
-   BRW_OPCODE_DIM, /**< Gfx7.5 only */
-   BRW_OPCODE_SMOV, /**< Gfx8+ */
+   BRW_OPCODE_SMOV,
    BRW_OPCODE_ASR,
    BRW_OPCODE_ROR,  /**< Gfx11+ */
    BRW_OPCODE_ROL,  /**< Gfx11+ */
    BRW_OPCODE_CMP,
    BRW_OPCODE_CMPN,
-   BRW_OPCODE_CSEL, /**< Gfx8+ */
-   BRW_OPCODE_F32TO16, /**< Gfx7 only */
-   BRW_OPCODE_F16TO32, /**< Gfx7 only */
-   BRW_OPCODE_BFREV, /**< Gfx7+ */
-   BRW_OPCODE_BFE, /**< Gfx7+ */
-   BRW_OPCODE_BFI1, /**< Gfx7+ */
-   BRW_OPCODE_BFI2, /**< Gfx7+ */
+   BRW_OPCODE_CSEL,
+   BRW_OPCODE_BFREV,
+   BRW_OPCODE_BFE,
+   BRW_OPCODE_BFI1,
+   BRW_OPCODE_BFI2,
    BRW_OPCODE_JMPI,
-   BRW_OPCODE_BRD, /**< Gfx7+ */
+   BRW_OPCODE_BRD,
    BRW_OPCODE_IF,
-   BRW_OPCODE_IFF, /**< Pre-Gfx6 */
-   BRW_OPCODE_BRC, /**< Gfx7+ */
+   BRW_OPCODE_BRC,
    BRW_OPCODE_ELSE,
    BRW_OPCODE_ENDIF,
-   BRW_OPCODE_DO, /**< Pre-Gfx6 */
-   BRW_OPCODE_CASE, /**< Gfx6 only */
+   BRW_OPCODE_DO, /**< Used as pseudo opcode, will be moved later. */
    BRW_OPCODE_WHILE,
    BRW_OPCODE_BREAK,
    BRW_OPCODE_CONTINUE,
    BRW_OPCODE_HALT,
-   BRW_OPCODE_CALLA, /**< Gfx7.5+ */
-   BRW_OPCODE_MSAVE, /**< Pre-Gfx6 */
-   BRW_OPCODE_CALL, /**< Gfx6+ */
-   BRW_OPCODE_MREST, /**< Pre-Gfx6 */
-   BRW_OPCODE_RET, /**< Gfx6+ */
-   BRW_OPCODE_PUSH, /**< Pre-Gfx6 */
-   BRW_OPCODE_FORK, /**< Gfx6 only */
-   BRW_OPCODE_GOTO, /**< Gfx8+ */
-   BRW_OPCODE_POP, /**< Pre-Gfx6 */
+   BRW_OPCODE_CALLA,
+   BRW_OPCODE_CALL,
+   BRW_OPCODE_RET,
+   BRW_OPCODE_GOTO,
    BRW_OPCODE_WAIT,
    BRW_OPCODE_SEND,
    BRW_OPCODE_SENDC,
-   BRW_OPCODE_SENDS, /**< Gfx9+ */
-   BRW_OPCODE_SENDSC, /**< Gfx9+ */
-   BRW_OPCODE_MATH, /**< Gfx6+ */
+   BRW_OPCODE_SENDS,
+   BRW_OPCODE_SENDSC,
+   BRW_OPCODE_MATH,
    BRW_OPCODE_ADD,
    BRW_OPCODE_MUL,
    BRW_OPCODE_AVG,
@@ -240,11 +218,11 @@ enum opcode {
    BRW_OPCODE_MAC,
    BRW_OPCODE_MACH,
    BRW_OPCODE_LZD,
-   BRW_OPCODE_FBH, /**< Gfx7+ */
-   BRW_OPCODE_FBL, /**< Gfx7+ */
-   BRW_OPCODE_CBIT, /**< Gfx7+ */
-   BRW_OPCODE_ADDC, /**< Gfx7+ */
-   BRW_OPCODE_SUBB, /**< Gfx7+ */
+   BRW_OPCODE_FBH,
+   BRW_OPCODE_FBL,
+   BRW_OPCODE_CBIT,
+   BRW_OPCODE_ADDC,
+   BRW_OPCODE_SUBB,
    BRW_OPCODE_SAD2,
    BRW_OPCODE_SADA2,
    BRW_OPCODE_ADD3, /* Gen12+ only */
@@ -254,31 +232,21 @@ enum opcode {
    BRW_OPCODE_DP2,
    BRW_OPCODE_DP4A, /**< Gfx12+ */
    BRW_OPCODE_LINE,
-   BRW_OPCODE_PLN, /**< G45+ */
-   BRW_OPCODE_MAD, /**< Gfx6+ */
-   BRW_OPCODE_LRP, /**< Gfx6+ */
-   BRW_OPCODE_MADM, /**< Gfx8+ */
-   BRW_OPCODE_NENOP, /**< G45 only */
+   BRW_OPCODE_DPAS,  /**< Gfx12.5+ */
+   BRW_OPCODE_PLN, /**< Up until Gfx9 */
+   BRW_OPCODE_MAD,
+   BRW_OPCODE_LRP,
+   BRW_OPCODE_MADM,
    BRW_OPCODE_NOP,
 
    NUM_BRW_OPCODES,
 
-   /* These are compiler backend opcodes that get translated into other
-    * instructions.
-    */
-   FS_OPCODE_FB_WRITE = NUM_BRW_OPCODES,
-
    /**
-    * Same as FS_OPCODE_FB_WRITE but expects its arguments separately as
-    * individual sources instead of as a single payload blob. The
-    * position/ordering of the arguments are defined by the enum
-    * fb_write_logical_srcs.
+    * The position/ordering of the arguments are defined
+    * by the enum fb_write_logical_srcs.
     */
-   FS_OPCODE_FB_WRITE_LOGICAL,
+   FS_OPCODE_FB_WRITE_LOGICAL = NUM_BRW_OPCODES,
 
-   FS_OPCODE_REP_FB_WRITE,
-
-   FS_OPCODE_FB_READ,
    FS_OPCODE_FB_READ_LOGICAL,
 
    SHADER_OPCODE_RCP,
@@ -310,41 +278,26 @@ enum opcode {
    /**
     * Texture sampling opcodes.
     *
-    * LOGICAL opcodes are eventually translated to the matching non-LOGICAL
-    * opcode but instead of taking a single payload blob they expect their
-    * arguments separately as individual sources. The position/ordering of the
-    * arguments are defined by the enum tex_logical_srcs.
+    * LOGICAL opcodes are eventually translated to SHADER_OPCODE_SEND but
+    * take parameters as individual sources.  See enum tex_logical_srcs.
     */
-   SHADER_OPCODE_TEX,
    SHADER_OPCODE_TEX_LOGICAL,
-   SHADER_OPCODE_TXD,
    SHADER_OPCODE_TXD_LOGICAL,
-   SHADER_OPCODE_TXF,
    SHADER_OPCODE_TXF_LOGICAL,
-   SHADER_OPCODE_TXF_LZ,
-   SHADER_OPCODE_TXL,
    SHADER_OPCODE_TXL_LOGICAL,
-   SHADER_OPCODE_TXL_LZ,
-   SHADER_OPCODE_TXS,
    SHADER_OPCODE_TXS_LOGICAL,
-   FS_OPCODE_TXB,
    FS_OPCODE_TXB_LOGICAL,
-   SHADER_OPCODE_TXF_CMS,
-   SHADER_OPCODE_TXF_CMS_LOGICAL,
-   SHADER_OPCODE_TXF_CMS_W,
    SHADER_OPCODE_TXF_CMS_W_LOGICAL,
    SHADER_OPCODE_TXF_CMS_W_GFX12_LOGICAL,
-   SHADER_OPCODE_TXF_UMS,
-   SHADER_OPCODE_TXF_UMS_LOGICAL,
-   SHADER_OPCODE_TXF_MCS,
    SHADER_OPCODE_TXF_MCS_LOGICAL,
-   SHADER_OPCODE_LOD,
    SHADER_OPCODE_LOD_LOGICAL,
-   SHADER_OPCODE_TG4,
    SHADER_OPCODE_TG4_LOGICAL,
-   SHADER_OPCODE_TG4_OFFSET,
+   SHADER_OPCODE_TG4_IMPLICIT_LOD_LOGICAL,
+   SHADER_OPCODE_TG4_EXPLICIT_LOD_LOGICAL,
+   SHADER_OPCODE_TG4_BIAS_LOGICAL,
    SHADER_OPCODE_TG4_OFFSET_LOGICAL,
-   SHADER_OPCODE_SAMPLEINFO,
+   SHADER_OPCODE_TG4_OFFSET_LOD_LOGICAL,
+   SHADER_OPCODE_TG4_OFFSET_BIAS_LOGICAL,
    SHADER_OPCODE_SAMPLEINFO_LOGICAL,
 
    SHADER_OPCODE_IMAGE_SIZE_LOGICAL,
@@ -386,11 +339,8 @@ enum opcode {
     * Source 4: [required] Opcode-specific control immediate, same as source 2
     *                      of the matching non-LOGICAL opcode.
     */
-   VEC4_OPCODE_UNTYPED_ATOMIC,
    SHADER_OPCODE_UNTYPED_ATOMIC_LOGICAL,
-   VEC4_OPCODE_UNTYPED_SURFACE_READ,
    SHADER_OPCODE_UNTYPED_SURFACE_READ_LOGICAL,
-   VEC4_OPCODE_UNTYPED_SURFACE_WRITE,
    SHADER_OPCODE_UNTYPED_SURFACE_WRITE_LOGICAL,
 
    SHADER_OPCODE_UNALIGNED_OWORD_BLOCK_READ_LOGICAL,
@@ -453,10 +403,6 @@ enum opcode {
     */
    FS_OPCODE_SCHEDULING_FENCE,
 
-   SHADER_OPCODE_GFX4_SCRATCH_READ,
-   SHADER_OPCODE_GFX4_SCRATCH_WRITE,
-   SHADER_OPCODE_GFX7_SCRATCH_READ,
-
    SHADER_OPCODE_SCRATCH_HEADER,
 
    /**
@@ -478,6 +424,14 @@ enum opcode {
     */
    SHADER_OPCODE_FIND_LAST_LIVE_CHANNEL,
 
+   /**
+    * Return the current execution mask and assign it to the first component
+    * of the destination.
+    *
+    * \sa opcode::FS_OPCODE_LOAD_LIVE_CHANNELS
+    */
+   SHADER_OPCODE_LOAD_LIVE_CHANNELS,
+
    /**
     * Return the current execution mask in the specified flag subregister.
     * Can be CSE'ed more easily than a plain MOV from the ce0 ARF register.
@@ -534,20 +488,6 @@ enum opcode {
     */
    SHADER_OPCODE_HALT_TARGET,
 
-   VEC4_OPCODE_MOV_BYTES,
-   VEC4_OPCODE_PACK_BYTES,
-   VEC4_OPCODE_UNPACK_UNIFORM,
-   VEC4_OPCODE_DOUBLE_TO_F32,
-   VEC4_OPCODE_DOUBLE_TO_D32,
-   VEC4_OPCODE_DOUBLE_TO_U32,
-   VEC4_OPCODE_TO_DOUBLE,
-   VEC4_OPCODE_PICK_LOW_32BIT,
-   VEC4_OPCODE_PICK_HIGH_32BIT,
-   VEC4_OPCODE_SET_LOW_32BIT,
-   VEC4_OPCODE_SET_HIGH_32BIT,
-   VEC4_OPCODE_MOV_FOR_SCRATCH,
-   VEC4_OPCODE_ZERO_OOB_PUSH_REGS,
-
    FS_OPCODE_DDX_COARSE,
    FS_OPCODE_DDX_FINE,
    /**
@@ -555,182 +495,15 @@ enum opcode {
     */
    FS_OPCODE_DDY_COARSE,
    FS_OPCODE_DDY_FINE,
-   FS_OPCODE_LINTERP,
    FS_OPCODE_PIXEL_X,
    FS_OPCODE_PIXEL_Y,
    FS_OPCODE_UNIFORM_PULL_CONSTANT_LOAD,
-   FS_OPCODE_VARYING_PULL_CONSTANT_LOAD_GFX4,
    FS_OPCODE_VARYING_PULL_CONSTANT_LOAD_LOGICAL,
-   FS_OPCODE_SET_SAMPLE_ID,
    FS_OPCODE_PACK_HALF_2x16_SPLIT,
    FS_OPCODE_INTERPOLATE_AT_SAMPLE,
    FS_OPCODE_INTERPOLATE_AT_SHARED_OFFSET,
    FS_OPCODE_INTERPOLATE_AT_PER_SLOT_OFFSET,
 
-   VEC4_VS_OPCODE_URB_WRITE,
-   VS_OPCODE_PULL_CONSTANT_LOAD,
-   VS_OPCODE_PULL_CONSTANT_LOAD_GFX7,
-
-   VS_OPCODE_UNPACK_FLAGS_SIMD4X2,
-
-   /**
-    * Write geometry shader output data to the URB.
-    *
-    * Unlike VEC4_VS_OPCODE_URB_WRITE, this opcode doesn't do an implied move from
-    * R0 to the first MRF.  This allows the geometry shader to override the
-    * "Slot {0,1} Offset" fields in the message header.
-    */
-   VEC4_GS_OPCODE_URB_WRITE,
-
-   /**
-    * Write geometry shader output data to the URB and request a new URB
-    * handle (gfx6).
-    *
-    * This opcode doesn't do an implied move from R0 to the first MRF.
-    */
-   VEC4_GS_OPCODE_URB_WRITE_ALLOCATE,
-
-   /**
-    * Terminate the geometry shader thread by doing an empty URB write.
-    *
-    * This opcode doesn't do an implied move from R0 to the first MRF.  This
-    * allows the geometry shader to override the "GS Number of Output Vertices
-    * for Slot {0,1}" fields in the message header.
-    */
-   GS_OPCODE_THREAD_END,
-
-   /**
-    * Set the "Slot {0,1} Offset" fields of a URB_WRITE message header.
-    *
-    * - dst is the MRF containing the message header.
-    *
-    * - src0.x indicates which portion of the URB should be written to (e.g. a
-    *   vertex number)
-    *
-    * - src1 is an immediate multiplier which will be applied to src0
-    *   (e.g. the size of a single vertex in the URB).
-    *
-    * Note: the hardware will apply this offset *in addition to* the offset in
-    * vec4_instruction::offset.
-    */
-   GS_OPCODE_SET_WRITE_OFFSET,
-
-   /**
-    * Set the "GS Number of Output Vertices for Slot {0,1}" fields of a
-    * URB_WRITE message header.
-    *
-    * - dst is the MRF containing the message header.
-    *
-    * - src0.x is the vertex count.  The upper 16 bits will be ignored.
-    */
-   GS_OPCODE_SET_VERTEX_COUNT,
-
-   /**
-    * Set DWORD 2 of dst to the value in src.
-    */
-   GS_OPCODE_SET_DWORD_2,
-
-   /**
-    * Prepare the dst register for storage in the "Channel Mask" fields of a
-    * URB_WRITE message header.
-    *
-    * DWORD 4 of dst is shifted left by 4 bits, so that later,
-    * GS_OPCODE_SET_CHANNEL_MASKS can OR DWORDs 0 and 4 together to form the
-    * final channel mask.
-    *
-    * Note: since GS_OPCODE_SET_CHANNEL_MASKS ORs DWORDs 0 and 4 together to
-    * form the final channel mask, DWORDs 0 and 4 of the dst register must not
-    * have any extraneous bits set prior to execution of this opcode (that is,
-    * they should be in the range 0x0 to 0xf).
-    */
-   GS_OPCODE_PREPARE_CHANNEL_MASKS,
-
-   /**
-    * Set the "Channel Mask" fields of a URB_WRITE message header.
-    *
-    * - dst is the MRF containing the message header.
-    *
-    * - src.x is the channel mask, as prepared by
-    *   GS_OPCODE_PREPARE_CHANNEL_MASKS.  DWORDs 0 and 4 are OR'ed together to
-    *   form the final channel mask.
-    */
-   GS_OPCODE_SET_CHANNEL_MASKS,
-
-   /**
-    * Get the "Instance ID" fields from the payload.
-    *
-    * - dst is the GRF for gl_InvocationID.
-    */
-   GS_OPCODE_GET_INSTANCE_ID,
-
-   /**
-    * Send a FF_SYNC message to allocate initial URB handles (gfx6).
-    *
-    * - dst will be used as the writeback register for the FF_SYNC operation.
-    *
-    * - src0 is the number of primitives written.
-    *
-    * - src1 is the value to hold in M0.0: number of SO vertices to write
-    *   and number of SO primitives needed. Its value will be overwritten
-    *   with the SVBI values if transform feedback is enabled.
-    *
-    * Note: This opcode uses an implicit MRF register for the ff_sync message
-    * header, so the caller is expected to set inst->base_mrf and initialize
-    * that MRF register to r0. This opcode will also write to this MRF register
-    * to include the allocated URB handle so it can then be reused directly as
-    * the header in the URB write operation we are allocating the handle for.
-    */
-   GS_OPCODE_FF_SYNC,
-
-   /**
-    * Move r0.1 (which holds PrimitiveID information in gfx6) to a separate
-    * register.
-    *
-    * - dst is the GRF where PrimitiveID information will be moved.
-    */
-   GS_OPCODE_SET_PRIMITIVE_ID,
-
-   /**
-    * Write transform feedback data to the SVB by sending a SVB WRITE message.
-    * Used in gfx6.
-    *
-    * - dst is the MRF register containing the message header.
-    *
-    * - src0 is the register where the vertex data is going to be copied from.
-    *
-    * - src1 is the destination register when write commit occurs.
-    */
-   GS_OPCODE_SVB_WRITE,
-
-   /**
-    * Set destination index in the SVB write message payload (M0.5). Used
-    * in gfx6 for transform feedback.
-    *
-    * - dst is the header to save the destination indices for SVB WRITE.
-    * - src is the register that holds the destination indices value.
-    */
-   GS_OPCODE_SVB_SET_DST_INDEX,
-
-   /**
-    * Prepare Mx.0 subregister for being used in the FF_SYNC message header.
-    * Used in gfx6 for transform feedback.
-    *
-    * - dst will hold the register with the final Mx.0 value.
-    *
-    * - src0 has the number of vertices emitted in SO (NumSOVertsToWrite)
-    *
-    * - src1 has the number of needed primitives for SO (NumSOPrimsNeeded)
-    *
-    * - src2 is the value to hold in M0: number of SO vertices to write
-    *   and number of SO primitives needed.
-    */
-   GS_OPCODE_FF_SYNC_SET_PRIMITIVES,
-
-   /**
-    * Terminate the compute shader.
-    */
-   CS_OPCODE_CS_TERMINATE,
-
    /**
     * GLSL barrier()
     */
@@ -760,88 +533,14 @@ enum opcode {
    /** Fills out a relocatable immediate */
    SHADER_OPCODE_MOV_RELOC_IMM,
 
-   VEC4_OPCODE_URB_READ,
-   TCS_OPCODE_GET_INSTANCE_ID,
-   VEC4_TCS_OPCODE_URB_WRITE,
-   VEC4_TCS_OPCODE_SET_INPUT_URB_OFFSETS,
-   VEC4_TCS_OPCODE_SET_OUTPUT_URB_OFFSETS,
-   TCS_OPCODE_GET_PRIMITIVE_ID,
-   TCS_OPCODE_CREATE_BARRIER_HEADER,
-   TCS_OPCODE_SRC0_010_IS_ZERO,
-   TCS_OPCODE_RELEASE_INPUT,
-   TCS_OPCODE_THREAD_END,
-
-   TES_OPCODE_GET_PRIMITIVE_ID,
-   TES_OPCODE_CREATE_INPUT_READ_HEADER,
-   TES_OPCODE_ADD_INDIRECT_URB_OFFSET,
-
    SHADER_OPCODE_BTD_SPAWN_LOGICAL,
    SHADER_OPCODE_BTD_RETIRE_LOGICAL,
 
-   SHADER_OPCODE_READ_SR_REG,
+   SHADER_OPCODE_READ_ARCH_REG,
 
    RT_OPCODE_TRACE_RAY_LOGICAL,
 };
 
-enum brw_urb_write_flags {
-   BRW_URB_WRITE_NO_FLAGS = 0,
-
-   /**
-    * Causes a new URB entry to be allocated, and its address stored in the
-    * destination register (gen < 7).
-    */
-   BRW_URB_WRITE_ALLOCATE = 0x1,
-
-   /**
-    * Causes the current URB entry to be deallocated (gen < 7).
-    */
-   BRW_URB_WRITE_UNUSED = 0x2,
-
-   /**
-    * Causes the thread to terminate.
-    */
-   BRW_URB_WRITE_EOT = 0x4,
-
-   /**
-    * Indicates that the given URB entry is complete, and may be sent further
-    * down the 3D pipeline (gen < 7).
-    */
-   BRW_URB_WRITE_COMPLETE = 0x8,
-
-   /**
-    * Indicates that an additional offset (which may be different for the two
-    * vec4 slots) is stored in the message header (gen == 7).
-    */
-   BRW_URB_WRITE_PER_SLOT_OFFSET = 0x10,
-
-   /**
-    * Indicates that the channel masks in the URB_WRITE message header should
-    * not be overridden to 0xff (gen == 7).
-    */
-   BRW_URB_WRITE_USE_CHANNEL_MASKS = 0x20,
-
-   /**
-    * Indicates that the data should be sent to the URB using the
-    * URB_WRITE_OWORD message rather than URB_WRITE_HWORD (gen == 7).  This
-    * causes offsets to be interpreted as multiples of an OWORD instead of an
-    * HWORD, and only allows one OWORD to be written.
-    */
-   BRW_URB_WRITE_OWORD = 0x40,
-
-   /**
-    * Convenient combination of flags: end the thread while simultaneously
-    * marking the given URB entry as complete.
-    */
-   BRW_URB_WRITE_EOT_COMPLETE = BRW_URB_WRITE_EOT | BRW_URB_WRITE_COMPLETE,
-
-   /**
-    * Convenient combination of flags: mark the given URB entry as complete
-    * and simultaneously allocate a new one.
-    */
-   BRW_URB_WRITE_ALLOCATE_COMPLETE =
-      BRW_URB_WRITE_ALLOCATE | BRW_URB_WRITE_COMPLETE,
-};
-
 enum fb_write_logical_srcs {
    FB_WRITE_LOGICAL_SRC_COLOR0,      /* REQUIRED */
    FB_WRITE_LOGICAL_SRC_COLOR1,      /* for dual source blend messages */
@@ -883,6 +582,8 @@ enum tex_logical_srcs {
    TEX_LOGICAL_SRC_COORD_COMPONENTS,
    /** REQUIRED: Number of derivative components (as UD immediate) */
    TEX_LOGICAL_SRC_GRAD_COMPONENTS,
+   /** REQUIRED: request residency (as UD immediate) */
+   TEX_LOGICAL_SRC_RESIDENCY,
 
    TEX_LOGICAL_NUM_SRCS,
 };
@@ -976,26 +677,25 @@ enum rt_logical_srcs {
 
 enum urb_logical_srcs {
    URB_LOGICAL_SRC_HANDLE,
+   /** Offset in bytes on Xe2+ or OWords on older platforms */
    URB_LOGICAL_SRC_PER_SLOT_OFFSETS,
    URB_LOGICAL_SRC_CHANNEL_MASK,
    /** Data to be written.  BAD_FILE for reads. */
    URB_LOGICAL_SRC_DATA,
-
+   URB_LOGICAL_SRC_COMPONENTS,
    URB_LOGICAL_NUM_SRCS
 };
 
+enum interpolator_logical_srcs {
+   /** Interpolation offset */
+   INTERP_SRC_OFFSET,
+   /** Message data  */
+   INTERP_SRC_MSG_DESC,
+   /** Flag register for dynamic mode */
+   INTERP_SRC_DYNAMIC_MODE,
 
-#ifdef __cplusplus
-/**
- * Allow brw_urb_write_flags enums to be ORed together.
- */
-inline brw_urb_write_flags
-operator|(brw_urb_write_flags x, brw_urb_write_flags y)
-{
-   return static_cast<brw_urb_write_flags>(static_cast<int>(x) |
-                                           static_cast<int>(y));
-}
-#endif
+   INTERP_NUM_SRCS
+};
 
 enum ENUM_PACKED brw_predicate {
    BRW_PREDICATE_NONE                =  0,
@@ -1018,17 +718,17 @@ enum ENUM_PACKED brw_predicate {
    BRW_PREDICATE_ALIGN16_REPLICATE_W =  5,
    BRW_PREDICATE_ALIGN16_ANY4H       =  6,
    BRW_PREDICATE_ALIGN16_ALL4H       =  7,
+   XE2_PREDICATE_ANY = 2,
+   XE2_PREDICATE_ALL = 3
 };
 
 enum ENUM_PACKED brw_reg_file {
    BRW_ARCHITECTURE_REGISTER_FILE = 0,
    BRW_GENERAL_REGISTER_FILE      = 1,
-   BRW_MESSAGE_REGISTER_FILE      = 2,
    BRW_IMMEDIATE_VALUE            = 3,
 
    ARF = BRW_ARCHITECTURE_REGISTER_FILE,
    FIXED_GRF = BRW_GENERAL_REGISTER_FILE,
-   MRF = BRW_MESSAGE_REGISTER_FILE,
    IMM = BRW_IMMEDIATE_VALUE,
 
    /* These are not hardware values */
@@ -1068,15 +768,6 @@ enum ENUM_PACKED gfx10_align1_3src_exec_type {
 #define BRW_ARF_TDR                   0xB0
 #define BRW_ARF_TIMESTAMP             0xC0
 
-#define BRW_MRF_COMPR4			(1 << 7)
-
-#define BRW_AMASK   0
-#define BRW_IMASK   1
-#define BRW_LMASK   2
-#define BRW_CMASK   3
-
-
-
 #define BRW_THREAD_NORMAL     0
 #define BRW_THREAD_ATOMIC     1
 #define BRW_THREAD_SWITCH     2
@@ -1124,6 +815,24 @@ enum tgl_sbid_mode {
    TGL_SBID_SET = 4
 };
 
+
+enum gfx12_sub_byte_precision {
+   BRW_SUB_BYTE_PRECISION_NONE = 0,
+
+   /** 4 bits. Signedness determined by base type */
+   BRW_SUB_BYTE_PRECISION_4BIT = 1,
+
+   /** 2 bits. Signedness determined by base type */
+   BRW_SUB_BYTE_PRECISION_2BIT = 2,
+};
+
+enum gfx12_systolic_depth {
+   BRW_SYSTOLIC_DEPTH_16 = 0,
+   BRW_SYSTOLIC_DEPTH_2 = 1,
+   BRW_SYSTOLIC_DEPTH_4 = 2,
+   BRW_SYSTOLIC_DEPTH_8 = 3,
+};
+
 #ifdef __cplusplus
 /**
  * Allow bitwise arithmetic of tgl_sbid_mode enums.
@@ -1166,6 +875,7 @@ enum tgl_pipe {
    TGL_PIPE_FLOAT,
    TGL_PIPE_INT,
    TGL_PIPE_LONG,
+   TGL_PIPE_MATH,
    TGL_PIPE_ALL
 };
 
@@ -1176,7 +886,7 @@ enum tgl_pipe {
 struct tgl_swsb {
    unsigned regdist : 3;
    enum tgl_pipe pipe : 3;
-   unsigned sbid : 4;
+   unsigned sbid : 5;
    enum tgl_sbid_mode mode : 3;
 };
 
@@ -1243,21 +953,46 @@ tgl_swsb_src_dep(struct tgl_swsb swsb)
  * Convert the provided tgl_swsb to the hardware's binary representation of an
  * SWSB annotation.
  */
-static inline uint8_t
+static inline uint32_t
 tgl_swsb_encode(const struct intel_device_info *devinfo, struct tgl_swsb swsb)
 {
    if (!swsb.mode) {
       const unsigned pipe = devinfo->verx10 < 125 ? 0 :
          swsb.pipe == TGL_PIPE_FLOAT ? 0x10 :
          swsb.pipe == TGL_PIPE_INT ? 0x18 :
-         swsb.pipe == TGL_PIPE_LONG ? 0x50 :
+         swsb.pipe == TGL_PIPE_LONG ? 0x20 :
+         swsb.pipe == TGL_PIPE_MATH ? 0x28 :
          swsb.pipe == TGL_PIPE_ALL ? 0x8 : 0;
       return pipe | swsb.regdist;
+
    } else if (swsb.regdist) {
-      return 0x80 | swsb.regdist << 4 | swsb.sbid;
+      if (devinfo->ver >= 20) {
+         if ((swsb.mode & TGL_SBID_SET)) {
+            assert(swsb.pipe == TGL_PIPE_ALL ||
+                   swsb.pipe == TGL_PIPE_INT || swsb.pipe == TGL_PIPE_FLOAT);
+            return (swsb.pipe == TGL_PIPE_INT ? 0x300 :
+                    swsb.pipe == TGL_PIPE_FLOAT ? 0x200 : 0x100) |
+                   swsb.regdist << 5 | swsb.sbid;
+         } else {
+            assert(!(swsb.mode & ~(TGL_SBID_DST | TGL_SBID_SRC)));
+            return (swsb.pipe == TGL_PIPE_ALL ? 0x300 :
+                    swsb.mode == TGL_SBID_SRC ? 0x200 : 0x100) |
+                   swsb.regdist << 5 | swsb.sbid;
+         }
+      } else {
+         assert(!(swsb.sbid & ~0xfu));
+         return 0x80 | swsb.regdist << 4 | swsb.sbid;
+      }
+
    } else {
-      return swsb.sbid | (swsb.mode & TGL_SBID_SET ? 0x40 :
-                          swsb.mode & TGL_SBID_DST ? 0x20 : 0x30);
+      if (devinfo->ver >= 20) {
+         return swsb.sbid | (swsb.mode & TGL_SBID_SET ? 0xc0 :
+                             swsb.mode & TGL_SBID_DST ? 0x80 : 0xa0);
+      } else {
+         assert(!(swsb.sbid & ~0xfu));
+         return swsb.sbid | (swsb.mode & TGL_SBID_SET ? 0x40 :
+                             swsb.mode & TGL_SBID_DST ? 0x20 : 0x30);
+      }
    }
 }
 
@@ -1267,29 +1002,70 @@ tgl_swsb_encode(const struct intel_device_info *devinfo, struct tgl_swsb swsb)
  */
 static inline struct tgl_swsb
 tgl_swsb_decode(const struct intel_device_info *devinfo,
-                const bool is_unordered, const uint8_t x)
+                const bool is_unordered, const uint32_t x)
 {
-   if (x & 0x80) {
-      const struct tgl_swsb swsb = { (x & 0x70u) >> 4, TGL_PIPE_NONE,
-                                     x & 0xfu,
-                                     is_unordered ?
-                                     TGL_SBID_SET : TGL_SBID_DST };
-      return swsb;
-   } else if ((x & 0x70) == 0x20) {
-      return tgl_swsb_sbid(TGL_SBID_DST, x & 0xfu);
-   } else if ((x & 0x70) == 0x30) {
-      return tgl_swsb_sbid(TGL_SBID_SRC, x & 0xfu);
-   } else if ((x & 0x70) == 0x40) {
-      return tgl_swsb_sbid(TGL_SBID_SET, x & 0xfu);
+   if (devinfo->ver >= 20) {
+      if (x & 0x300) {
+         if (is_unordered) {
+            const struct tgl_swsb swsb = {
+               (x & 0xe0u) >> 5,
+               ((x & 0x300) == 0x300 ? TGL_PIPE_INT :
+                (x & 0x300) == 0x200 ? TGL_PIPE_FLOAT :
+                TGL_PIPE_ALL),
+               x & 0x1fu,
+               TGL_SBID_SET
+            };
+            return swsb;
+         } else {
+            const struct tgl_swsb swsb = {
+               (x & 0xe0u) >> 5,
+               ((x & 0x300) == 0x300 ? TGL_PIPE_ALL : TGL_PIPE_NONE),
+               x & 0x1fu,
+               ((x & 0x300) == 0x200 ? TGL_SBID_SRC : TGL_SBID_DST)
+            };
+            return swsb;
+         }
+
+      } else if ((x & 0xe0) == 0x80) {
+         return tgl_swsb_sbid(TGL_SBID_DST, x & 0x1f);
+      } else if ((x & 0xe0) == 0xa0) {
+         return tgl_swsb_sbid(TGL_SBID_SRC, x & 0x1fu);
+      } else if ((x & 0xe0) == 0xc0) {
+         return tgl_swsb_sbid(TGL_SBID_SET, x & 0x1fu);
+      } else {
+            const struct tgl_swsb swsb = { x & 0x7u,
+                                           ((x & 0x38) == 0x10 ? TGL_PIPE_FLOAT :
+                                            (x & 0x38) == 0x18 ? TGL_PIPE_INT :
+                                            (x & 0x38) == 0x20 ? TGL_PIPE_LONG :
+                                            (x & 0x38) == 0x28 ? TGL_PIPE_MATH :
+                                            (x & 0x38) == 0x8 ? TGL_PIPE_ALL :
+                                            TGL_PIPE_NONE) };
+            return swsb;
+      }
+
    } else {
-      const struct tgl_swsb swsb = { x & 0x7u,
-                                     ((x & 0x78) == 0x10 ? TGL_PIPE_FLOAT :
-                                      (x & 0x78) == 0x18 ? TGL_PIPE_INT :
-                                      (x & 0x78) == 0x50 ? TGL_PIPE_LONG :
-                                      (x & 0x78) == 0x8 ? TGL_PIPE_ALL :
-                                      TGL_PIPE_NONE) };
-      assert(devinfo->verx10 >= 125 || swsb.pipe == TGL_PIPE_NONE);
-      return swsb;
+      if (x & 0x80) {
+         const struct tgl_swsb swsb = { (x & 0x70u) >> 4, TGL_PIPE_NONE,
+                                        x & 0xfu,
+                                        is_unordered ?
+                                        TGL_SBID_SET : TGL_SBID_DST };
+         return swsb;
+      } else if ((x & 0x70) == 0x20) {
+         return tgl_swsb_sbid(TGL_SBID_DST, x & 0xfu);
+      } else if ((x & 0x70) == 0x30) {
+         return tgl_swsb_sbid(TGL_SBID_SRC, x & 0xfu);
+      } else if ((x & 0x70) == 0x40) {
+         return tgl_swsb_sbid(TGL_SBID_SET, x & 0xfu);
+      } else {
+         const struct tgl_swsb swsb = { x & 0x7u,
+                                        ((x & 0x78) == 0x10 ? TGL_PIPE_FLOAT :
+                                         (x & 0x78) == 0x18 ? TGL_PIPE_INT :
+                                         (x & 0x78) == 0x50 ? TGL_PIPE_LONG :
+                                         (x & 0x78) == 0x8 ? TGL_PIPE_ALL :
+                                         TGL_PIPE_NONE) };
+         assert(devinfo->verx10 >= 125 || swsb.pipe == TGL_PIPE_NONE);
+         return swsb;
+      }
    }
 }
 
@@ -1297,6 +1073,7 @@ enum tgl_sync_function {
    TGL_SYNC_NOP = 0x0,
    TGL_SYNC_ALLRD = 0x2,
    TGL_SYNC_ALLWR = 0x3,
+   TGL_SYNC_FENCE = 0xd,
    TGL_SYNC_BAR = 0xe,
    TGL_SYNC_HOST = 0xf
 };
@@ -1312,11 +1089,8 @@ enum tgl_sync_function {
  */
 enum brw_message_target {
    BRW_SFID_NULL                     = 0,
-   BRW_SFID_MATH                     = 1, /* Only valid on Gfx4-5 */
    BRW_SFID_SAMPLER                  = 2,
    BRW_SFID_MESSAGE_GATEWAY          = 3,
-   BRW_SFID_DATAPORT_READ            = 4,
-   BRW_SFID_DATAPORT_WRITE           = 5,
    BRW_SFID_URB                      = 6,
    BRW_SFID_THREAD_SPAWNER           = 7,
    BRW_SFID_VME                      = 8,
@@ -1347,25 +1121,6 @@ enum brw_message_target {
 #define GFX8_SAMPLER_RETURN_FORMAT_32BITS    0
 #define GFX8_SAMPLER_RETURN_FORMAT_16BITS    1
 
-#define BRW_SAMPLER_MESSAGE_SIMD8_SAMPLE              0
-#define BRW_SAMPLER_MESSAGE_SIMD16_SAMPLE             0
-#define BRW_SAMPLER_MESSAGE_SIMD16_SAMPLE_BIAS        0
-#define BRW_SAMPLER_MESSAGE_SIMD8_KILLPIX             1
-#define BRW_SAMPLER_MESSAGE_SIMD4X2_SAMPLE_LOD        1
-#define BRW_SAMPLER_MESSAGE_SIMD16_SAMPLE_LOD         1
-#define BRW_SAMPLER_MESSAGE_SIMD4X2_SAMPLE_GRADIENTS  2
-#define BRW_SAMPLER_MESSAGE_SIMD8_SAMPLE_GRADIENTS    2
-#define BRW_SAMPLER_MESSAGE_SIMD4X2_SAMPLE_COMPARE    0
-#define BRW_SAMPLER_MESSAGE_SIMD16_SAMPLE_COMPARE     2
-#define BRW_SAMPLER_MESSAGE_SIMD8_SAMPLE_BIAS_COMPARE 0
-#define BRW_SAMPLER_MESSAGE_SIMD4X2_SAMPLE_LOD_COMPARE 1
-#define BRW_SAMPLER_MESSAGE_SIMD8_SAMPLE_LOD_COMPARE  1
-#define BRW_SAMPLER_MESSAGE_SIMD4X2_RESINFO           2
-#define BRW_SAMPLER_MESSAGE_SIMD16_RESINFO            2
-#define BRW_SAMPLER_MESSAGE_SIMD4X2_LD                3
-#define BRW_SAMPLER_MESSAGE_SIMD8_LD                  3
-#define BRW_SAMPLER_MESSAGE_SIMD16_LD                 3
-
 #define GFX5_SAMPLER_MESSAGE_SAMPLE              0
 #define GFX5_SAMPLER_MESSAGE_SAMPLE_BIAS         1
 #define GFX5_SAMPLER_MESSAGE_SAMPLE_LOD          2
@@ -1378,10 +1133,17 @@ enum brw_message_target {
 #define GFX5_SAMPLER_MESSAGE_LOD                 9
 #define GFX5_SAMPLER_MESSAGE_SAMPLE_RESINFO      10
 #define GFX6_SAMPLER_MESSAGE_SAMPLE_SAMPLEINFO   11
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_L     13
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_B     14
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_I     15
 #define GFX7_SAMPLER_MESSAGE_SAMPLE_GATHER4_C    16
 #define GFX7_SAMPLER_MESSAGE_SAMPLE_GATHER4_PO   17
 #define GFX7_SAMPLER_MESSAGE_SAMPLE_GATHER4_PO_C 18
+#define XE2_SAMPLER_MESSAGE_SAMPLE_MLOD          18
+#define XE2_SAMPLER_MESSAGE_SAMPLE_COMPARE_MLOD  19
 #define HSW_SAMPLER_MESSAGE_SAMPLE_DERIV_COMPARE 20
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_I_C   21
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_L_C   23
 #define GFX9_SAMPLER_MESSAGE_SAMPLE_LZ           24
 #define GFX9_SAMPLER_MESSAGE_SAMPLE_C_LZ         25
 #define GFX9_SAMPLER_MESSAGE_SAMPLE_LD_LZ        26
@@ -1389,6 +1151,9 @@ enum brw_message_target {
 #define GFX7_SAMPLER_MESSAGE_SAMPLE_LD_MCS       29
 #define GFX7_SAMPLER_MESSAGE_SAMPLE_LD2DMS       30
 #define GFX7_SAMPLER_MESSAGE_SAMPLE_LD2DSS       31
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_PO_L           45
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_PO_B           46
+#define XE2_SAMPLER_MESSAGE_SAMPLE_GATHER4_PO_L_C         55
 
 /* for GFX5 only */
 #define BRW_SAMPLER_SIMD_MODE_SIMD4X2                   0
@@ -1399,6 +1164,11 @@ enum brw_message_target {
 #define GFX10_SAMPLER_SIMD_MODE_SIMD8H                  5
 #define GFX10_SAMPLER_SIMD_MODE_SIMD16H                 6
 
+#define XE2_SAMPLER_SIMD_MODE_SIMD16                  1
+#define XE2_SAMPLER_SIMD_MODE_SIMD32                  2
+#define XE2_SAMPLER_SIMD_MODE_SIMD16H                 5
+#define XE2_SAMPLER_SIMD_MODE_SIMD32H                 6
+
 /* GFX9 changes SIMD mode 0 to mean SIMD8D, but lets us get the SIMD4x2
  * behavior by setting bit 22 of dword 2 in the message header. */
 #define GFX9_SAMPLER_SIMD_MODE_SIMD8D                   0
@@ -1432,16 +1202,6 @@ enum brw_message_target {
 
 /* This one stays the same across generations. */
 #define BRW_DATAPORT_READ_MESSAGE_OWORD_BLOCK_READ          0
-/* GFX4 */
-#define BRW_DATAPORT_READ_MESSAGE_OWORD_DUAL_BLOCK_READ     1
-#define BRW_DATAPORT_READ_MESSAGE_MEDIA_BLOCK_READ          2
-#define BRW_DATAPORT_READ_MESSAGE_DWORD_SCATTERED_READ      3
-/* G45, GFX5 */
-#define G45_DATAPORT_READ_MESSAGE_RENDER_UNORM_READ	    1
-#define G45_DATAPORT_READ_MESSAGE_OWORD_DUAL_BLOCK_READ     2
-#define G45_DATAPORT_READ_MESSAGE_AVC_LOOP_FILTER_READ	    3
-#define G45_DATAPORT_READ_MESSAGE_MEDIA_BLOCK_READ          4
-#define G45_DATAPORT_READ_MESSAGE_DWORD_SCATTERED_READ      6
 /* GFX6 */
 #define GFX6_DATAPORT_READ_MESSAGE_RENDER_UNORM_READ	    1
 #define GFX6_DATAPORT_READ_MESSAGE_OWORD_DUAL_BLOCK_READ     2
@@ -1449,23 +1209,14 @@ enum brw_message_target {
 #define GFX6_DATAPORT_READ_MESSAGE_OWORD_UNALIGN_BLOCK_READ  5
 #define GFX6_DATAPORT_READ_MESSAGE_DWORD_SCATTERED_READ      6
 
-#define BRW_DATAPORT_READ_TARGET_DATA_CACHE      0
-#define BRW_DATAPORT_READ_TARGET_RENDER_CACHE    1
-#define BRW_DATAPORT_READ_TARGET_SAMPLER_CACHE   2
-
 #define BRW_DATAPORT_RENDER_TARGET_WRITE_SIMD16_SINGLE_SOURCE                0
 #define BRW_DATAPORT_RENDER_TARGET_WRITE_SIMD16_SINGLE_SOURCE_REPLICATED     1
 #define BRW_DATAPORT_RENDER_TARGET_WRITE_SIMD8_DUAL_SOURCE_SUBSPAN01         2
 #define BRW_DATAPORT_RENDER_TARGET_WRITE_SIMD8_DUAL_SOURCE_SUBSPAN23         3
 #define BRW_DATAPORT_RENDER_TARGET_WRITE_SIMD8_SINGLE_SOURCE_SUBSPAN01       4
 
-#define BRW_DATAPORT_WRITE_MESSAGE_OWORD_BLOCK_WRITE                0
-#define BRW_DATAPORT_WRITE_MESSAGE_OWORD_DUAL_BLOCK_WRITE           1
-#define BRW_DATAPORT_WRITE_MESSAGE_MEDIA_BLOCK_WRITE                2
-#define BRW_DATAPORT_WRITE_MESSAGE_DWORD_SCATTERED_WRITE            3
-#define BRW_DATAPORT_WRITE_MESSAGE_RENDER_TARGET_WRITE              4
-#define BRW_DATAPORT_WRITE_MESSAGE_STREAMED_VERTEX_BUFFER_WRITE     5
-#define BRW_DATAPORT_WRITE_MESSAGE_FLUSH_RENDER_CACHE               7
+#define XE2_DATAPORT_RENDER_TARGET_WRITE_SIMD32_SINGLE_SOURCE                1
+#define XE2_DATAPORT_RENDER_TARGET_WRITE_SIMD16_DUAL_SOURCE                  2
 
 /* GFX6 */
 #define GFX6_DATAPORT_WRITE_MESSAGE_DWORD_ATOMIC_WRITE              7
@@ -1560,12 +1311,6 @@ enum brw_message_target {
 #define BRW_BTI_STATELESS                255
 #define GFX7_BTI_SLM                     254
 
-#define HSW_BTI_STATELESS_LOCALLY_COHERENT 255
-#define HSW_BTI_STATELESS_NON_COHERENT 253
-#define HSW_BTI_STATELESS_GLOBALLY_COHERENT 252
-#define HSW_BTI_STATELESS_LLC_COHERENT 251
-#define HSW_BTI_STATELESS_L3_UNCACHED 250
-
 /* The hardware docs are a bit contradictory here.  On Haswell, where they
  * first added cache ability control, there were 5 different cache modes (see
  * HSW_BTI_STATELESS_* above).  On Broadwell, they reduced to two:
@@ -1632,7 +1377,6 @@ enum brw_message_target {
 #define BRW_MATH_FUNCTION_RSQ                              5
 #define BRW_MATH_FUNCTION_SIN                              6
 #define BRW_MATH_FUNCTION_COS                              7
-#define BRW_MATH_FUNCTION_SINCOS                           8 /* gfx4, gfx5 */
 #define BRW_MATH_FUNCTION_FDIV                             9 /* gfx6+ */
 #define BRW_MATH_FUNCTION_POW                              10
 #define BRW_MATH_FUNCTION_INT_DIV_QUOTIENT_AND_REMAINDER   11
@@ -1641,22 +1385,6 @@ enum brw_message_target {
 #define GFX8_MATH_FUNCTION_INVM                            14
 #define GFX8_MATH_FUNCTION_RSQRTM                          15
 
-#define BRW_MATH_INTEGER_UNSIGNED     0
-#define BRW_MATH_INTEGER_SIGNED       1
-
-#define BRW_MATH_PRECISION_FULL        0
-#define BRW_MATH_PRECISION_PARTIAL     1
-
-#define BRW_MATH_SATURATE_NONE         0
-#define BRW_MATH_SATURATE_SATURATE     1
-
-#define BRW_MATH_DATA_VECTOR  0
-#define BRW_MATH_DATA_SCALAR  1
-
-#define BRW_URB_OPCODE_WRITE_HWORD  0
-#define BRW_URB_OPCODE_WRITE_OWORD  1
-#define BRW_URB_OPCODE_READ_HWORD   2
-#define BRW_URB_OPCODE_READ_OWORD   3
 #define GFX7_URB_OPCODE_ATOMIC_MOV  4
 #define GFX7_URB_OPCODE_ATOMIC_INC  5
 #define GFX8_URB_OPCODE_ATOMIC_ADD  6
@@ -1668,19 +1396,6 @@ enum brw_message_target {
 #define BRW_URB_SWIZZLE_INTERLEAVE    1
 #define BRW_URB_SWIZZLE_TRANSPOSE     2
 
-#define BRW_SCRATCH_SPACE_SIZE_1K     0
-#define BRW_SCRATCH_SPACE_SIZE_2K     1
-#define BRW_SCRATCH_SPACE_SIZE_4K     2
-#define BRW_SCRATCH_SPACE_SIZE_8K     3
-#define BRW_SCRATCH_SPACE_SIZE_16K    4
-#define BRW_SCRATCH_SPACE_SIZE_32K    5
-#define BRW_SCRATCH_SPACE_SIZE_64K    6
-#define BRW_SCRATCH_SPACE_SIZE_128K   7
-#define BRW_SCRATCH_SPACE_SIZE_256K   8
-#define BRW_SCRATCH_SPACE_SIZE_512K   9
-#define BRW_SCRATCH_SPACE_SIZE_1M     10
-#define BRW_SCRATCH_SPACE_SIZE_2M     11
-
 #define BRW_MESSAGE_GATEWAY_SFID_OPEN_GATEWAY         0
 #define BRW_MESSAGE_GATEWAY_SFID_CLOSE_GATEWAY        1
 #define BRW_MESSAGE_GATEWAY_SFID_FORWARD_MSG          2
@@ -1700,14 +1415,6 @@ enum brw_message_target {
 #define GFX7_MAX_HS_URB_ENTRY_SIZE_BYTES                (512*64)
 #define GFX7_MAX_VS_URB_ENTRY_SIZE_BYTES                (512*64)
 
-#define BRW_GS_EDGE_INDICATOR_0			(1 << 8)
-#define BRW_GS_EDGE_INDICATOR_1			(1 << 9)
-
-/* Gfx6 "GS URB Entry Allocation Size" is defined as a number of 1024-bit
- * (128 bytes) URB rows and the maximum allowed value is 5 rows.
- */
-#define GFX6_MAX_GS_URB_ENTRY_SIZE_BYTES                (5*128)
-
 /* GS Thread Payload
  */
 
@@ -1717,9 +1424,6 @@ enum brw_message_target {
 #define GFX7_MAX_GS_OUTPUT_VERTEX_SIZE_BYTES            (62*16)
 
 
-/* R0 */
-# define GFX7_GS_PAYLOAD_INSTANCE_ID_SHIFT		27
-
 /* CR0.0[5:4] Floating-Point Rounding Modes
  *  Skylake PRM, Volume 7 Part 1, "Control Register", page 756
  */
@@ -1861,6 +1565,39 @@ enum lsc_cache_load {
    LSC_CACHE_LOAD_L1IAR_L3C      = 7,
 };
 
+/*
+ * Specifies the dataport message override to the default L1 and L3 memory
+ * cache policies. Dataport L1 cache policies are uncached (UC), cached (C),
+ * streaming (S) and invalidate-after-read (IAR). Dataport L3 cache policies
+ * are uncached (UC), cached (C), cached-as-a-constand (CC) and
+ * invalidate-after-read (IAR).
+ */
+enum PACKED xe2_lsc_cache_load {
+   /* No override. Use the non-pipelined or surface state cache settings for L1
+    * and L3.
+    */
+   XE2_LSC_CACHE_LOAD_L1STATE_L3MOCS = 0,
+   /* Override to L1 uncached and L3 uncached */
+   XE2_LSC_CACHE_LOAD_L1UC_L3UC = 2,
+   /* Override to L1 uncached and L3 cached */
+   XE2_LSC_CACHE_LOAD_L1UC_L3C = 4,
+   /* Override to L1 uncached and L3 cached as a constant */
+   XE2_LSC_CACHE_LOAD_L1UC_L3CC = 5,
+   /* Override to L1 cached and L3 uncached */
+   XE2_LSC_CACHE_LOAD_L1C_L3UC = 6,
+   /* Override to L1 cached and L3 cached */
+   XE2_LSC_CACHE_LOAD_L1C_L3C = 8,
+   /* Override to L1 cached and L3 cached as a constant */
+   XE2_LSC_CACHE_LOAD_L1C_L3CC = 9,
+   /* Override to L1 cached as streaming load and L3 uncached */
+   XE2_LSC_CACHE_LOAD_L1S_L3UC = 10,
+   /* Override to L1 cached as streaming load and L3 cached */
+   XE2_LSC_CACHE_LOAD_L1S_L3C = 12,
+   /* Override to L1 and L3 invalidate after read */
+   XE2_LSC_CACHE_LOAD_L1IAR_L3IAR = 14,
+
+};
+
 /*
  * Specifies the dataport message override to the default L1 and L3 memory
  * cache policies. Dataport L1 cache policies are uncached (UC), write-through
@@ -1889,6 +1626,38 @@ enum ENUM_PACKED lsc_cache_store {
 
 };
 
+/*
+ * Specifies the dataport message override to the default L1 and L3 memory
+ * cache policies. Dataport L1 cache policies are uncached (UC), write-through
+ * (WT), write-back (WB) and streaming (S). Dataport L3 cache policies are
+ * uncached (UC) and cached (WB).
+ */
+enum PACKED xe2_lsc_cache_store {
+   /* No override. Use the non-pipelined or surface state cache settings for L1
+    * and L3.
+    */
+   XE2_LSC_CACHE_STORE_L1STATE_L3MOCS = 0,
+   /* Override to L1 uncached and L3 uncached */
+   XE2_LSC_CACHE_STORE_L1UC_L3UC = 2,
+   /* Override to L1 uncached and L3 cached */
+   XE2_LSC_CACHE_STORE_L1UC_L3WB = 4,
+   /* Override to L1 write-through and L3 uncached */
+   XE2_LSC_CACHE_STORE_L1WT_L3UC = 6,
+   /* Override to L1 write-through and L3 cached */
+   XE2_LSC_CACHE_STORE_L1WT_L3WB = 8,
+   /* Override to L1 streaming and L3 uncached */
+   XE2_LSC_CACHE_STORE_L1S_L3UC = 10,
+   /* Override to L1 streaming and L3 cached */
+   XE2_LSC_CACHE_STORE_L1S_L3WB = 12,
+   /* Override to L1 write-back and L3 cached */
+   XE2_LSC_CACHE_STORE_L1WB_L3WB = 14,
+
+};
+
+#define LSC_CACHE(devinfo, l_or_s, cc)                                  \
+   ((devinfo)->ver < 20 ? (unsigned)LSC_CACHE_ ## l_or_s ## _ ## cc :   \
+                          (unsigned)XE2_LSC_CACHE_ ## l_or_s ## _ ## cc)
+
 /*
  * Specifies which components of the data payload 4-element vector (X,Y,Z,W) is
  * packed into the register payload.
