@@ -61,17 +61,6 @@ struct intel_device_info;
 #define BRW_MAX_GRF 128
 #define XE2_MAX_GRF 256
 
-/**
- * First GRF used for the MRF hack.
- *
- * On gfx7, MRFs are no longer used, and contiguous GRFs are used instead.  We
- * haven't converted our compiler to be aware of this, so it asks for MRFs and
- * brw_eu_emit.c quietly converts them to be accesses of the top GRFs.  The
- * register allocators have to be careful of this to avoid corrupting the "MRF"s
- * with actual GRF allocations.
- */
-#define GFX7_MRF_HACK_START 112
-
 /**
  * BRW hardware swizzles.
  * Only defines XYZW to ensure it can be contained in 2 bits
@@ -81,9 +70,6 @@ struct intel_device_info;
 #define BRW_SWIZZLE_Z 2
 #define BRW_SWIZZLE_W 3
 
-/** Number of message register file registers */
-#define BRW_MAX_MRF(gen) (gen == 6 ? 24 : 16)
-
 #define BRW_SWIZZLE4(a,b,c,d) (((a)<<0) | ((b)<<2) | ((c)<<4) | ((d)<<6))
 #define BRW_GET_SWZ(swz, idx) (((swz) >> ((idx)*2)) & 0x3)
 
@@ -133,41 +119,6 @@ brw_compose_swizzle(unsigned swz0, unsigned swz1)
       BRW_GET_SWZ(swz1, BRW_GET_SWZ(swz0, 3)));
 }
 
-/**
- * Return the result of applying swizzle \p swz to shuffle the bits of \p mask
- * (AKA image).
- */
-static inline unsigned
-brw_apply_swizzle_to_mask(unsigned swz, unsigned mask)
-{
-   unsigned result = 0;
-
-   for (unsigned i = 0; i < 4; i++) {
-      if (mask & (1 << BRW_GET_SWZ(swz, i)))
-         result |= 1 << i;
-   }
-
-   return result;
-}
-
-/**
- * Return the result of applying the inverse of swizzle \p swz to shuffle the
- * bits of \p mask (AKA preimage).  Useful to find out which components are
- * read from a swizzled source given the instruction writemask.
- */
-static inline unsigned
-brw_apply_inv_swizzle_to_mask(unsigned swz, unsigned mask)
-{
-   unsigned result = 0;
-
-   for (unsigned i = 0; i < 4; i++) {
-      if (mask & (1 << i))
-         result |= 1 << BRW_GET_SWZ(swz, i);
-   }
-
-   return result;
-}
-
 /**
  * Construct an identity swizzle for the set of enabled channels given by \p
  * mask.  The result will only reference channels enabled in the provided \p
@@ -193,28 +144,6 @@ brw_swizzle_for_mask(unsigned mask)
    return BRW_SWIZZLE4(swz[0], swz[1], swz[2], swz[3]);
 }
 
-/**
- * Construct an identity swizzle for the first \p n components of a vector.
- * When only a subset of channels of a vec4 are used we don't want to
- * reference the other channels, as that will tell optimization passes that
- * those other channels are used.
- */
-static inline unsigned
-brw_swizzle_for_size(unsigned n)
-{
-   return brw_swizzle_for_mask((1 << n) - 1);
-}
-
-/**
- * Converse of brw_swizzle_for_mask().  Returns the mask of components
- * accessed by the specified swizzle \p swz.
- */
-static inline unsigned
-brw_mask_for_swizzle(unsigned swz)
-{
-   return brw_apply_inv_swizzle_to_mask(swz, ~0);
-}
-
 uint32_t brw_swizzle_immediate(enum brw_reg_type type, uint32_t x, unsigned swz);
 
 #define REG_SIZE (8*4)
@@ -454,10 +383,6 @@ brw_reg(enum brw_reg_file file,
       assert(nr < XE2_MAX_GRF);
    else if (file == BRW_ARCHITECTURE_REGISTER_FILE)
       assert(nr <= BRW_ARF_TIMESTAMP);
-   /* Asserting on the MRF register number requires to know the hardware gen
-    * (gfx6 has 24 MRF registers), which we don't know here, so we assert
-    * for that in the generators and in brw_eu_emit.c
-    */
 
    reg.type = type;
    reg.file = file;
@@ -1050,19 +975,6 @@ brw_mask_stack_depth_reg(unsigned subnr)
                       BRW_ARF_MASK_STACK_DEPTH, subnr);
 }
 
-static inline struct brw_reg
-brw_message_reg(unsigned nr)
-{
-   return brw_vec8_reg(BRW_MESSAGE_REGISTER_FILE, nr, 0);
-}
-
-static inline struct brw_reg
-brw_uvec_mrf(unsigned width, unsigned nr, unsigned subnr)
-{
-   return retype(brw_vecn_reg(width, BRW_MESSAGE_REGISTER_FILE, nr, subnr),
-                 BRW_REGISTER_TYPE_UD);
-}
-
 /* This is almost always called with a numeric constant argument, so
  * make things easy to evaluate at compile time:
  */
