@@ -58,9 +58,9 @@ bool anv_nir_lower_ycbcr_textures(nir_shader *shader,
 
 static inline nir_address_format
 anv_nir_ssbo_addr_format(const struct anv_physical_device *pdevice,
-                         bool robust_buffer_access)
+                         enum brw_robustness_flags robust_flags)
 {
-   if (robust_buffer_access)
+   if (robust_flags & BRW_ROBUSTNESS_SSBO)
       return nir_address_format_64bit_bounded_global;
    else
       return nir_address_format_64bit_global_32bit_offset;
@@ -68,9 +68,9 @@ anv_nir_ssbo_addr_format(const struct anv_physical_device *pdevice,
 
 static inline nir_address_format
 anv_nir_ubo_addr_format(const struct anv_physical_device *pdevice,
-                        bool robust_buffer_access)
+                        enum brw_robustness_flags robust_flags)
 {
-   if (robust_buffer_access)
+   if (robust_flags & BRW_ROBUSTNESS_UBO)
       return nir_address_format_64bit_bounded_global;
    else
       return nir_address_format_64bit_global_32bit_offset;
@@ -80,7 +80,7 @@ bool anv_nir_lower_ubo_loads(nir_shader *shader);
 
 void anv_nir_apply_pipeline_layout(nir_shader *shader,
                                    const struct anv_physical_device *pdevice,
-                                   bool robust_buffer_access,
+                                   enum brw_robustness_flags robust_flags,
                                    bool independent_sets,
                                    const struct anv_pipeline_sets_layout *layout,
                                    struct anv_pipeline_bind_map *map,
@@ -89,11 +89,12 @@ void anv_nir_apply_pipeline_layout(nir_shader *shader,
 
 void anv_nir_compute_push_layout(nir_shader *nir,
                                  const struct anv_physical_device *pdevice,
-                                 bool robust_buffer_access,
+                                 enum brw_robustness_flags robust_flags,
                                  bool fragment_dynamic,
                                  struct brw_stage_prog_data *prog_data,
                                  struct anv_pipeline_bind_map *map,
                                  const struct anv_pipeline_push_map *push_map,
+                                 enum anv_descriptor_set_layout_type desc_type,
                                  void *mem_ctx);
 
 void anv_nir_validate_push_layout(struct brw_stage_prog_data *prog_data,
