@@ -29,7 +29,7 @@
  */
 
 
-#include "main/glheader.h"
+#include "util/glheader.h"
 #include "main/context.h"
 #include "main/draw_validate.h"
 #include "main/hash.h"
@@ -79,7 +79,6 @@ lookup_or_create_program(GLuint id, GLenum target, const char* caller)
       /* Bind a user program */
       newProg = _mesa_lookup_program(ctx, id);
       if (!newProg || newProg == &_mesa_DummyProgram) {
-         bool isGenName = newProg != NULL;
          /* allocate a new program now */
          newProg = ctx->Driver.NewProgram(ctx, _mesa_program_enum_to_shader_stage(target),
                                           id, true);
@@ -87,7 +86,7 @@ lookup_or_create_program(GLuint id, GLenum target, const char* caller)
             _mesa_error(ctx, GL_OUT_OF_MEMORY, "%s", caller);
             return NULL;
          }
-         _mesa_HashInsert(ctx->Shared->Programs, id, newProg, isGenName);
+         _mesa_HashInsert(&ctx->Shared->Programs, id, newProg);
       }
       else if (newProg->Target != target) {
          _mesa_error(ctx, GL_INVALID_OPERATION,
@@ -181,7 +180,7 @@ _mesa_DeleteProgramsARB(GLsizei n, const GLuint *ids)
       if (ids[i] != 0) {
          struct gl_program *prog = _mesa_lookup_program(ctx, ids[i]);
          if (prog == &_mesa_DummyProgram) {
-            _mesa_HashRemove(ctx->Shared->Programs, ids[i]);
+            _mesa_HashRemove(&ctx->Shared->Programs, ids[i]);
          }
          else if (prog) {
             /* Unbind program if necessary */
@@ -205,7 +204,7 @@ _mesa_DeleteProgramsARB(GLsizei n, const GLuint *ids)
                return;
             }
             /* The ID is immediately available for re-use now */
-            _mesa_HashRemove(ctx->Shared->Programs, ids[i]);
+            _mesa_HashRemove(&ctx->Shared->Programs, ids[i]);
             _mesa_reference_program(ctx, &prog, NULL);
          }
       }
@@ -232,17 +231,17 @@ _mesa_GenProgramsARB(GLsizei n, GLuint *ids)
    if (!ids)
       return;
 
-   _mesa_HashLockMutex(ctx->Shared->Programs);
+   _mesa_HashLockMutex(&ctx->Shared->Programs);
 
-   _mesa_HashFindFreeKeys(ctx->Shared->Programs, ids, n);
+   _mesa_HashFindFreeKeys(&ctx->Shared->Programs, ids, n);
 
    /* Insert pointer to dummy program as placeholder */
    for (i = 0; i < (GLuint) n; i++) {
-      _mesa_HashInsertLocked(ctx->Shared->Programs, ids[i],
-                             &_mesa_DummyProgram, true);
+      _mesa_HashInsertLocked(&ctx->Shared->Programs, ids[i],
+                             &_mesa_DummyProgram);
    }
 
-   _mesa_HashUnlockMutex(ctx->Shared->Programs);
+   _mesa_HashUnlockMutex(&ctx->Shared->Programs);
 }
 
 
@@ -382,7 +381,7 @@ set_program_string(struct gl_program *prog, GLenum target, GLenum format, GLsize
    gl_shader_stage stage = _mesa_program_enum_to_shader_stage(target);
 
    uint8_t sha1[SHA1_DIGEST_LENGTH];
-   _mesa_sha1_compute(string, strlen(string), sha1);
+   _mesa_sha1_compute(string, len, sha1);
 
    /* Dump original shader source to MESA_SHADER_DUMP_PATH and replace
     * if corresponding entry found from MESA_SHADER_READ_PATH.
@@ -403,6 +402,9 @@ set_program_string(struct gl_program *prog, GLenum target, GLenum format, GLsize
    }
    else {
       _mesa_error(ctx, GL_INVALID_ENUM, "glProgramStringARB(target)");
+#ifdef ENABLE_SHADER_CACHE
+      free(replacement);
+#endif
       return;
    }
 
@@ -461,6 +463,9 @@ set_program_string(struct gl_program *prog, GLenum target, GLenum format, GLsize
       }
       ralloc_free(filename);
    }
+#ifdef ENABLE_SHADER_CACHE
+   free(replacement);
+#endif
 }
 
 void GLAPIENTRY
