@@ -58,7 +58,7 @@
 static int
 type_size(const struct glsl_type *type)
 {
-   return type->count_attribute_slots(false);
+   return glsl_count_attribute_slots(type, false);
 }
 
 /* Depending on PIPE_CAP_TGSI_TEXCOORD (st->needs_texcoord_semantic) we
@@ -123,7 +123,7 @@ st_nir_assign_vs_in_locations(struct nir_shader *nir)
 
    /* Re-lower global vars, to deal with any dead VS inputs. */
    if (removed_inputs)
-      NIR_PASS_V(nir, nir_lower_global_vars_to_local);
+      NIR_PASS(_, nir, nir_lower_global_vars_to_local);
 }
 
 static int
@@ -189,8 +189,8 @@ st_nir_assign_uniform_locations(struct gl_context *ctx,
       int loc;
 
       const struct glsl_type *type = glsl_without_array(uniform->type);
-      if (!uniform->data.bindless && (type->is_sampler() || type->is_image())) {
-         if (type->is_sampler()) {
+      if (!uniform->data.bindless && (glsl_type_is_sampler(type) || glsl_type_is_image(type))) {
+         if (glsl_type_is_sampler(type)) {
             loc = shaderidx;
             shaderidx += type_size(uniform->type);
          } else {
@@ -231,10 +231,10 @@ st_nir_assign_uniform_locations(struct gl_context *ctx,
 }
 
 static bool
-dest_is_64bit(nir_dest *dest, void *state)
+def_is_64bit(nir_def *def, void *state)
 {
    bool *lower = (bool *)state;
-   if (dest && (nir_dest_bit_size(*dest) == 64)) {
+   if (def && (def->bit_size == 64)) {
       *lower = true;
       return false;
    }
@@ -260,7 +260,7 @@ filter_64_bit_instr(const nir_instr *const_instr, UNUSED const void *data)
     * doesn't have const variants, so do the ugly const_cast here. */
    nir_instr *instr = const_cast<nir_instr *>(const_instr);
 
-   nir_foreach_dest(instr, dest_is_64bit, &lower);
+   nir_foreach_def(instr, def_is_64bit, &lower);
    if (lower)
       return true;
    nir_foreach_src(instr, src_is_64bit, &lower);
@@ -320,13 +320,13 @@ st_glsl_to_nir_post_opts(struct st_context *st, struct gl_program *prog,
     */
    if (!shader_program->data->spirv &&
        !st->ctx->Const.PackedDriverUniformStorage)
-      NIR_PASS_V(nir, st_nir_lower_builtin);
+      NIR_PASS(_, nir, st_nir_lower_builtin);
 
    if (!screen->get_param(screen, PIPE_CAP_NIR_ATOMICS_AS_DEREF))
-      NIR_PASS_V(nir, gl_nir_lower_atomics, shader_program, true);
+      NIR_PASS(_, nir, gl_nir_lower_atomics, shader_program, true);
 
-   NIR_PASS_V(nir, nir_opt_intrinsics);
-   NIR_PASS_V(nir, nir_opt_fragdepth);
+   NIR_PASS(_, nir, nir_opt_intrinsics);
+   NIR_PASS(_, nir, nir_opt_fragdepth);
 
    /* Lower 64-bit ops. */
    if (nir->options->lower_int64_options ||
@@ -357,7 +357,7 @@ st_glsl_to_nir_post_opts(struct st_context *st, struct gl_program *prog,
          NIR_PASS(lowered_64bit_ops, nir, nir_lower_int64);
 
       if (revectorize && !nir->options->vectorize_vec2_16bit)
-         NIR_PASS_V(nir, nir_opt_vectorize, nullptr, nullptr);
+         NIR_PASS(_, nir, nir_opt_vectorize, nullptr, nullptr);
 
       if (revectorize || lowered_64bit_ops)
          gl_nir_opts(nir);
@@ -377,7 +377,7 @@ st_glsl_to_nir_post_opts(struct st_context *st, struct gl_program *prog,
          }
          align_offset_state = STATE_ATOMIC_COUNTER_OFFSET;
       }
-      NIR_PASS_V(nir, nir_lower_atomics_to_ssbo, align_offset_state);
+      NIR_PASS(_, nir, nir_lower_atomics_to_ssbo, align_offset_state);
    }
 
    st_set_prog_affected_state_flags(prog);
@@ -385,8 +385,10 @@ st_glsl_to_nir_post_opts(struct st_context *st, struct gl_program *prog,
    st_finalize_nir_before_variants(nir);
 
    char *msg = NULL;
-   if (st->allow_st_finalize_nir_twice)
-      msg = st_finalize_nir(st, prog, shader_program, nir, true, true);
+   if (st->allow_st_finalize_nir_twice) {
+      st_serialize_base_nir(prog, nir);
+      msg = st_finalize_nir(st, prog, shader_program, nir, true, true, false);
+   }
 
    if (st->ctx->_Shader->Flags & GLSL_DUMP) {
       _mesa_log("\n");
@@ -404,18 +406,18 @@ static void
 st_nir_vectorize_io(nir_shader *producer, nir_shader *consumer)
 {
    if (consumer)
-      NIR_PASS_V(consumer, nir_lower_io_to_vector, nir_var_shader_in);
+      NIR_PASS(_, consumer, nir_lower_io_to_vector, nir_var_shader_in);
 
    if (!producer)
       return;
 
-   NIR_PASS_V(producer, nir_lower_io_to_vector, nir_var_shader_out);
+   NIR_PASS(_, producer, nir_lower_io_to_vector, nir_var_shader_out);
 
    if (producer->info.stage == MESA_SHADER_TESS_CTRL &&
        producer->options->vectorize_tess_levels)
-      NIR_PASS_V(producer, nir_vectorize_tess_levels);
+      NIR_PASS(_, producer, nir_vectorize_tess_levels);
 
-   NIR_PASS_V(producer, nir_opt_combine_stores, nir_var_shader_out);
+   NIR_PASS(_, producer, nir_opt_combine_stores, nir_var_shader_out);
 
    if ((producer)->info.stage != MESA_SHADER_TESS_CTRL) {
       /* Calling lower_io_to_vector creates output variable writes with
@@ -424,30 +426,34 @@ st_nir_vectorize_io(nir_shader *producer, nir_shader *consumer)
        * them.  This, in turn, creates temporary variables and extra
        * copy_deref intrinsics that we need to clean up.
        */
-      NIR_PASS_V(producer, nir_lower_io_to_temporaries,
+      NIR_PASS(_, producer, nir_lower_io_to_temporaries,
                  nir_shader_get_entrypoint(producer), true, false);
-      NIR_PASS_V(producer, nir_lower_global_vars_to_local);
-      NIR_PASS_V(producer, nir_split_var_copies);
-      NIR_PASS_V(producer, nir_lower_var_copies);
+      NIR_PASS(_, producer, nir_lower_global_vars_to_local);
+      NIR_PASS(_, producer, nir_split_var_copies);
+      NIR_PASS(_, producer, nir_lower_var_copies);
    }
 
    /* Undef scalar store_deref intrinsics are not ignored by nir_lower_io,
     * so they must be removed before that. These passes remove them.
     */
-   NIR_PASS_V(producer, nir_lower_vars_to_ssa);
-   NIR_PASS_V(producer, nir_opt_undef);
-   NIR_PASS_V(producer, nir_opt_dce);
+   NIR_PASS(_, producer, nir_lower_vars_to_ssa);
+   NIR_PASS(_, producer, nir_opt_undef);
+   NIR_PASS(_, producer, nir_opt_dce);
 }
 
 extern "C" {
 
-void
+bool
 st_nir_lower_wpos_ytransform(struct nir_shader *nir,
                              struct gl_program *prog,
                              struct pipe_screen *pscreen)
 {
-   if (nir->info.stage != MESA_SHADER_FRAGMENT)
-      return;
+   bool progress = false;
+
+   if (nir->info.stage != MESA_SHADER_FRAGMENT) {
+      nir_shader_preserve_all_metadata(nir);
+      return progress;
+   }
 
    static const gl_state_index16 wposTransformState[STATE_LENGTH] = {
       STATE_FB_WPOS_Y_TRANSFORM
@@ -470,8 +476,8 @@ st_nir_lower_wpos_ytransform(struct nir_shader *nir,
                          PIPE_CAP_FS_COORD_PIXEL_CENTER_HALF_INTEGER);
 
    if (nir_lower_wpos_ytransform(nir, &wpos_options)) {
-      nir_validate_shader(nir, "after nir_lower_wpos_ytransform");
       _mesa_add_state_reference(prog->Parameters, wposTransformState);
+      progress = true;
    }
 
    static const gl_state_index16 pntcTransformState[STATE_LENGTH] = {
@@ -480,7 +486,10 @@ st_nir_lower_wpos_ytransform(struct nir_shader *nir,
 
    if (nir_lower_pntc_ytransform(nir, &pntcTransformState)) {
       _mesa_add_state_reference(prog->Parameters, pntcTransformState);
+      progress = true;
    }
+
+   return progress;
 }
 
 static bool
@@ -488,7 +497,6 @@ st_link_glsl_to_nir(struct gl_context *ctx,
                     struct gl_shader_program *shader_program)
 {
    struct st_context *st = st_context(ctx);
-   struct pipe_screen *pscreen = st->screen;
    struct gl_linked_shader *linked_shader[MESA_SHADER_STAGES];
    unsigned num_shaders = 0;
 
@@ -501,42 +509,6 @@ st_link_glsl_to_nir(struct gl_context *ctx,
 
    assert(shader_program->data->LinkStatus);
 
-   /* Skip the GLSL steps when using SPIR-V. */
-   if (!shader_program->data->spirv) {
-      for (unsigned i = 0; i < MESA_SHADER_STAGES; i++) {
-         if (shader_program->_LinkedShaders[i] == NULL)
-            continue;
-
-         struct gl_linked_shader *shader = shader_program->_LinkedShaders[i];
-         exec_list *ir = shader->ir;
-         gl_shader_stage stage = shader->Stage;
-         const struct gl_shader_compiler_options *options =
-               &ctx->Const.ShaderCompilerOptions[stage];
-
-         enum pipe_shader_type ptarget = pipe_shader_type_from_mesa(stage);
-         bool have_dround = pscreen->get_shader_param(pscreen, ptarget,
-                                                      PIPE_SHADER_CAP_DROUND_SUPPORTED);
-
-         if (!pscreen->get_param(pscreen, PIPE_CAP_INT64_DIVMOD))
-            lower_64bit_integer_instructions(ir, DIV64 | MOD64);
-
-         lower_packing_builtins(ir, ctx->Extensions.ARB_shading_language_packing,
-                                ctx->Extensions.ARB_gpu_shader5,
-                                ctx->st->has_half_float_packing);
-         do_mat_op_to_vec(ir);
-
-         lower_instructions(ir, have_dround,
-                            ctx->Extensions.ARB_gpu_shader5);
-
-         do_vec_index_to_cond_assign(ir);
-         if (options->MaxIfDepth == 0) {
-            lower_discard(ir);
-         }
-
-         validate_ir_tree(ir);
-      }
-   }
-
    for (unsigned i = 0; i < MESA_SHADER_STAGES; i++) {
       if (shader_program->_LinkedShaders[i])
          linked_shader[num_shaders++] = shader_program->_LinkedShaders[i];
@@ -548,7 +520,7 @@ st_link_glsl_to_nir(struct gl_context *ctx,
          st->ctx->Const.ShaderCompilerOptions[shader->Stage].NirOptions;
       struct gl_program *prog = shader->Program;
 
-      _mesa_copy_linked_program_data(shader_program, shader);
+      shader->Program->info.separate_shader = shader_program->SeparateShader;
 
       assert(!prog->nir);
       prog->shader_program = shader_program;
@@ -560,8 +532,6 @@ st_link_glsl_to_nir(struct gl_context *ctx,
       if (shader_program->data->spirv) {
          prog->nir = _mesa_spirv_to_nir(ctx, shader_program, shader->Stage, options);
       } else {
-         validate_ir_tree(shader->ir);
-
          if (ctx->_Shader->Flags & GLSL_DUMP) {
             _mesa_log("\n");
             _mesa_log("GLSL IR for linked %s program %d:\n",
@@ -598,8 +568,7 @@ st_link_glsl_to_nir(struct gl_context *ctx,
                              &opts))
          return GL_FALSE;
    } else {
-      if (!gl_nir_link_glsl(&ctx->Const, &ctx->Extensions, ctx->API,
-                            shader_program))
+      if (!gl_nir_link_glsl(ctx, shader_program))
          return GL_FALSE;
    }
 
@@ -619,28 +588,50 @@ st_link_glsl_to_nir(struct gl_context *ctx,
       const struct gl_shader_compiler_options *options =
             &ctx->Const.ShaderCompilerOptions[stage];
 
+      if (nir->info.io_lowered) {
+         /* Since IO is lowered, we won't need the IO variables from now on.
+          * nir_build_program_resource_list was the last pass that needed them.
+          */
+         NIR_PASS_V(nir, nir_remove_dead_variables,
+                    nir_var_shader_in | nir_var_shader_out, NULL);
+      }
+
       /* If there are forms of indirect addressing that the driver
        * cannot handle, perform the lowering pass.
        */
       if (options->EmitNoIndirectInput || options->EmitNoIndirectOutput ||
           options->EmitNoIndirectTemp || options->EmitNoIndirectUniform) {
-         nir_variable_mode mode = options->EmitNoIndirectInput ?
-            nir_var_shader_in : (nir_variable_mode)0;
-         mode |= options->EmitNoIndirectOutput ?
-            nir_var_shader_out : (nir_variable_mode)0;
+         nir_variable_mode mode = (nir_variable_mode)0;
+
+         if (!nir->info.io_lowered) {
+            mode |= options->EmitNoIndirectInput ?
+               nir_var_shader_in : (nir_variable_mode)0;
+            mode |= options->EmitNoIndirectOutput ?
+               nir_var_shader_out : (nir_variable_mode)0;
+         }
          mode |= options->EmitNoIndirectTemp ?
             nir_var_function_temp : (nir_variable_mode)0;
          mode |= options->EmitNoIndirectUniform ?
             nir_var_uniform | nir_var_mem_ubo | nir_var_mem_ssbo :
             (nir_variable_mode)0;
 
-         nir_lower_indirect_derefs(nir, mode, UINT32_MAX);
+         if (mode)
+            nir_lower_indirect_derefs(nir, mode, UINT32_MAX);
       }
 
       /* This needs to run after the initial pass of nir_lower_vars_to_ssa, so
        * that the buffer indices are constants in nir where they where
        * constants in GLSL. */
-      NIR_PASS_V(nir, gl_nir_lower_buffers, shader_program);
+      NIR_PASS(_, nir, gl_nir_lower_buffers, shader_program);
+
+      NIR_PASS(_, nir, st_nir_lower_wpos_ytransform, shader->Program,
+               st->screen);
+
+      NIR_PASS(_, nir, nir_lower_system_values);
+      NIR_PASS(_, nir, nir_lower_compute_system_values, NULL);
+
+      if (nir->info.io_lowered)
+         continue; /* the rest is for non-lowered IO only */
 
       /* Remap the locations to slots so those requiring two slots will occupy
        * two locations. For instance, if we have in the IR code a dvec3 attr0 in
@@ -650,12 +641,6 @@ st_link_glsl_to_nir(struct gl_context *ctx,
       if (nir->info.stage == MESA_SHADER_VERTEX && !shader_program->data->spirv)
          nir_remap_dual_slot_attributes(nir, &shader->Program->DualSlotInputs);
 
-      NIR_PASS_V(nir, st_nir_lower_wpos_ytransform, shader->Program,
-                 st->screen);
-
-      NIR_PASS_V(nir, nir_lower_system_values);
-      NIR_PASS_V(nir, nir_lower_compute_system_values, NULL);
-
       if (i >= 1) {
          struct gl_program *prev_shader = linked_shader[i - 1]->Program;
 
@@ -731,13 +716,19 @@ st_link_glsl_to_nir(struct gl_context *ctx,
       prog->info.num_abos = old_info.num_abos;
 
       if (prog->info.stage == MESA_SHADER_VERTEX) {
-         /* NIR expands dual-slot inputs out to two locations.  We need to
-          * compact things back down GL-style single-slot inputs to avoid
-          * confusing the state tracker.
-          */
-         prog->info.inputs_read =
-            nir_get_single_slot_attribs_mask(prog->nir->info.inputs_read,
-                                             prog->DualSlotInputs);
+         if (prog->nir->info.io_lowered &&
+             prog->nir->options->io_options & nir_io_glsl_opt_varyings) {
+            prog->info.inputs_read = prog->nir->info.inputs_read;
+            prog->DualSlotInputs = prog->nir->info.dual_slot_inputs;
+         } else {
+            /* NIR expands dual-slot inputs out to two locations.  We need to
+             * compact things back down GL-style single-slot inputs to avoid
+             * confusing the state tracker.
+             */
+            prog->info.inputs_read =
+               nir_get_single_slot_attribs_mask(prog->nir->info.inputs_read,
+                                                prog->DualSlotInputs);
+         }
 
          /* Initialize st_vertex_program members. */
          st_prepare_vertex_program(prog);
@@ -780,6 +771,10 @@ st_link_glsl_to_nir(struct gl_context *ctx,
 void
 st_nir_assign_varying_locations(struct st_context *st, nir_shader *nir)
 {
+   /* Lowered IO don't have variables, so exit. */
+   if (nir->info.io_lowered)
+      return;
+
    if (nir->info.stage == MESA_SHADER_VERTEX) {
       nir_assign_io_var_locations(nir, nir_var_shader_out,
                                   &nir->num_outputs,
@@ -818,9 +813,9 @@ st_nir_lower_samplers(struct pipe_screen *screen, nir_shader *nir,
                       struct gl_program *prog)
 {
    if (screen->get_param(screen, PIPE_CAP_NIR_SAMPLERS_AS_DEREF))
-      NIR_PASS_V(nir, gl_nir_lower_samplers_as_deref, shader_program);
+      NIR_PASS(_, nir, gl_nir_lower_samplers_as_deref, shader_program);
    else
-      NIR_PASS_V(nir, gl_nir_lower_samplers, shader_program);
+      NIR_PASS(_, nir, gl_nir_lower_samplers, shader_program);
 
    if (prog) {
       BITSET_COPY(prog->info.textures_used, nir->info.textures_used);
@@ -848,17 +843,17 @@ void
 st_nir_lower_uniforms(struct st_context *st, nir_shader *nir)
 {
    if (st->ctx->Const.PackedDriverUniformStorage) {
-      NIR_PASS_V(nir, nir_lower_io, nir_var_uniform,
+      NIR_PASS(_, nir, nir_lower_io, nir_var_uniform,
                  st_packed_uniforms_type_size,
                  (nir_lower_io_options)0);
    } else {
-      NIR_PASS_V(nir, nir_lower_io, nir_var_uniform,
+      NIR_PASS(_, nir, nir_lower_io, nir_var_uniform,
                  st_unpacked_uniforms_type_size,
                  (nir_lower_io_options)0);
    }
 
    if (nir->options->lower_uniforms_to_ubo)
-      NIR_PASS_V(nir, nir_lower_uniforms_to_ubo,
+      NIR_PASS(_, nir, nir_lower_uniforms_to_ubo,
                  st->ctx->Const.PackedDriverUniformStorage,
                  !st->ctx->Const.NativeIntegers);
 }
@@ -870,24 +865,25 @@ char *
 st_finalize_nir(struct st_context *st, struct gl_program *prog,
                 struct gl_shader_program *shader_program,
                 nir_shader *nir, bool finalize_by_driver,
-                bool is_before_variants)
+                bool is_before_variants,
+                bool is_draw_shader)
 {
    struct pipe_screen *screen = st->screen;
 
    MESA_TRACE_FUNC();
 
-   NIR_PASS_V(nir, nir_split_var_copies);
-   NIR_PASS_V(nir, nir_lower_var_copies);
+   NIR_PASS(_, nir, nir_split_var_copies);
+   NIR_PASS(_, nir, nir_lower_var_copies);
 
    const bool lower_tg4_offsets =
-      !st->screen->get_param(screen, PIPE_CAP_TEXTURE_GATHER_OFFSETS);
+      !is_draw_shader && !st->screen->get_param(screen, PIPE_CAP_TEXTURE_GATHER_OFFSETS);
 
-   if (st->lower_rect_tex || lower_tg4_offsets) {
+   if (!is_draw_shader && (st->lower_rect_tex || lower_tg4_offsets)) {
       struct nir_lower_tex_options opts = {0};
       opts.lower_rect = !!st->lower_rect_tex;
       opts.lower_tg4_offsets = lower_tg4_offsets;
 
-      NIR_PASS_V(nir, nir_lower_tex, &opts);
+      NIR_PASS(_, nir, nir_lower_tex, &opts);
    }
 
    st_nir_assign_varying_locations(st, nir);
@@ -895,10 +891,13 @@ st_finalize_nir(struct st_context *st, struct gl_program *prog,
 
    /* Lower load_deref/store_deref of inputs and outputs.
     * This depends on st_nir_assign_varying_locations.
+    *
+    * TODO: remove this once nir_io_glsl_opt_varyings is enabled by default.
     */
-   if (nir->options->lower_io_variables) {
+   if (!is_draw_shader && nir->options->io_options & nir_io_glsl_lower_derefs &&
+       !(nir->options->io_options & nir_io_glsl_opt_varyings)) {
       nir_lower_io_passes(nir, false);
-      NIR_PASS_V(nir, nir_remove_dead_variables,
+      NIR_PASS(_, nir, nir_remove_dead_variables,
                  nir_var_shader_in | nir_var_shader_out, NULL);
    }
 
@@ -907,7 +906,7 @@ st_finalize_nir(struct st_context *st, struct gl_program *prog,
 
    st_nir_lower_uniforms(st, nir);
 
-   if (is_before_variants && nir->options->lower_uniforms_to_ubo) {
+   if (!is_draw_shader && is_before_variants && nir->options->lower_uniforms_to_ubo) {
       /* This must be done after uniforms are lowered to UBO and all
        * nir_var_uniform variables are removed from NIR to prevent conflicts
        * between state parameter merging and shader variant generation.
@@ -916,11 +915,11 @@ st_finalize_nir(struct st_context *st, struct gl_program *prog,
    }
 
    st_nir_lower_samplers(screen, nir, shader_program, prog);
-   if (!screen->get_param(screen, PIPE_CAP_NIR_IMAGES_AS_DEREF))
-      NIR_PASS_V(nir, gl_nir_lower_images, false);
+   if (!is_draw_shader && !screen->get_param(screen, PIPE_CAP_NIR_IMAGES_AS_DEREF))
+      NIR_PASS(_, nir, gl_nir_lower_images, false);
 
    char *msg = NULL;
-   if (finalize_by_driver && screen->finalize_nir)
+   if (!is_draw_shader && finalize_by_driver && screen->finalize_nir)
       msg = screen->finalize_nir(screen, nir);
 
    return msg;
