@@ -504,6 +504,7 @@ typedef struct {
 
          /* Used for valhall texturing */
          bool shadow;
+         bool wide_indices;
          bool texel_offset;
          bool array_enable;
          bool integer_coordinates;
@@ -946,6 +947,12 @@ bi_temp(bi_context *ctx)
    return bi_get_index(ctx->ssa_alloc++);
 }
 
+static inline bi_index
+bi_def_index(nir_def *def)
+{
+   return bi_get_index(def->index);
+}
+
 /* Inline constants automatically, will be lowered out by bi_lower_fau where a
  * constant is not allowed. load_const_to_scalar gaurantees that this makes
  * sense */
@@ -956,18 +963,10 @@ bi_src_index(nir_src *src)
    if (nir_src_is_const(*src) && nir_src_bit_size(*src) <= 32) {
       return bi_imm_u32(nir_src_as_uint(*src));
    } else {
-      assert(src->is_ssa);
-      return bi_get_index(src->ssa->index);
+      return bi_def_index(src->ssa);
    }
 }
 
-static inline bi_index
-bi_dest_index(nir_dest *dst)
-{
-   assert(dst->is_ssa);
-   return bi_get_index(dst->ssa.index);
-}
-
 /* Iterators for Bifrost IR */
 
 #define bi_foreach_block(ctx, v)                                               \
