@@ -21,9 +21,10 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#include "panvk_private.h"
+#include "panvk_entrypoints.h"
+#include "panvk_macros.h"
 
-VkResult
+VKAPI_ATTR VkResult VKAPI_CALL
 panvk_CreateQueryPool(VkDevice _device,
                       const VkQueryPoolCreateInfo *pCreateInfo,
                       const VkAllocationCallbacks *pAllocator,
@@ -33,71 +34,57 @@ panvk_CreateQueryPool(VkDevice _device,
    return VK_SUCCESS;
 }
 
-void
-panvk_DestroyQueryPool(VkDevice _device,
-                       VkQueryPool _pool,
+VKAPI_ATTR void VKAPI_CALL
+panvk_DestroyQueryPool(VkDevice _device, VkQueryPool _pool,
                        const VkAllocationCallbacks *pAllocator)
 {
    panvk_stub();
 }
 
-VkResult
-panvk_GetQueryPoolResults(VkDevice _device,
-                          VkQueryPool queryPool,
-                          uint32_t firstQuery,
-                          uint32_t queryCount,
-                          size_t dataSize,
-                          void *pData,
-                          VkDeviceSize stride,
+VKAPI_ATTR VkResult VKAPI_CALL
+panvk_GetQueryPoolResults(VkDevice _device, VkQueryPool queryPool,
+                          uint32_t firstQuery, uint32_t queryCount,
+                          size_t dataSize, void *pData, VkDeviceSize stride,
                           VkQueryResultFlags flags)
 {
    panvk_stub();
    return VK_SUCCESS;
 }
 
-void
+VKAPI_ATTR void VKAPI_CALL
 panvk_CmdCopyQueryPoolResults(VkCommandBuffer commandBuffer,
-                              VkQueryPool queryPool,
-                              uint32_t firstQuery,
-                              uint32_t queryCount,
-                              VkBuffer dstBuffer,
-                              VkDeviceSize dstOffset,
-                              VkDeviceSize stride,
+                              VkQueryPool queryPool, uint32_t firstQuery,
+                              uint32_t queryCount, VkBuffer dstBuffer,
+                              VkDeviceSize dstOffset, VkDeviceSize stride,
                               VkQueryResultFlags flags)
 {
    panvk_stub();
 }
 
-void
-panvk_CmdResetQueryPool(VkCommandBuffer commandBuffer,
-                        VkQueryPool queryPool,
-                        uint32_t firstQuery,
-                        uint32_t queryCount)
+VKAPI_ATTR void VKAPI_CALL
+panvk_CmdResetQueryPool(VkCommandBuffer commandBuffer, VkQueryPool queryPool,
+                        uint32_t firstQuery, uint32_t queryCount)
 {
    panvk_stub();
 }
 
-void
-panvk_CmdBeginQuery(VkCommandBuffer commandBuffer,
-                    VkQueryPool queryPool,
-                    uint32_t query,
-                    VkQueryControlFlags flags)
+VKAPI_ATTR void VKAPI_CALL
+panvk_CmdBeginQuery(VkCommandBuffer commandBuffer, VkQueryPool queryPool,
+                    uint32_t query, VkQueryControlFlags flags)
 {
    panvk_stub();
 }
 
-void
-panvk_CmdEndQuery(VkCommandBuffer commandBuffer,
-                  VkQueryPool queryPool,
+VKAPI_ATTR void VKAPI_CALL
+panvk_CmdEndQuery(VkCommandBuffer commandBuffer, VkQueryPool queryPool,
                   uint32_t query)
 {
    panvk_stub();
 }
 
-void
+VKAPI_ATTR void VKAPI_CALL
 panvk_CmdWriteTimestamp2(VkCommandBuffer commandBuffer,
-                         VkPipelineStageFlags2 stage,
-                         VkQueryPool queryPool,
+                         VkPipelineStageFlags2 stage, VkQueryPool queryPool,
                          uint32_t query)
 {
    panvk_stub();
