@@ -27,12 +27,14 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
 #include "util/compress.h"
 #include "util/crc32.h"
+#include "util/u_debug.h"
 #include "util/disk_cache.h"
 #include "util/disk_cache_os.h"
 
@@ -94,6 +96,7 @@ disk_cache_get_function_identifier(void *ptr, struct mesa_sha1 *ctx)
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include "utime.h"
 
 #include "util/blob.h"
 #include "util/crc32.h"
@@ -134,6 +137,45 @@ mkdir_if_needed(const char *path)
    return -1;
 }
 
+/* Create a directory named 'path' if it does not already exist,
+ * including parent directories if required.
+ *
+ * Returns: 0 if path already exists as a directory or if created.
+ *         -1 in all other cases.
+ */
+static int
+mkdir_with_parents_if_needed(const char *path)
+{
+   char *p;
+   const char *end;
+
+   if (path[0] == '\0')
+      return -1;
+
+   p = strdup(path);
+   end = p + strlen(p) + 1; /* end points to the \0 terminator */
+   for (char *q = p; q != end; q++) {
+      if (*q == '/' || q == end - 1) {
+         if (q == p) {
+            /* Skip the first / of an absolute path. */
+            continue;
+         }
+
+         *q = '\0';
+
+         if (mkdir_if_needed(p) == -1) {
+            free(p);
+            return -1;
+         }
+
+         *q = '/';
+      }
+   }
+   free(p);
+
+   return 0;
+}
+
 /* Concatenate an existing path and a new name to form a new path.  If the new
  * path does not exist as a directory, create it then return the resulting
  * name of the new path (ralloc'ed off of 'ctx').
@@ -456,7 +498,7 @@ disk_cache_evict_lru_item(struct disk_cache *cache)
    free(dir_path);
 
    if (size) {
-      p_atomic_add(cache->size, - (uint64_t)size);
+      p_atomic_add(&cache->size->value, - (uint64_t)size);
       return;
    }
 
@@ -483,7 +525,7 @@ disk_cache_evict_lru_item(struct disk_cache *cache)
    free_lru_file_list(lru_file_list);
 
    if (size)
-      p_atomic_add(cache->size, - (uint64_t)size);
+      p_atomic_add(&cache->size->value, - (uint64_t)size);
 }
 
 void
@@ -499,7 +541,7 @@ disk_cache_evict_item(struct disk_cache *cache, char *filename)
    free(filename);
 
    if (sb.st_blocks)
-      p_atomic_add(cache->size, - (uint64_t)sb.st_blocks * 512);
+      p_atomic_add(&cache->size->value, - (uint64_t)sb.st_blocks * 512);
 }
 
 static void *
@@ -819,7 +861,7 @@ disk_cache_write_item_to_disk(struct disk_cache_put_job *dc_job,
       goto done;
    }
 
-   p_atomic_add(dc_job->cache->size, sb.st_blocks * 512);
+   p_atomic_add(&dc_job->cache->size->value, sb.st_blocks * 512);
 
  done:
    if (fd_final != -1)
@@ -837,6 +879,7 @@ disk_cache_write_item_to_disk(struct disk_cache_put_job *dc_job,
  *
  *   $MESA_SHADER_CACHE_DIR
  *   $XDG_CACHE_HOME/mesa_shader_cache
+ *   $HOME/.cache/mesa_shader_cache
  *   <pwd.pw_dir>/.cache/mesa_shader_cache
  */
 char *
@@ -850,10 +893,10 @@ disk_cache_generate_cache_dir(void *mem_ctx, const char *gpu_name,
    else if (cache_type == DISK_CACHE_DATABASE)
       cache_dir_name = CACHE_DIR_NAME_DB;
 
-   char *path = getenv("MESA_SHADER_CACHE_DIR");
+   char *path = secure_getenv("MESA_SHADER_CACHE_DIR");
 
    if (!path) {
-      path = getenv("MESA_GLSL_CACHE_DIR");
+      path = secure_getenv("MESA_GLSL_CACHE_DIR");
       if (path)
          fprintf(stderr,
                  "*** MESA_GLSL_CACHE_DIR is deprecated; "
@@ -861,7 +904,7 @@ disk_cache_generate_cache_dir(void *mem_ctx, const char *gpu_name,
    }
 
    if (path) {
-      if (mkdir_if_needed(path) == -1)
+      if (mkdir_with_parents_if_needed(path) == -1)
          return NULL;
 
       path = concatenate_and_mkdir(mem_ctx, path, cache_dir_name);
@@ -870,7 +913,7 @@ disk_cache_generate_cache_dir(void *mem_ctx, const char *gpu_name,
    }
 
    if (path == NULL) {
-      char *xdg_cache_home = getenv("XDG_CACHE_HOME");
+      char *xdg_cache_home = secure_getenv("XDG_CACHE_HOME");
 
       if (xdg_cache_home) {
          if (mkdir_if_needed(xdg_cache_home) == -1)
@@ -882,6 +925,20 @@ disk_cache_generate_cache_dir(void *mem_ctx, const char *gpu_name,
       }
    }
 
+   if (!path) {
+      char *home = getenv("HOME");
+
+      if (home) {
+         path = concatenate_and_mkdir(mem_ctx, home, ".cache");
+         if (!path)
+            return NULL;
+
+         path = concatenate_and_mkdir(mem_ctx, path, cache_dir_name);
+         if (!path)
+            return NULL;
+      }
+   }
+
    if (!path) {
       char *buf;
       size_t buf_size;
@@ -933,8 +990,14 @@ disk_cache_generate_cache_dir(void *mem_ctx, const char *gpu_name,
 bool
 disk_cache_enabled()
 {
+   /* Disk cache is not enabled for android, but android's EGL layer
+    * uses EGL_ANDROID_blob_cache to manage the cache itself:
+    */
+   if (DETECT_OS_ANDROID)
+      return false;
+
    /* If running as a users other than the real user disable cache */
-   if (geteuid() != getuid())
+   if (!__normal_user())
       return false;
 
    /* At user request, disable shader cache entirely. */
@@ -997,6 +1060,29 @@ disk_cache_load_cache_index_foz(void *mem_ctx, struct disk_cache *cache)
    return foz_prepare(&cache->foz_db, cache->path);
 }
 
+
+void
+disk_cache_touch_cache_user_marker(char *path)
+{
+   char *marker_path = NULL;
+   asprintf(&marker_path, "%s/marker", path);
+   if (!marker_path)
+      return;
+
+   time_t now = time(NULL);
+
+   struct stat attr;
+   if (stat(marker_path, &attr) == -1) {
+      int fd = open(marker_path, O_WRONLY | O_CREAT | O_CLOEXEC, 0644);
+      if (fd != -1) {
+         close(fd);
+      }
+   } else if (now - attr.st_mtime > 60 * 60 * 24 /* One day */) {
+      (void)utime(marker_path, NULL);
+   }
+   free(marker_path);
+}
+
 bool
 disk_cache_mmap_cache_index(void *mem_ctx, struct disk_cache *cache,
                             char *path)
@@ -1056,7 +1142,7 @@ disk_cache_mmap_cache_index(void *mem_ctx, struct disk_cache *cache,
       goto path_fail;
    cache->index_mmap_size = size;
 
-   cache->size = (uint64_t *) cache->index_mmap;
+   cache->size = (p_atomic_uint64_t *) cache->index_mmap;
    cache->stored_keys = cache->index_mmap + sizeof(uint64_t);
    mapped = true;
 
