@@ -31,15 +31,10 @@
 
 
 #include "util/format/u_formats.h"
-#include "pipe/p_defines.h"
 #include "util/u_debug.h"
 
 #include "c99_compat.h"
 
-union pipe_color_union;
-struct pipe_screen;
-
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -470,6 +465,25 @@ util_format_short_name(enum pipe_format format)
    return desc->short_name;
 }
 
+static inline const char *
+util_chroma_format_name(enum pipe_video_chroma_format chroma_format)
+{
+   switch (chroma_format) {
+   case PIPE_VIDEO_CHROMA_FORMAT_400:
+      return "PIPE_VIDEO_CHROMA_FORMAT_400";
+   case PIPE_VIDEO_CHROMA_FORMAT_420:
+      return "PIPE_VIDEO_CHROMA_FORMAT_420";
+   case PIPE_VIDEO_CHROMA_FORMAT_422:
+      return "PIPE_VIDEO_CHROMA_FORMAT_422";
+   case PIPE_VIDEO_CHROMA_FORMAT_444:
+      return "PIPE_VIDEO_CHROMA_FORMAT_444";
+   case PIPE_VIDEO_CHROMA_FORMAT_NONE:
+      return "PIPE_VIDEO_CHROMA_FORMAT_NONE";
+   default:
+      return "PIPE_VIDEO_CHROMA_FORMAT_???";
+   }
+}
+
 /**
  * Whether this format is plain, see UTIL_FORMAT_LAYOUT_PLAIN for more info.
  */
@@ -540,6 +554,12 @@ static inline bool
 util_format_is_srgb(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
+
+   assert(desc);
+   if (!desc) {
+      return false;
+   }
+
    return desc->colorspace == UTIL_FORMAT_COLORSPACE_SRGB;
 }
 
@@ -737,6 +757,11 @@ util_format_is_alpha(enum pipe_format format) ATTRIBUTE_CONST;
 bool
 util_format_is_luminance_alpha(enum pipe_format format) ATTRIBUTE_CONST;
 
+bool
+util_format_is_red_alpha(enum pipe_format format) ATTRIBUTE_CONST;
+
+bool
+util_format_is_red_green(enum pipe_format format) ATTRIBUTE_CONST;
 
 bool
 util_format_is_intensity(enum pipe_format format) ATTRIBUTE_CONST;
@@ -981,6 +1006,49 @@ util_format_get_component_bits(enum pipe_format format,
    }
 }
 
+static inline unsigned
+util_format_get_component_shift(enum pipe_format format,
+                                enum util_format_colorspace colorspace,
+                                unsigned component)
+{
+   const struct util_format_description *desc = util_format_description(format);
+   enum util_format_colorspace desc_colorspace;
+
+   assert(format);
+   if (!format) {
+      return 0;
+   }
+
+   assert(component < 4);
+
+   /* Treat RGB and SRGB as equivalent. */
+   if (colorspace == UTIL_FORMAT_COLORSPACE_SRGB) {
+      colorspace = UTIL_FORMAT_COLORSPACE_RGB;
+   }
+   if (desc->colorspace == UTIL_FORMAT_COLORSPACE_SRGB) {
+      desc_colorspace = UTIL_FORMAT_COLORSPACE_RGB;
+   } else {
+      desc_colorspace = desc->colorspace;
+   }
+
+   if (desc_colorspace != colorspace) {
+      return 0;
+   }
+
+   switch (desc->swizzle[component]) {
+   case PIPE_SWIZZLE_X:
+      return desc->channel[0].shift;
+   case PIPE_SWIZZLE_Y:
+      return desc->channel[1].shift;
+   case PIPE_SWIZZLE_Z:
+      return desc->channel[2].shift;
+   case PIPE_SWIZZLE_W:
+      return desc->channel[3].shift;
+   default:
+      return 0;
+   }
+}
+
 /**
  * Given a linear RGB colorspace format, return the corresponding SRGB
  * format, or PIPE_FORMAT_NONE if none.
@@ -1494,12 +1562,9 @@ util_format_get_first_non_void_channel(enum pipe_format format)
 
    for (i = 0; i < 4; i++)
       if (desc->channel[i].type != UTIL_FORMAT_TYPE_VOID)
-         break;
+         return i;
 
-   if (i == 4)
-       return -1;
-
-   return i;
+   return -1;
 }
 
 /**
@@ -1738,6 +1803,10 @@ util_format_get_array(const enum util_format_type type, const unsigned bits,
                       const unsigned nr_components, const bool normalized,
                       const bool pure_integer);
 
+unsigned util_format_get_last_component(enum pipe_format format);
+int util_format_get_largest_non_void_channel(enum pipe_format format);
+unsigned util_format_get_max_channel_size(enum pipe_format format);
+
 #ifdef __cplusplus
 } // extern "C" {
 #endif
