@@ -16,6 +16,7 @@
 #include <unistd.h>
 
 #include "util/os_file.h"
+#include "util/os_misc.h"
 #include "util/sparse_array.h"
 #include "util/u_process.h"
 #define VIRGL_RENDERER_UNSTABLE_APIS
@@ -535,15 +536,13 @@ vtest_vcmd_submit_cmd2(struct vtest *vtest,
    for (uint32_t i = 0; i < submit->batch_count; i++) {
       const struct vn_renderer_submit_batch *batch = &submit->batches[i];
       struct vcmd_submit_cmd2_batch dst = {
+         .flags = VCMD_SUBMIT_CMD2_FLAG_RING_IDX,
          .cmd_offset = cs_offset / sizeof(uint32_t),
          .cmd_size = batch->cs_size / sizeof(uint32_t),
          .sync_offset = sync_offset / sizeof(uint32_t),
          .sync_count = batch->sync_count,
+         .ring_idx = batch->ring_idx,
       };
-      if (vtest->base.info.supports_multiple_timelines) {
-         dst.flags = VCMD_SUBMIT_CMD2_FLAG_RING_IDX;
-         dst.ring_idx = batch->ring_idx;
-      }
       vtest_write(vtest, &dst, sizeof(dst));
 
       cs_offset += batch->cs_size;
@@ -937,7 +936,6 @@ vtest_init_renderer_info(struct vtest *vtest)
    info->pci.device_id = VTEST_PCI_DEVICE_ID;
 
    info->has_dma_buf_import = false;
-   info->has_cache_management = false;
    info->has_external_sync = false;
    info->has_implicit_fencing = false;
 
@@ -948,7 +946,7 @@ vtest_init_renderer_info(struct vtest *vtest)
       capset->vk_ext_command_serialization_spec_version;
    info->vk_mesa_venus_protocol_spec_version =
       capset->vk_mesa_venus_protocol_spec_version;
-   info->supports_blob_id_0 = capset->supports_blob_id_0;
+   assert(capset->supports_blob_id_0);
 
    /* ensure vk_extension_mask is large enough to hold all capset masks */
    STATIC_ASSERT(sizeof(info->vk_extension_mask) >=
@@ -956,9 +954,9 @@ vtest_init_renderer_info(struct vtest *vtest)
    memcpy(info->vk_extension_mask, capset->vk_extension_mask1,
           sizeof(capset->vk_extension_mask1));
 
-   info->allow_vk_wait_syncs = capset->allow_vk_wait_syncs;
+   assert(capset->allow_vk_wait_syncs);
 
-   info->supports_multiple_timelines = capset->supports_multiple_timelines;
+   assert(capset->supports_multiple_timelines);
    info->max_timeline_count = vtest->max_timeline_count;
 }
 
@@ -1032,13 +1030,15 @@ vtest_init_protocol_version(struct vtest *vtest)
 static VkResult
 vtest_init(struct vtest *vtest)
 {
+   const char *socket_name = os_get_option("VTEST_SOCKET_NAME");
+
    util_sparse_array_init(&vtest->shmem_array, sizeof(struct vtest_shmem),
                           1024);
    util_sparse_array_init(&vtest->bo_array, sizeof(struct vtest_bo), 1024);
 
    mtx_init(&vtest->sock_mutex, mtx_plain);
-   vtest->sock_fd =
-      vtest_connect_socket(vtest->instance, VTEST_DEFAULT_SOCKET_NAME);
+   vtest->sock_fd = vtest_connect_socket(
+      vtest->instance, socket_name ? socket_name : VTEST_DEFAULT_SOCKET_NAME);
    if (vtest->sock_fd < 0)
       return VK_ERROR_INITIALIZATION_FAILED;
 
@@ -1056,9 +1056,8 @@ vtest_init(struct vtest *vtest)
       return result;
 
    /* see virtgpu_init_shmem_blob_mem */
-   vtest->shmem_blob_mem = vtest->capset.data.supports_blob_id_0
-                              ? VCMD_BLOB_TYPE_HOST3D
-                              : VCMD_BLOB_TYPE_GUEST;
+   assert(vtest->capset.data.supports_blob_id_0);
+   vtest->shmem_blob_mem = VCMD_BLOB_TYPE_HOST3D;
 
    vn_renderer_shmem_cache_init(&vtest->shmem_cache, &vtest->base,
                                 vtest_shmem_destroy_now);
